@@ -10,7 +10,7 @@
  *
  * This module is loaded lazily by devtools-clhandler.js, once the first
  * browser window is ready (i.e. fired browser-delayed-startup-finished event)
- **/
+ */
 
 const lazy = {};
 ChromeUtils.defineESModuleGetters(lazy, {
@@ -244,13 +244,13 @@ var gDevToolsBrowser = (exports.gDevToolsBrowser = {
    *
    * @param  {Window} window
    *         The top level browser window from which the key shortcut is pressed.
-   * @param  {Object} key
+   * @param  {object} key
    *         Key object describing the key shortcut being pressed. It comes
    *         from devtools-startup.js's KeyShortcuts array. The useful fields here
    *         are:
    *         - `toolId` used to identify a toolbox's panel like inspector or webconsole,
    *         - `id` used to identify any other key shortcuts like about:debugging
-   * @param {Number} startTime
+   * @param {number} startTime
    *        Optional, indicates the time at which the key event fired. This is a
    *        `ChromeUtils.now()` timing.
    */
@@ -282,24 +282,26 @@ var gDevToolsBrowser = (exports.gDevToolsBrowser = {
       case "browserToolbox":
         lazy.BrowserToolboxLauncher.init();
         break;
-      case "browserConsole":
+      case "browserConsole": {
         const {
           BrowserConsoleManager,
         } = require("resource://devtools/client/webconsole/browser-console-manager.js");
         BrowserConsoleManager.openBrowserConsoleOrFocus();
         break;
+      }
       case "responsiveDesignMode":
         ResponsiveUIManager.toggle(window, window.gBrowser.selectedTab, {
           trigger: "shortcut",
         });
         break;
-      case "javascriptTracingToggle":
+      case "javascriptTracingToggle": {
         const toolbox = gDevTools.getToolboxForTab(window.gBrowser.selectedTab);
         if (!toolbox) {
           break;
         }
         await toolbox.commands.tracerCommand.toggle();
         break;
+      }
     }
   },
 
@@ -488,7 +490,6 @@ var gDevToolsBrowser = (exports.gDevToolsBrowser = {
    *        The chrome window containing about:devtools-toolbox. Will match
    *        toolbox.topWindow.
    * @return {Toolbox} The toolbox instance loaded in about:devtools-toolbox
-   *
    */
   _getAboutDevtoolsToolbox(win) {
     if (!gDevToolsBrowser._isAboutDevtoolsToolbox(win)) {
