@@ -14,11 +14,11 @@ const {
  * For inline stylesheets, return a string indicating that.
  * For URLs, return just the stylesheet filename.
  *
- * @param {Object} source
+ * @param {object} source
  *        Information about the style source. Contains:
- *        - type: {String} One of "element" or "stylesheet"
+ *        - type: {string} One of "element" or "stylesheet"
  *        - href: {String|null} Stylesheet URL or document URL for elmeent inline styles
- * @return {String}
+ * @return {string}
  */
 function getSourceForDisplay(source) {
   let href;
@@ -30,10 +30,11 @@ function getSourceForDisplay(source) {
     case "inline":
       href = getStr("changes.inlineStyleSheetLabel2");
       break;
-    case "stylesheet":
+    case "stylesheet": {
       const url = new URL(source.href);
       href = url.pathname.substring(url.pathname.lastIndexOf("/") + 1);
       break;
+    }
   }
 
   return href;
