@@ -900,55 +900,3 @@ async function checkBackgroundColorMatchedSelectors(
     content.document.getElementById(`style-${id}`)?.remove();
   });
 }
-
-async function checkMatchedSelectorForProperty(
-  view,
-  { property, expectedComputedValue, expectedMatchedSelectors }
-) {
-  const propertyView = getPropertyView(view, property);
-  ok(propertyView, `found PropertyView for "${property}"`);
-  const { valueNode } = propertyView;
-  is(
-    valueNode.textContent,
-    expectedComputedValue,
-    `Expected displayed computed value for "${property}"`
-  );
-
-  is(propertyView.hasMatchedSelectors, true, "hasMatchedSelectors is true");
-
-  info("Expanding the matched selectors");
-  propertyView.matchedExpanded = true;
-  await propertyView.refreshMatchedSelectors();
-
-  const selectorsEl =
-    propertyView.matchedSelectorsContainer.querySelectorAll(".rule-text");
-  is(
-    selectorsEl.length,
-    expectedMatchedSelectors.length,
-    "Expected number of selectors are displayed"
-  );
-
-  selectorsEl.forEach((selectorEl, index) => {
-    is(
-      selectorEl.querySelector(".fix-get-selection").innerText,
-      expectedMatchedSelectors[index].selector,
-      `Selector #${index} is the expected one`
-    );
-    is(
-      selectorEl.querySelector(".computed-other-property-value").innerText,
-      expectedMatchedSelectors[index].value,
-      `Selector #${index} ("${expectedMatchedSelectors[index].selector}") has the expected "${property}"`
-    );
-    const classToMatch = index === 0 ? "bestmatch" : "matched";
-    const expectedMatch = expectedMatchedSelectors[index].match ?? true;
-    is(
-      selectorEl.classList.contains(classToMatch),
-      expectedMatch,
-      `Selector #${index} ("${expectedMatchedSelectors[index].selector}") element does ${expectedMatch ? "" : "not "}have a matching class`
-    );
-  });
-}
-
-function getPropertyView(computedView, name) {
-  return computedView.propertyViews.find(view => view.name === name);
-}
