@@ -28,4 +28,16 @@ add_task(async function () {
   const prop = textProps[textProps.length - 1];
   is(prop.name, "font-weight", "The last property name is font-weight");
   is(prop.value, "bold", "The last property value is bold");
+
+  info(
+    "Add another rule to make sure we reuse the stylesheet we created the first time we added a rule"
+  );
+  await addNewRuleAndDismissEditor(inspector, view, "#testid", 1);
+
+  const styleSheetsCount = await SpecialPowers.spawn(
+    gBrowser.selectedBrowser,
+    [],
+    () => content.document.styleSheets.length
+  );
+  is(styleSheetsCount, 1, "Only one stylesheet was created in the document");
 });
