@@ -148,7 +148,7 @@ const {
  *
  * @param  {nsIAccessible}  accessible
  *         object to be tested.
- * @return {Boolean}
+ * @return {boolean}
  *         True if accessible object is stale, false otherwise.
  */
 function isStale(accessible) {
@@ -162,14 +162,14 @@ function isStale(accessible) {
 /**
  * Get accessibility audit starting with the passed accessible object as a root.
  *
- * @param {Object} acc
+ * @param {object} acc
  *        AccessibileActor to be used as the root for the audit.
- * @param {Object} options
+ * @param {object} options
  *        Options for running audit, may include:
  *        - types: Array of audit types to be performed during audit.
  * @param {Map} report
  *        An accumulator map to be used to store audit information.
- * @param {Object} progress
+ * @param {object} progress
  *        An audit project object that is used to track the progress of the
  *        audit and send progress "audit-event" events to the client.
  */
@@ -473,7 +473,8 @@ class AccessibleWalkerActor extends Actor {
 
   /**
    * Get an accessible actor for a domnode actor.
-   * @param  {Object} domNode
+   *
+   * @param  {object} domNode
    *         domnode actor for which accessible actor is being created.
    * @return {Promse}
    *         A promise that resolves when accessible actor is created for a
@@ -496,6 +497,7 @@ class AccessibleWalkerActor extends Actor {
 
   /**
    * Get a raw accessible object for a raw node.
+   *
    * @param  {DOMNode} rawNode
    *         Raw node for which accessible object is being retrieved.
    * @return {nsIAccessible}
@@ -544,7 +546,7 @@ class AccessibleWalkerActor extends Actor {
    * Run accessibility audit and return relevant ancestries for AccessibleActors
    * that have non-empty audit checks.
    *
-   * @param  {Object} options
+   * @param  {object} options
    *         Options for running audit, may include:
    *         - types: Array of audit types to be performed during audit.
    *
@@ -588,7 +590,7 @@ class AccessibleWalkerActor extends Actor {
    * report. Instead, an "audit-event" event will be fired when the audit is
    * completed or fails.
    *
-   * @param {Object} options
+   * @param {object} options
    *        Options for running audit, may include:
    *        - types: Array of audit types to be performed during audit.
    */
@@ -639,12 +641,12 @@ class AccessibleWalkerActor extends Actor {
         this.clearRefs();
         // If it's a top level document notify listeners about the document
         // being ready.
-        events.emit(this, "document-ready", rawAccessible);
+        this.emit("document-ready", rawAccessible);
       }
     }
 
     switch (event.eventType) {
-      case EVENT_STATE_CHANGE:
+      case EVENT_STATE_CHANGE: {
         const { state, isEnabled } = event.QueryInterface(
           Ci.nsIAccessibleStateChangeEvent
         );
@@ -658,14 +660,14 @@ class AccessibleWalkerActor extends Actor {
             }
             return;
           }
-          events.emit(accessible, "states-change", accessible.states);
+          accessible.emit("states-change", accessible.states);
         }
 
         break;
+      }
       case EVENT_NAME_CHANGE:
         if (accessible) {
-          events.emit(
-            accessible,
+          accessible.emit(
             "name-change",
             rawAccessible.name,
             event.DOMNode == this.rootDoc
@@ -676,16 +678,12 @@ class AccessibleWalkerActor extends Actor {
         break;
       case EVENT_VALUE_CHANGE:
         if (accessible) {
-          events.emit(accessible, "value-change", rawAccessible.value);
+          accessible.emit("value-change", rawAccessible.value);
         }
         break;
       case EVENT_DESCRIPTION_CHANGE:
         if (accessible) {
-          events.emit(
-            accessible,
-            "description-change",
-            rawAccessible.description
-          );
+          accessible.emit("description-change", rawAccessible.description);
         }
         break;
       case EVENT_REORDER:
@@ -693,9 +691,9 @@ class AccessibleWalkerActor extends Actor {
           accessible
             .children()
             .forEach(child =>
-              events.emit(child, "index-in-parent-change", child.indexInParent)
+              child.emit("index-in-parent-change", child.indexInParent)
             );
-          events.emit(accessible, "reorder", rawAccessible.childCount);
+          accessible.emit("reorder", rawAccessible.childCount);
         }
         break;
       case EVENT_HIDE:
@@ -708,17 +706,16 @@ class AccessibleWalkerActor extends Actor {
       case EVENT_DEFACTION_CHANGE:
       case EVENT_ACTION_CHANGE:
         if (accessible) {
-          events.emit(accessible, "actions-change", accessible.actions);
+          accessible.emit("actions-change", accessible.actions);
         }
         break;
       case EVENT_TEXT_CHANGED:
       case EVENT_TEXT_INSERTED:
       case EVENT_TEXT_REMOVED:
         if (accessible) {
-          events.emit(accessible, "text-change");
+          accessible.emit("text-change");
           if (NAME_FROM_SUBTREE_RULE_ROLES.has(rawAccessible.role)) {
-            events.emit(
-              accessible,
+            accessible.emit(
               "name-change",
               rawAccessible.name,
               event.DOMNode == this.rootDoc
@@ -732,17 +729,13 @@ class AccessibleWalkerActor extends Actor {
       case EVENT_OBJECT_ATTRIBUTE_CHANGED:
       case EVENT_TEXT_ATTRIBUTE_CHANGED:
         if (accessible) {
-          events.emit(accessible, "attributes-change", accessible.attributes);
+          accessible.emit("attributes-change", accessible.attributes);
         }
         break;
       // EVENT_ACCELERATOR_CHANGE is currently not fired by gecko accessibility.
       case EVENT_ACCELERATOR_CHANGE:
         if (accessible) {
-          events.emit(
-            accessible,
-            "shortcut-change",
-            accessible.keyboardShortcut
-          );
+          accessible.emit("shortcut-change", accessible.keyboardShortcut);
         }
         break;
       default:
@@ -755,7 +748,8 @@ class AccessibleWalkerActor extends Actor {
    * (CSS, overlays) by load accessibility highlighter style sheet used for
    * preventing transitions and applying transparency when calculating colour
    * contrast as well as temporarily hiding accessible highlighter overlay.
-   * @param  {Object} win
+   *
+   * @param  {object} win
    *         Window where highlighting happens.
    */
   async clearStyles(win) {
@@ -779,7 +773,8 @@ class AccessibleWalkerActor extends Actor {
    * accessible object by unloading accessibility highlighter style sheet used
    * for preventing transitions and applying transparency when calculating
    * colour contrast and potentially restoring accessible highlighter overlay.
-   * @param  {Object} win
+   *
+   * @param  {object} win
    *         Window where highlighting was happenning.
    */
   async restoreStyles(win) {
@@ -822,13 +817,13 @@ class AccessibleWalkerActor extends Actor {
    * Public method used to show an accessible object highlighter on the client
    * side.
    *
-   * @param  {Object} accessible
+   * @param  {object} accessible
    *         AccessibleActor to be highlighted.
-   * @param  {Object} options
+   * @param  {object} options
    *         Object used for passing options. Available options:
    *         - duration {Number}
    *                    Duration of time that the highlighter should be shown.
-   * @return {Boolean}
+   * @return {boolean}
    *         True if highlighter shows the accessible object.
    */
   async highlightAccessible(accessible, options = {}) {
@@ -895,6 +890,7 @@ class AccessibleWalkerActor extends Actor {
 
   /**
    * Check if the DOM event received when picking shold be ignored.
+   *
    * @param {Event} event
    */
   _ignoreEventWhenPicking(event) {
@@ -941,7 +937,7 @@ class AccessibleWalkerActor extends Actor {
   /**
    * Click event handler for when picking is enabled.
    *
-   * @param  {Object} event
+   * @param  {object} event
    *         Current click event.
    */
   onPick(event) {
@@ -960,7 +956,7 @@ class AccessibleWalkerActor extends Actor {
       if (!this._currentAccessible) {
         this._currentAccessible = this._findAndAttachAccessible(event);
       }
-      events.emit(this, "picker-accessible-previewed", this._currentAccessible);
+      this.emit("picker-accessible-previewed", this._currentAccessible);
       return;
     }
 
@@ -969,13 +965,13 @@ class AccessibleWalkerActor extends Actor {
     if (!this._currentAccessible) {
       this._currentAccessible = this._findAndAttachAccessible(event);
     }
-    events.emit(this, "picker-accessible-picked", this._currentAccessible);
+    this.emit("picker-accessible-picked", this._currentAccessible);
   }
 
   /**
    * Hover event handler for when picking is enabled.
    *
-   * @param  {Object} event
+   * @param  {object} event
    *         Current hover event.
    */
   async onHovered(event) {
@@ -999,14 +995,14 @@ class AccessibleWalkerActor extends Actor {
     // the most current accessible again.
     const shown = await this.highlightAccessible(accessible);
     if (this._isPicking && shown && accessible === this._currentAccessible) {
-      events.emit(this, "picker-accessible-hovered", accessible);
+      this.emit("picker-accessible-hovered", accessible);
     }
   }
 
   /**
    * Keyboard event handler for when picking is enabled.
    *
-   * @param  {Object} event
+   * @param  {object} event
    *         Current keyboard event.
    */
   onKey(event) {
@@ -1032,7 +1028,7 @@ class AccessibleWalkerActor extends Actor {
       // Cancel pick mode.
       case event.DOM_VK_ESCAPE:
         this.cancelPick();
-        events.emit(this, "picker-accessible-canceled");
+        this.emit("picker-accessible-canceled");
         break;
       case event.DOM_VK_C:
         if (
@@ -1040,7 +1036,7 @@ class AccessibleWalkerActor extends Actor {
           (!IS_OSX && event.ctrlKey && event.shiftKey)
         ) {
           this.cancelPick();
-          events.emit(this, "picker-accessible-canceled");
+          this.emit("picker-accessible-canceled");
         }
         break;
       default:
@@ -1097,9 +1093,9 @@ class AccessibleWalkerActor extends Actor {
    * Find deepest accessible object that corresponds to the screen coordinates of the
    * mouse pointer and attach it to the AccessibilityWalker tree.
    *
-   * @param  {Object} event
+   * @param  {object} event
    *         Correspoinding content event.
-   * @return {null|Object}
+   * @return {null | object}
    *         Accessible object, if available, that corresponds to a DOM node.
    */
   _findAndAttachAccessible(event) {
@@ -1220,7 +1216,7 @@ class AccessibleWalkerActor extends Actor {
   /**
    * Focusin event handler for when interacting with tabbing order overlay.
    *
-   * @param  {Object} event
+   * @param  {object} event
    *         Most recent focusin event.
    */
   async onFocusIn(event) {
@@ -1243,7 +1239,7 @@ class AccessibleWalkerActor extends Actor {
   /**
    * Focusout event handler for when interacting with tabbing order overlay.
    *
-   * @param  {Object} event
+   * @param  {object} event
    *         Most recent focusout event.
    */
   async onFocusOut(event) {
@@ -1272,10 +1268,10 @@ class AccessibleWalkerActor extends Actor {
   /**
    * Show tabbing order overlay for a given target.
    *
-   * @param  {Object} elm
+   * @param  {object} elm
    *         domnode actor to be used as the starting point for generating the
    *         tabbing order.
-   * @param  {Number} index
+   * @param  {number} index
    *         Starting index for the tabbing order.
    *
    * @return {JSON}
