@@ -8,17 +8,26 @@
 // It basically displays a magnifier that tracks mouse moves and shows a magnified version
 // of the page. On click, it samples the color at the pixel being hovered.
 
-const {Ci, Cc} = require("chrome");
-const {CanvasFrameAnonymousContentHelper, createNode} = require("./utils/markup");
-const Services = require("Services");
-const EventEmitter = require("devtools/shared/old-event-emitter");
-const {rgbToHsl, rgbToColorName} = require("devtools/shared/css/color").colorUtils;
-const {getCurrentZoom, getFrameOffsets} = require("devtools/shared/layout/utils");
+const {
+  CanvasFrameAnonymousContentHelper,
+} = require("resource://devtools/server/actors/highlighters/utils/markup.js");
+const EventEmitter = require("resource://devtools/shared/event-emitter.js");
+const { rgbToHsl } =
+  require("resource://devtools/shared/css/color.js").colorUtils;
+const {
+  getCurrentZoom,
+  getFrameOffsets,
+} = require("resource://devtools/shared/layout/utils.js");
+const { debounce } = require("resource://devtools/shared/debounce.js");
 
-loader.lazyGetter(this, "clipboardHelper",
-  () => Cc["@mozilla.org/widget/clipboardhelper;1"].getService(Ci.nsIClipboardHelper));
-loader.lazyGetter(this, "l10n",
-  () => Services.strings.createBundle("chrome://devtools-shared/locale/eyedropper.properties"));
+loader.lazyGetter(this, "clipboardHelper", () =>
+  Cc["@mozilla.org/widget/clipboardhelper;1"].getService(Ci.nsIClipboardHelper)
+);
+loader.lazyGetter(this, "l10n", () =>
+  Services.strings.createBundle(
+    "chrome://devtools-shared/locale/eyedropper.properties"
+  )
+);
 
 const ZOOM_LEVEL_PREF = "devtools.eyedropper.zoom";
 const FORMAT_PREF = "devtools.defaultColorUnit";
@@ -34,96 +43,111 @@ const DEFAULT_START_POS_Y = 100;
 const CLOSE_DELAY = 750;
 
 /**
- * The EyeDropper is the class that draws the gradient line and
- * color stops as an overlay on top of a linear-gradient background-image.
+ * The EyeDropper allows the user to select a color of a pixel within the content page,
+ * showing a magnified circle and color preview while the user hover the page.
  */
-function EyeDropper(highlighterEnv) {
-  EventEmitter.decorate(this);
+class EyeDropper {
+  #pageEventListenersAbortController;
+  #debouncedUpdateScreenshot;
+  constructor(highlighterEnv) {
+    EventEmitter.decorate(this);
 
-  this.highlighterEnv = highlighterEnv;
-  this.markup = new CanvasFrameAnonymousContentHelper(this.highlighterEnv,
-                                                      this._buildMarkup.bind(this));
+    this.highlighterEnv = highlighterEnv;
+    this.markup = new CanvasFrameAnonymousContentHelper(
+      this.highlighterEnv,
+      this._buildMarkup.bind(this),
+      {
+        contentRootHostClassName: "devtools-highlighter-eye-dropper",
+      }
+    );
+    this.isReady = this.markup.initialize();
 
-  // Get a couple of settings from prefs.
-  this.format = Services.prefs.getCharPref(FORMAT_PREF);
-  this.eyeDropperZoomLevel = Services.prefs.getIntPref(ZOOM_LEVEL_PREF);
-}
+    // Get a couple of settings from prefs.
+    this.format = Services.prefs.getCharPref(FORMAT_PREF);
+    this.eyeDropperZoomLevel = Services.prefs.getIntPref(ZOOM_LEVEL_PREF);
 
-EyeDropper.prototype = {
-  typeName: "EyeDropper",
-
-  ID_CLASS_PREFIX: "eye-dropper-",
+    this.#debouncedUpdateScreenshot = debounce(
+      this.updateScreenshot.bind(this),
+      200,
+      this
+    );
+  }
 
   get win() {
     return this.highlighterEnv.window;
-  },
+  }
 
   _buildMarkup() {
     // Highlighter main container.
-    let container = createNode(this.win, {
-      attributes: {"class": "highlighter-container"}
+    const container = this.markup.createNode({
+      attributes: { class: "highlighter-container" },
     });
 
     // Wrapper element.
-    let wrapper = createNode(this.win, {
+    const wrapper = this.markup.createNode({
       parent: container,
       attributes: {
-        "id": "root",
-        "class": "root",
-        "hidden": "true"
+        id: "eye-dropper-root",
+        class: "eye-dropper-root",
+        hidden: "true",
       },
-      prefix: this.ID_CLASS_PREFIX
     });
 
     // The magnifier canvas element.
-    createNode(this.win, {
+    this.markup.createNode({
       parent: wrapper,
       nodeType: "canvas",
       attributes: {
-        "id": "canvas",
-        "class": "canvas",
-        "width": MAGNIFIER_WIDTH,
-        "height": MAGNIFIER_HEIGHT
+        id: "eye-dropper-canvas",
+        class: "eye-dropper-canvas",
+        width: MAGNIFIER_WIDTH,
+        height: MAGNIFIER_HEIGHT,
       },
-      prefix: this.ID_CLASS_PREFIX
     });
 
     // The color label element.
-    let colorLabelContainer = createNode(this.win, {
+    const colorLabelContainer = this.markup.createNode({
       parent: wrapper,
-      attributes: {"class": "color-container"},
-      prefix: this.ID_CLASS_PREFIX
+      attributes: { class: "eye-dropper-color-container" },
     });
-    createNode(this.win, {
+    this.markup.createNode({
       nodeType: "div",
       parent: colorLabelContainer,
-      attributes: {"id": "color-preview", "class": "color-preview"},
-      prefix: this.ID_CLASS_PREFIX
+      attributes: {
+        id: "eye-dropper-color-preview",
+        class: "eye-dropper-color-preview",
+      },
     });
-    createNode(this.win, {
+    this.markup.createNode({
       nodeType: "div",
       parent: colorLabelContainer,
-      attributes: {"id": "color-value", "class": "color-value"},
-      prefix: this.ID_CLASS_PREFIX
+      attributes: {
+        id: "eye-dropper-color-value",
+        class: "eye-dropper-color-value",
+      },
     });
 
     return container;
-  },
+  }
 
   destroy() {
     this.hide();
     this.markup.destroy();
-  },
+  }
 
   getElement(id) {
-    return this.markup.getElement(this.ID_CLASS_PREFIX + id);
-  },
+    return this.markup.getElement(id);
+  }
 
   /**
    * Show the eye-dropper highlighter.
+   *
    * @param {DOMNode} node The node which document the highlighter should be inserted in.
-   * @param {Object} options The options object may contain the following properties:
-   * - {Boolean} copyOnSelect Whether selecting a color should copy it to the clipboard.
+   * @param {object} options The options object may contain the following properties:
+   * - {Boolean} copyOnSelect: Whether selecting a color should copy it to the clipboard.
+   * - {String|null} screenshot: a dataURL representation of the page screenshot. If null,
+   *                 the eyedropper will use `drawWindow` to get the the screenshot
+   *                 (⚠️ but it won't handle remote frames).
    */
   show(node, options = {}) {
     if (this.highlighterEnv.isXUL) {
@@ -135,81 +159,120 @@ EyeDropper.prototype = {
     // Get the page's current zoom level.
     this.pageZoom = getCurrentZoom(this.win);
 
-    // Take a screenshot of the viewport. This needs to be done first otherwise the
-    // eyedropper UI will appear in the screenshot itself (since the UI is injected as
-    // native anonymous content in the page).
+    // Take a screenshot of the viewport.
     // Once the screenshot is ready, the magnified area will be drawn.
-    this.prepareImageCapture();
+    this.updateScreenshot(options.screenshot);
 
     // Start listening for user events.
-    let {pageListenerTarget} = this.highlighterEnv;
-    pageListenerTarget.addEventListener("mousemove", this);
-    pageListenerTarget.addEventListener("click", this, true);
-    pageListenerTarget.addEventListener("keydown", this);
-    pageListenerTarget.addEventListener("DOMMouseScroll", this);
-    pageListenerTarget.addEventListener("FullZoomChange", this);
-
-    // Show the eye-dropper.
-    this.getElement("root").removeAttribute("hidden");
+    const { pageListenerTarget } = this.highlighterEnv;
+    this.#pageEventListenersAbortController = new AbortController();
+    const signal = this.#pageEventListenersAbortController.signal;
+    pageListenerTarget.addEventListener("mousemove", this, { signal });
+    pageListenerTarget.addEventListener("click", this, {
+      signal,
+      useCapture: true,
+    });
+    pageListenerTarget.addEventListener("keydown", this, { signal });
+    pageListenerTarget.addEventListener("DOMMouseScroll", this, { signal });
+    pageListenerTarget.addEventListener("FullZoomChange", this, { signal });
+    pageListenerTarget.addEventListener("resize", this, { signal });
 
     // Prepare the canvas context on which we're drawing the magnified page portion.
-    this.ctx = this.getElement("canvas").getCanvasContext();
+    this.ctx = this.getElement("eye-dropper-canvas").getCanvasContext();
     this.ctx.imageSmoothingEnabled = false;
 
-    this.magnifiedArea = {width: MAGNIFIER_WIDTH, height: MAGNIFIER_HEIGHT,
-                          x: DEFAULT_START_POS_X, y: DEFAULT_START_POS_Y};
+    this.magnifiedArea = {
+      width: MAGNIFIER_WIDTH,
+      height: MAGNIFIER_HEIGHT,
+      x: DEFAULT_START_POS_X,
+      y: DEFAULT_START_POS_Y,
+    };
 
     this.moveTo(DEFAULT_START_POS_X, DEFAULT_START_POS_Y);
 
     // Focus the content so the keyboard can be used.
     this.win.focus();
 
+    // Make sure we receive mouse events when the debugger has paused execution
+    // in the page.
+    this.win.document.setSuppressedEventListener(this);
+
     return true;
-  },
+  }
 
   /**
    * Hide the eye-dropper highlighter.
    */
   hide() {
-    if (this.highlighterEnv.isXUL) {
-      return;
-    }
-
     this.pageImage = null;
 
-    let {pageListenerTarget} = this.highlighterEnv;
+    if (this.#pageEventListenersAbortController) {
+      this.#pageEventListenersAbortController.abort();
+      this.#pageEventListenersAbortController = null;
 
-    if (pageListenerTarget) {
-      pageListenerTarget.removeEventListener("mousemove", this);
-      pageListenerTarget.removeEventListener("click", this, true);
-      pageListenerTarget.removeEventListener("keydown", this);
-      pageListenerTarget.removeEventListener("DOMMouseScroll", this);
-      pageListenerTarget.removeEventListener("FullZoomChange", this);
+      const rootElement = this.getElement("eye-dropper-root");
+      rootElement.setAttribute("hidden", "true");
+      rootElement.removeAttribute("drawn");
+
+      this.emit("hidden");
+
+      this.win.document.setSuppressedEventListener(null);
+    }
+  }
+
+  /**
+   * Convert a base64 png data-uri to raw binary data.
+   */
+  #dataURItoBlob(dataURI) {
+    const byteString = atob(dataURI.split(",")[1]);
+
+    // write the bytes of the string to an ArrayBuffer
+    const buffer = new ArrayBuffer(byteString.length);
+    // Update the buffer through a typed array.
+    const typedArray = new Uint8Array(buffer);
+    for (let i = 0; i < byteString.length; i++) {
+      typedArray[i] = byteString.charCodeAt(i);
     }
 
-    this.getElement("root").setAttribute("hidden", "true");
-    this.getElement("root").removeAttribute("drawn");
+    return new Blob([buffer], { type: "image/png" });
+  }
 
-    this.emit("hidden");
-  },
+  /**
+   * Create an image bitmap from the page screenshot, draw the eyedropper and set the
+   * "drawn" attribute on the "root" element once it's done.
+   *
+   * @params {string | null} screenshot: a dataURL representation of the page screenshot.
+   *                       If null, we'll use `drawWindow` to get the the page screenshot
+   *                       (⚠️ but it won't handle remote frames).
+   */
+  async updateScreenshot(screenshot) {
+    const rootElement = this.getElement("eye-dropper-root");
 
-  prepareImageCapture() {
-    // Get the image data from the content window.
-    let imageData = getWindowAsImageData(this.win);
+    let imageSource;
+    if (screenshot) {
+      imageSource = this.#dataURItoBlob(screenshot);
+    } else {
+      // Hide the eyedropper while we take the screenshot.
+      rootElement.setAttribute("hidden", "true");
+      imageSource = getWindowAsImageData(this.win);
+    }
 
-    // We need to transform imageData to something drawWindow will consume. An ImageBitmap
-    // works well. We could have used an Image, but doing so results in errors if the page
-    // defines CSP headers.
-    this.win.createImageBitmap(imageData).then(image => {
-      this.pageImage = image;
-      // We likely haven't drawn anything yet (no mousemove events yet), so start now.
-      this.draw();
+    // We need to transform the blob/imageData to something drawWindow will consume.
+    // An ImageBitmap works well. We could have used an Image, but doing so results
+    // in errors if the page defines CSP headers.
+    const image = await this.win.createImageBitmap(imageSource);
 
-      // Set an attribute on the root element to be able to run tests after the first draw
-      // was done.
-      this.getElement("root").setAttribute("drawn", "true");
-    });
-  },
+    this.pageImage = image;
+    // We likely haven't drawn anything yet (no mousemove events yet), so start now.
+    this.draw();
+
+    // Set an attribute on the root element to be able to run tests after the first draw
+    // was done.
+    rootElement.setAttribute("drawn", "true");
+
+    // Show the eyedropper.
+    rootElement.removeAttribute("hidden");
+  }
 
   /**
    * Get the number of cells (blown-up pixels) per direction in the grid.
@@ -217,34 +280,36 @@ EyeDropper.prototype = {
   get cellsWide() {
     // Canvas will render whole "pixels" (cells) only, and an even number at that. Round
     // up to the nearest even number of pixels.
-    let cellsWide = Math.ceil(this.magnifiedArea.width / this.eyeDropperZoomLevel);
+    let cellsWide = Math.ceil(
+      this.magnifiedArea.width / this.eyeDropperZoomLevel
+    );
     cellsWide += cellsWide % 2;
 
     return cellsWide;
-  },
+  }
 
   /**
    * Get the size of each cell (blown-up pixel) in the grid.
    */
   get cellSize() {
     return this.magnifiedArea.width / this.cellsWide;
-  },
+  }
 
   /**
    * Get index of cell in the center of the grid.
    */
   get centerCell() {
     return Math.floor(this.cellsWide / 2);
-  },
+  }
 
   /**
    * Get color of center cell in the grid.
    */
   get centerColor() {
-    let pos = (this.centerCell * this.cellSize) + (this.cellSize / 2);
-    let rgb = this.ctx.getImageData(pos, pos, 1, 1).data;
+    const pos = this.centerCell * this.cellSize + this.cellSize / 2;
+    const rgb = this.ctx.getImageData(pos, pos, 1, 1).data;
     return rgb;
-  },
+  }
 
   draw() {
     // If the image of the page isn't ready yet, bail out, we'll draw later on mousemove.
@@ -252,15 +317,15 @@ EyeDropper.prototype = {
       return;
     }
 
-    let {width, height, x, y} = this.magnifiedArea;
+    const { width, height, x, y } = this.magnifiedArea;
 
-    let zoomedWidth = width / this.eyeDropperZoomLevel;
-    let zoomedHeight = height / this.eyeDropperZoomLevel;
+    const zoomedWidth = width / this.eyeDropperZoomLevel;
+    const zoomedHeight = height / this.eyeDropperZoomLevel;
 
-    let sx = x - (zoomedWidth / 2);
-    let sy = y - (zoomedHeight / 2);
-    let sw = zoomedWidth;
-    let sh = zoomedHeight;
+    const sx = x - zoomedWidth / 2;
+    const sy = y - zoomedHeight / 2;
+    const sw = zoomedWidth;
+    const sh = zoomedHeight;
 
     this.ctx.drawImage(this.pageImage, sx, sy, sw, sh, 0, 0, width, height);
 
@@ -272,56 +337,65 @@ EyeDropper.prototype = {
     this.drawCrosshair();
 
     // Update the color preview and value.
-    let rgb = this.centerColor;
-    this.getElement("color-preview").setAttribute("style",
-      `background-color:${toColorString(rgb, "rgb")};`);
-    this.getElement("color-value").setTextContent(toColorString(rgb, this.format));
-  },
+    const rgb = this.centerColor;
+    this.getElement("eye-dropper-color-preview").setAttribute(
+      "style",
+      `background-color:${toColorString(rgb, "rgb")};`
+    );
+    this.getElement("eye-dropper-color-value").setTextContent(
+      toColorString(rgb, this.format)
+    );
+  }
 
   /**
    * Draw a grid on the canvas representing pixel boundaries.
    */
   drawGrid() {
-    let {width, height} = this.magnifiedArea;
+    const { width, height } = this.magnifiedArea;
 
     this.ctx.lineWidth = 1;
     this.ctx.strokeStyle = "rgba(143, 143, 143, 0.2)";
 
     for (let i = 0; i < width; i += this.cellSize) {
       this.ctx.beginPath();
-      this.ctx.moveTo(i - .5, 0);
-      this.ctx.lineTo(i - .5, height);
+      this.ctx.moveTo(i - 0.5, 0);
+      this.ctx.lineTo(i - 0.5, height);
       this.ctx.stroke();
 
       this.ctx.beginPath();
-      this.ctx.moveTo(0, i - .5);
-      this.ctx.lineTo(width, i - .5);
+      this.ctx.moveTo(0, i - 0.5);
+      this.ctx.lineTo(width, i - 0.5);
       this.ctx.stroke();
     }
-  },
+  }
 
   /**
    * Draw a box on the canvas to highlight the center cell.
    */
   drawCrosshair() {
-    let pos = this.centerCell * this.cellSize;
+    const pos = this.centerCell * this.cellSize;
 
     this.ctx.lineWidth = 1;
     this.ctx.lineJoin = "miter";
     this.ctx.strokeStyle = "rgba(0, 0, 0, 1)";
-    this.ctx.strokeRect(pos - 1.5, pos - 1.5, this.cellSize + 2, this.cellSize + 2);
+    this.ctx.strokeRect(
+      pos - 1.5,
+      pos - 1.5,
+      this.cellSize + 2,
+      this.cellSize + 2
+    );
 
     this.ctx.strokeStyle = "rgba(255, 255, 255, 1)";
     this.ctx.strokeRect(pos - 0.5, pos - 0.5, this.cellSize, this.cellSize);
-  },
+  }
 
   handleEvent(e) {
     switch (e.type) {
-      case "mousemove":
+      case "mousemove": {
         // We might be getting an event from a child frame, so account for the offset.
-        let [xOffset, yOffset] = getFrameOffsets(this.win, e.target);
-        let x = xOffset + e.pageX - this.win.scrollX;
-        let y = yOffset + e.pageY - this.win.scrollY;
+        const [xOffset, yOffset] = getFrameOffsets(this.win, e.target);
+        const x = xOffset + e.pageX - this.win.scrollX;
+        const y = yOffset + e.pageY - this.win.scrollY;
         // Update the zoom area.
         this.magnifiedArea.x = x * this.pageZoom;
         this.magnifiedArea.y = y * this.pageZoom;
@@ -330,7 +404,11 @@ EyeDropper.prototype = {
         // And move the eye-dropper's UI so it follows the mouse.
         this.moveTo(x, y);
         break;
+      }
+      // Note: when events are suppressed we will only get mousedown/mouseup and
+      // not any click events.
       case "click":
+      case "mouseup":
         this.selectColor();
         break;
       case "keydown":
@@ -347,11 +425,15 @@ EyeDropper.prototype = {
         this.hide();
         this.show();
         break;
+      case "resize":
+        this.getElement("eye-dropper-root").removeAttribute("drawn");
+        this.#debouncedUpdateScreenshot();
+        break;
     }
-  },
+  }
 
   moveTo(x, y) {
-    let root = this.getElement("root");
+    const root = this.getElement("eye-dropper-root");
     root.setAttribute("style", `top:${y}px;left:${x}px;`);
 
     // Move the label container to the top if the magnifier is close to the bottom edge.
@@ -370,7 +452,7 @@ EyeDropper.prototype = {
     } else if (x >= this.win.innerWidth - MAGNIFIER_WIDTH) {
       root.setAttribute("left", "");
     }
-  },
+  }
 
   /**
    * Select the current color that's being previewed. Depending on the current options,
@@ -384,7 +466,7 @@ EyeDropper.prototype = {
 
     this.emit("selected", toColorString(this.centerColor, this.format));
     onColorSelected.then(() => this.hide(), console.error);
-  },
+  }
 
   /**
    * Handler for the keydown event. Either select the color or move the panel in a
@@ -432,33 +514,43 @@ EyeDropper.prototype = {
     offsetX *= modifier;
 
     if (offsetX !== 0 || offsetY !== 0) {
-      this.magnifiedArea.x = cap(this.magnifiedArea.x + offsetX,
-                                 0, this.win.innerWidth * this.pageZoom);
-      this.magnifiedArea.y = cap(this.magnifiedArea.y + offsetY, 0,
-                                 this.win.innerHeight * this.pageZoom);
+      this.magnifiedArea.x = cap(
+        this.magnifiedArea.x + offsetX,
+        0,
+        this.win.innerWidth * this.pageZoom
+      );
+      this.magnifiedArea.y = cap(
+        this.magnifiedArea.y + offsetY,
+        0,
+        this.win.innerHeight * this.pageZoom
+      );
 
       this.draw();
 
-      this.moveTo(this.magnifiedArea.x / this.pageZoom,
-                  this.magnifiedArea.y / this.pageZoom);
+      this.moveTo(
+        this.magnifiedArea.x / this.pageZoom,
+        this.magnifiedArea.y / this.pageZoom
+      );
 
       e.preventDefault();
     }
-  },
+  }
 
   /**
    * Copy the currently inspected color to the clipboard.
+   *
    * @return {Promise} Resolves when the copy has been done (after a delay that is used to
    * let users know that something was copied).
    */
   copyColor() {
     // Copy to the clipboard.
-    let color = toColorString(this.centerColor, this.format);
+    const color = toColorString(this.centerColor, this.format);
     clipboardHelper.copyString(color);
 
     // Provide some feedback.
-    this.getElement("color-value").setTextContent(
-      "✓ " + l10n.GetStringFromName("colorValue.copied"));
+    this.getElement("eye-dropper-color-value").setTextContent(
+      "✓ " + l10n.GetStringFromName("colorValue.copied")
+    );
 
     // Hide the tool after a delay.
     clearTimeout(this._copyTimeout);
@@ -466,25 +558,29 @@ EyeDropper.prototype = {
       this._copyTimeout = setTimeout(resolve, CLOSE_DELAY);
     });
   }
-};
+}
 
 exports.EyeDropper = EyeDropper;
 
 /**
  * Draw the visible portion of the window on a canvas and get the resulting ImageData.
+ *
  * @param {Window} win
  * @return {ImageData} The image data for the window.
  */
 function getWindowAsImageData(win) {
-  let canvas = win.document.createElementNS("http://www.w3.org/1999/xhtml", "canvas");
-  let scale = getCurrentZoom(win);
-  let width = win.innerWidth;
-  let height = win.innerHeight;
+  const canvas = win.document.createElementNS(
+    "http://www.w3.org/1999/xhtml",
+    "canvas"
+  );
+  const scale = getCurrentZoom(win);
+  const width = win.innerWidth;
+  const height = win.innerHeight;
   canvas.width = width * scale;
   canvas.height = height * scale;
   canvas.mozOpaque = true;
 
-  let ctx = canvas.getContext("2d");
+  const ctx = canvas.getContext("2d");
 
   ctx.scale(scale, scale);
   ctx.drawWindow(win, win.scrollX, win.scrollY, width, height, "#fff");
@@ -494,24 +590,25 @@ function getWindowAsImageData(win) {
 
 /**
  * Get a formatted CSS color string from a color value.
- * @param {array} rgb Rgb values of a color to format.
+ *
+ * @param {Array} rgb Rgb values of a color to format.
  * @param {string} format Format of string. One of "hex", "rgb", "hsl", "name".
  * @return {string} Formatted color value, e.g. "#FFF" or "hsl(20, 10%, 10%)".
  */
 function toColorString(rgb, format) {
-  let [r, g, b] = rgb;
+  const [r, g, b] = rgb;
 
   switch (format) {
     case "hex":
       return hexString(rgb);
     case "rgb":
       return "rgb(" + r + ", " + g + ", " + b + ")";
-    case "hsl":
-      let [h, s, l] = rgbToHsl(rgb);
+    case "hsl": {
+      const [h, s, l] = rgbToHsl(rgb);
       return "hsl(" + h + ", " + s + "%, " + l + "%)";
+    }
     case "name":
-      let str = rgbToColorName(r, g, b) || hexString(rgb);
-      return str;
+      return InspectorUtils.rgbToColorName(r, g, b) || hexString(rgb);
     default:
       return hexString(rgb);
   }
@@ -519,11 +616,12 @@ function toColorString(rgb, format) {
 
 /**
  * Produce a hex-formatted color string from rgb values.
- * @param {array} rgb Rgb values of color to stringify.
+ *
+ * @param {Array} rgb Rgb values of color to stringify.
  * @return {string} Hex formatted string for color, e.g. "#FFEE00".
  */
 function hexString([r, g, b]) {
-  let val = (1 << 24) + (r << 16) + (g << 8) + (b << 0);
+  const val = (1 << 24) + (r << 16) + (g << 8) + (b << 0);
   return "#" + val.toString(16).substr(-6);
 }
 
