@@ -4,21 +4,23 @@
 
 "use strict";
 
-const Services = require("Services");
-const DevToolsUtils = require("devtools/shared/DevToolsUtils");
-
 const {
   apply,
   getNodeTransformationMatrix,
+  getWritingModeMatrix,
   identity,
   isIdentity,
   multiply,
-  reflectAboutY,
-  rotate,
   scale,
   translate,
-} = require("devtools/shared/layout/dom-matrix-2d");
-const { getViewportDimensions } = require("devtools/shared/layout/utils");
+} = require("resource://devtools/shared/layout/dom-matrix-2d.js");
+const {
+  getCurrentZoom,
+  getViewportDimensions,
+} = require("resource://devtools/shared/layout/utils.js");
+const {
+  getComputedStyle,
+} = require("resource://devtools/server/actors/highlighters/utils/markup.js");
 
 // A set of utility functions for highlighters that render their content to a <canvas>
 // element.
@@ -30,8 +32,8 @@ const { getViewportDimensions } = require("devtools/shared/layout/utils");
 //
 // This canvas size value is the safest we can use because most GPUs can handle it.
 // It's also far from the maximum canvas memory allocation limit (4096x4096x4 is
-// 67.108.864 bytes, where the limit is 500.000.000 bytes, see:
-// http://searchfox.org/mozilla-central/source/gfx/thebes/gfxPrefs.h#401).
+// 67.108.864 bytes, where the limit is 500.000.000 bytes, see
+// gfx_max_alloc_size in modules/libpref/init/StaticPrefList.yaml.
 //
 // Note:
 // Once bug 1232491 lands, we could try to refactor this code to use the values from
@@ -43,31 +45,51 @@ const CANVAS_SIZE = 4096;
 // The default color used for the canvas' font, fill and stroke colors.
 const DEFAULT_COLOR = "#9400FF";
 
-// Boolean pref to enable adjustment for writing mode and RTL content.
-DevToolsUtils.defineLazyGetter(this, "WRITING_MODE_ADJUST_ENABLED", () => {
-  return Services.prefs.getBoolPref("devtools.highlighter.writingModeAdjust");
-});
-
 /**
  * Draws a rect to the context given and applies a transformation matrix if passed.
  * The coordinates are the start and end points of the rectangle's diagonal.
  *
  * @param  {CanvasRenderingContext2D} ctx
  *         The 2D canvas context.
- * @param  {Number} x1
+ * @param  {number} x1
  *         The x-axis coordinate of the rectangle's diagonal start point.
- * @param  {Number} y1
+ * @param  {number} y1
  *         The y-axis coordinate of the rectangle's diagonal start point.
- * @param  {Number} x2
+ * @param  {number} x2
  *         The x-axis coordinate of the rectangle's diagonal end point.
- * @param  {Number} y2
+ * @param  {number} y2
  *         The y-axis coordinate of the rectangle's diagonal end point.
  * @param  {Array} [matrix=identity()]
  *         The transformation matrix to apply.
  */
 function clearRect(ctx, x1, y1, x2, y2, matrix = identity()) {
-  let p = getPointsFromDiagonal(x1, y1, x2, y2, matrix);
-  ctx.clearRect(p[0].x, p[0].y, p[1].x - p[0].x, p[3].y - p[0].y);
+  const p = getPointsFromDiagonal(x1, y1, x2, y2, matrix);
+
+  // We are creating a clipping path and want it removed after we clear it's
+  // contents so we need to save the context.
+  ctx.save();
+
+  // Create a path to be cleared.
+  ctx.beginPath();
+  ctx.moveTo(Math.round(p[0].x), Math.round(p[0].y));
+  ctx.lineTo(Math.round(p[1].x), Math.round(p[1].y));
+  ctx.lineTo(Math.round(p[2].x), Math.round(p[2].y));
+  ctx.lineTo(Math.round(p[3].x), Math.round(p[3].y));
+  ctx.closePath();
+
+  // Restrict future drawing to the inside of the path.
+  ctx.clip();
+
+  // Clear any transforms applied to the canvas so that clearRect() really does
+  // clear everything.
+  ctx.setTransform(1, 0, 0, 1, 0, 0);
+
+  // Clear the contents of our clipped path by attempting to clear the canvas.
+  ctx.clearRect(0, 0, CANVAS_SIZE, CANVAS_SIZE);
+
+  // Restore the context to the state it was before changing transforms and
+  // adding clipping paths.
+  ctx.restore();
 }
 
 /**
@@ -75,24 +97,34 @@ function clearRect(ctx, x1, y1, x2, y2, matrix = identity()) {
  *
  * @param  {CanvasRenderingContext2D} ctx
  *         The 2D canvas context.
- * @param  {Number} x
+ * @param  {number} x
  *         The x-axis origin of the rectangle.
- * @param  {Number} y
+ * @param  {number} y
  *         The y-axis origin of the rectangle.
- * @param  {Number} width
+ * @param  {number} width
  *         The width of the rectangle.
- * @param  {Number} height
+ * @param  {number} height
  *         The height of the rectangle.
- * @param  {Number} radius
+ * @param  {number} radius
  *         The radius of the rounding.
- * @param  {Number} margin
+ * @param  {number} margin
  *         The distance of the origin point from the pointer.
- * @param  {Number} arrowSize
+ * @param  {number} arrowSize
  *         The size of the arrow.
- * @param  {String} alignment
+ * @param  {string} alignment
  *         The alignment of the rectangle in relation to its position to the grid.
  */
-function drawBubbleRect(ctx, x, y, width, height, radius, margin, arrowSize, alignment) {
+function drawBubbleRect(
+  ctx,
+  x,
+  y,
+  width,
+  height,
+  radius,
+  margin,
+  arrowSize,
+  alignment
+) {
   let angle = 0;
 
   if (alignment === "bottom") {
@@ -105,8 +137,8 @@ function drawBubbleRect(ctx, x, y, width, height, radius, margin, arrowSize, ali
     angle = 270;
   }
 
-  let originX = x;
-  let originY = y;
+  const originX = x;
+  const originY = y;
 
   ctx.save();
   ctx.translate(originX, originY);
@@ -114,17 +146,25 @@ function drawBubbleRect(ctx, x, y, width, height, radius, margin, arrowSize, ali
   ctx.translate(-originX, -originY);
   ctx.translate(-width / 2, -height - arrowSize - margin);
 
+  // The contour of the bubble is drawn with a path. The canvas context will have taken
+  // care of transforming the coordinates before calling the function, so we just always
+  // draw with the arrow pointing down. The top edge has rounded corners too.
   ctx.beginPath();
+  // Start at the top/left corner (below the rounded corner).
   ctx.moveTo(x, y + radius);
-  ctx.lineTo(x, y + height - radius);
-  ctx.arcTo(x, y + height, x + radius, y + height, radius);
-  ctx.lineTo(x + width / 2 - arrowSize, y + height);
+  // Go down.
+  ctx.lineTo(x, y + height);
+  // Go down and the right, to draw the first half of the arrow tip.
   ctx.lineTo(x + width / 2, y + height + arrowSize);
-  ctx.lineTo(x + width / 2 + arrowSize, y + height);
-  ctx.arcTo(x + width, y + height, x + width, y + height - radius, radius);
+  // Go back up and to the right, to draw the second half of the arrow tip.
+  ctx.lineTo(x + width, y + height);
+  // Go up to just below the top/right rounded corner.
   ctx.lineTo(x + width, y + radius);
+  // Draw the top/right rounded corner.
   ctx.arcTo(x + width, y, x + width - radius, y, radius);
+  // Go to the left.
   ctx.lineTo(x + radius, y);
+  // Draw the top/left rounded corner.
   ctx.arcTo(x, y, x, y + radius, radius);
 
   ctx.stroke();
@@ -138,15 +178,15 @@ function drawBubbleRect(ctx, x, y, width, height, radius, margin, arrowSize, ali
  *
  * @param  {CanvasRenderingContext2D} ctx
  *         The 2D canvas context.
- * @param  {Number} x1
+ * @param  {number} x1
  *         The x-axis of the coordinate for the begin of the line.
- * @param  {Number} y1
+ * @param  {number} y1
  *         The y-axis of the coordinate for the begin of the line.
- * @param  {Number} x2
+ * @param  {number} x2
  *         The x-axis of the coordinate for the end of the line.
- * @param  {Number} y2
+ * @param  {number} y2
  *         The y-axis of the coordinate for the end of the line.
- * @param  {Object} [options]
+ * @param  {object} [options]
  *         The options object.
  * @param  {Array} [options.matrix=identity()]
  *         The transformation matrix to apply.
@@ -154,10 +194,10 @@ function drawBubbleRect(ctx, x, y, width, height, radius, margin, arrowSize, ali
  *         If set, the line will be extended to reach the boundaries specified.
  */
 function drawLine(ctx, x1, y1, x2, y2, options) {
-  let matrix = options.matrix || identity();
+  const matrix = options.matrix || identity();
 
-  let p1 = apply(matrix, [x1, y1]);
-  let p2 = apply(matrix, [x2, y2]);
+  const p1 = apply(matrix, [x1, y1]);
+  const p2 = apply(matrix, [x2, y2]);
 
   x1 = p1[0];
   y1 = p1[1];
@@ -170,12 +210,13 @@ function drawLine(ctx, x1, y1, x2, y2, options) {
       x2 = options.extendToBoundaries[2];
     } else {
       y1 = options.extendToBoundaries[1];
-      x1 = (p2[0] - p1[0]) * (y1 - p1[1]) / (p2[1] - p1[1]) + p1[0];
+      x1 = ((p2[0] - p1[0]) * (y1 - p1[1])) / (p2[1] - p1[1]) + p1[0];
       y2 = options.extendToBoundaries[3];
-      x2 = (p2[0] - p1[0]) * (y2 - p1[1]) / (p2[1] - p1[1]) + p1[0];
+      x2 = ((p2[0] - p1[0]) * (y2 - p1[1])) / (p2[1] - p1[1]) + p1[0];
     }
   }
 
+  ctx.beginPath();
   ctx.moveTo(Math.round(x1), Math.round(y1));
   ctx.lineTo(Math.round(x2), Math.round(y2));
 }
@@ -186,19 +227,19 @@ function drawLine(ctx, x1, y1, x2, y2, options) {
  *
  * @param  {CanvasRenderingContext2D} ctx
  *         The 2D canvas context.
- * @param  {Number} x1
+ * @param  {number} x1
  *         The x-axis coordinate of the rectangle's diagonal start point.
- * @param  {Number} y1
+ * @param  {number} y1
  *         The y-axis coordinate of the rectangle's diagonal start point.
- * @param  {Number} x2
+ * @param  {number} x2
  *         The x-axis coordinate of the rectangle's diagonal end point.
- * @param  {Number} y2
+ * @param  {number} y2
  *         The y-axis coordinate of the rectangle's diagonal end point.
  * @param  {Array} [matrix=identity()]
  *         The transformation matrix to apply.
  */
 function drawRect(ctx, x1, y1, x2, y2, matrix = identity()) {
-  let p = getPointsFromDiagonal(x1, y1, x2, y2, matrix);
+  const p = getPointsFromDiagonal(x1, y1, x2, y2, matrix);
 
   ctx.beginPath();
   ctx.moveTo(Math.round(p[0].x), Math.round(p[0].y));
@@ -213,15 +254,15 @@ function drawRect(ctx, x1, y1, x2, y2, matrix = identity()) {
  *
  * @param  {CanvasRenderingContext2D} ctx
  *         The 2D canvas context.
- * @param  {Number} x
+ * @param  {number} x
  *         The x-axis origin of the rectangle.
- * @param  {Number} y
+ * @param  {number} y
  *         The y-axis origin of the rectangle.
- * @param  {Number} width
+ * @param  {number} width
  *         The width of the rectangle.
- * @param  {Number} height
+ * @param  {number} height
  *         The height of the rectangle.
- * @param  {Number} radius
+ * @param  {number} radius
  *         The radius of the rounding.
  */
 function drawRoundedRect(ctx, x, y, width, height, radius) {
@@ -245,10 +286,10 @@ function drawRoundedRect(ctx, x, y, width, height, radius) {
  *
  * @param  {Array} points
  *         An array with 4 pointer objects {x, y} representing the box quads.
- * @return {Object} DOMRect-like object of the 4 points.
+ * @return {object} DOMRect-like object of the 4 points.
  */
 function getBoundsFromPoints(points) {
-  let bounds = {};
+  const bounds = {};
 
   bounds.left = Math.min(points[0].x, points[1].x, points[2].x, points[3].x);
   bounds.right = Math.max(points[0].x, points[1].x, points[2].x, points[3].x);
@@ -283,21 +324,35 @@ function getBoundsFromPoints(points) {
  *         The current element.
  * @param  {Window} window
  *         The window object.
- * @return {Object} An object with the following properties:
+ * @param  {object} [options.ignoreWritingModeAndTextDirection=false]
+ *                  Avoid transforming the current matrix to match the text direction
+ *                  and writing mode.
+ * @return {object} An object with the following properties:
  *         - {Array} currentMatrix
  *           The current matrix.
  *         - {Boolean} hasNodeTransformations
  *           true if the node has transformed and false otherwise.
  */
-function getCurrentMatrix(element, window) {
-  let computedStyle = element.ownerGlobal.getComputedStyle(element);
+function getCurrentMatrix(
+  element,
+  window,
+  { ignoreWritingModeAndTextDirection } = {}
+) {
+  const computedStyle = getComputedStyle(element);
 
-  let paddingTop = parseFloat(computedStyle.paddingTop);
-  let paddingLeft = parseFloat(computedStyle.paddingLeft);
-  let borderTop = parseFloat(computedStyle.borderTopWidth);
-  let borderLeft = parseFloat(computedStyle.borderLeftWidth);
+  const paddingTop = parseFloat(computedStyle.paddingTop);
+  const paddingRight = parseFloat(computedStyle.paddingRight);
+  const paddingBottom = parseFloat(computedStyle.paddingBottom);
+  const paddingLeft = parseFloat(computedStyle.paddingLeft);
+  const borderTop = parseFloat(computedStyle.borderTopWidth);
+  const borderRight = parseFloat(computedStyle.borderRightWidth);
+  const borderBottom = parseFloat(computedStyle.borderBottomWidth);
+  const borderLeft = parseFloat(computedStyle.borderLeftWidth);
 
-  let nodeMatrix = getNodeTransformationMatrix(element, window.document.documentElement);
+  const nodeMatrix = getNodeTransformationMatrix(
+    element,
+    window.document.documentElement
+  );
 
   let currentMatrix = identity();
   let hasNodeTransformations = false;
@@ -315,12 +370,29 @@ function getCurrentMatrix(element, window) {
   }
 
   // Translate the origin based on the node's padding and border values.
-  currentMatrix = multiply(currentMatrix,
-    translate(paddingLeft + borderLeft, paddingTop + borderTop));
+  currentMatrix = multiply(
+    currentMatrix,
+    translate(paddingLeft + borderLeft, paddingTop + borderTop)
+  );
 
-  if (WRITING_MODE_ADJUST_ENABLED) {
-    // Adjust as needed to match the writing mode and text direction of the element.
-    let writingModeMatrix = getWritingModeMatrix(element, computedStyle);
+  // Adjust as needed to match the writing mode and text direction of the element.
+  const size = {
+    width:
+      element.offsetWidth -
+      borderLeft -
+      borderRight -
+      paddingLeft -
+      paddingRight,
+    height:
+      element.offsetHeight -
+      borderTop -
+      borderBottom -
+      paddingTop -
+      paddingBottom,
+  };
+
+  if (!ignoreWritingModeAndTextDirection) {
+    const writingModeMatrix = getWritingModeMatrix(size, computedStyle);
     if (!isIdentity(writingModeMatrix)) {
       currentMatrix = multiply(currentMatrix, writingModeMatrix);
     }
@@ -334,13 +406,30 @@ function getCurrentMatrix(element, window) {
  *
  * @param  {Array} points
  *         An array with 4 pointer objects {x, y} representing the box quads.
- * @return {String} a Path Description that can be used in svg's <path> element.
+ * @return {string} a Path Description that can be used in svg's <path> element.
  */
 function getPathDescriptionFromPoints(points) {
-  return "M" + points[0].x + "," + points[0].y + " " +
-         "L" + points[1].x + "," + points[1].y + " " +
-         "L" + points[2].x + "," + points[2].y + " " +
-         "L" + points[3].x + "," + points[3].y;
+  return (
+    "M" +
+    points[0].x +
+    "," +
+    points[0].y +
+    " " +
+    "L" +
+    points[1].x +
+    "," +
+    points[1].y +
+    " " +
+    "L" +
+    points[2].x +
+    "," +
+    points[2].y +
+    " " +
+    "L" +
+    points[3].x +
+    "," +
+    points[3].y
+  );
 }
 
 /**
@@ -348,13 +437,13 @@ function getPathDescriptionFromPoints(points) {
  * the four coordinates of a rectangle. If a matrix is provided, applies the matrix
  * function to each of the coordinates' value.
  *
- * @param  {Number} x1
+ * @param  {number} x1
  *         The x-axis coordinate of the rectangle's diagonal start point.
- * @param  {Number} y1
+ * @param  {number} y1
  *         The y-axis coordinate of the rectangle's diagonal start point.
- * @param  {Number} x2
+ * @param  {number} x2
  *         The x-axis coordinate of the rectangle's diagonal end point.
- * @param  {Number} y2
+ * @param  {number} y2
  *         The y-axis coordinate of the rectangle's diagonal end point.
  * @param  {Array} [matrix=identity()]
  *         A transformation matrix to apply.
@@ -366,80 +455,14 @@ function getPointsFromDiagonal(x1, y1, x2, y2, matrix = identity()) {
     [x1, y1],
     [x2, y1],
     [x2, y2],
-    [x1, y2]
+    [x1, y2],
   ].map(point => {
-    let transformedPoint = apply(matrix, point);
+    const transformedPoint = apply(matrix, point);
 
     return { x: transformedPoint[0], y: transformedPoint[1] };
   });
 }
 
-/**
- * Returns the matrix to rotate, translate, and reflect (if needed) from the element's
- * top-left origin into the actual writing mode and text direction applied to the element.
- *
- * @param  {Element} element
- *         The current element.
- * @param  {CSSStyleDeclaration} computedStyle
- *         The computed style for the element.
- * @return {Array}
- *         The matrix with adjustments for writing mode and text direction, if any.
- */
-function getWritingModeMatrix(element, computedStyle) {
-  let currentMatrix = identity();
-  let { direction, writingMode } = computedStyle;
-
-  switch (writingMode) {
-    case "horizontal-tb":
-      // This is the initial value.  No further adjustment needed.
-      break;
-    case "vertical-rl":
-      currentMatrix = multiply(
-        translate(element.offsetWidth, 0),
-        rotate(-Math.PI / 2)
-      );
-      break;
-    case "vertical-lr":
-      currentMatrix = multiply(
-        reflectAboutY(),
-        rotate(-Math.PI / 2)
-      );
-      break;
-    case "sideways-rl":
-      currentMatrix = multiply(
-        translate(element.offsetWidth, 0),
-        rotate(-Math.PI / 2)
-      );
-      break;
-    case "sideways-lr":
-      currentMatrix = multiply(
-        rotate(Math.PI / 2),
-        translate(-element.offsetHeight, 0)
-      );
-      break;
-    default:
-      console.error(`Unexpected writing-mode: ${writingMode}`);
-  }
-
-  switch (direction) {
-    case "ltr":
-      // This is the initial value.  No further adjustment needed.
-      break;
-    case "rtl":
-      let rowLength = element.offsetWidth;
-      if (writingMode != "horizontal-tb") {
-        rowLength = element.offsetHeight;
-      }
-      currentMatrix = multiply(currentMatrix, translate(rowLength, 0));
-      currentMatrix = multiply(currentMatrix, reflectAboutY());
-      break;
-    default:
-      console.error(`Unexpected direction: ${direction}`);
-  }
-
-  return currentMatrix;
-}
-
 /**
  * Updates the <canvas> element's style in accordance with the current window's
  * device pixel ratio, and the position calculated in `getCanvasPosition`. It also
@@ -448,20 +471,35 @@ function getWritingModeMatrix(element, computedStyle) {
  *
  * @param  {Canvas} canvas
  *         The <canvas> element.
- * @param  {Object} canvasPosition
+ * @param  {object} canvasPosition
  *         A pointer object {x, y} representing the <canvas> position to the top left
  *         corner of the page.
- * @param  {Number} devicePixelRatio
+ * @param  {number} devicePixelRatio
  *         The device pixel ratio.
+ * @param  {Window} [options.zoomWindow]
+ *         Optional window object used to calculate zoom (default = undefined).
  */
-function updateCanvasElement(canvas, canvasPosition, devicePixelRatio) {
+function updateCanvasElement(
+  canvas,
+  canvasPosition,
+  devicePixelRatio,
+  { zoomWindow } = {}
+) {
   let { x, y } = canvasPosition;
-  let size = CANVAS_SIZE / devicePixelRatio;
+  const size = CANVAS_SIZE / devicePixelRatio;
+
+  if (zoomWindow) {
+    const zoom = getCurrentZoom(zoomWindow);
+    x *= zoom;
+    y *= zoom;
+  }
 
   // Resize the canvas taking the dpr into account so as to have crisp lines, and
   // translating it to give the perception that it always covers the viewport.
-  canvas.setAttribute("style",
-    `width: ${size}px; height: ${size}px; transform: translate(${x}px, ${y}px);`);
+  canvas.setAttribute(
+    "style",
+    `width: ${size}px; height: ${size}px; transform: translate(${x}px, ${y}px);`
+  );
   canvas.getCanvasContext("2d").clearRect(0, 0, CANVAS_SIZE, CANVAS_SIZE);
 }
 
@@ -470,26 +508,31 @@ function updateCanvasElement(canvas, canvasPosition, devicePixelRatio) {
  * document's size, canvas size, and viewport's size. This is called when a page's scroll
  * is detected.
  *
- * @param  {Object} canvasPosition
+ * @param  {object} canvasPosition
  *         A pointer object {x, y} representing the <canvas> position to the top left
  *         corner of the page.
- * @param  {Object} scrollPosition
+ * @param  {object} scrollPosition
  *         A pointer object {x, y} representing the window's pageXOffset and pageYOffset.
  * @param  {Window} window
  *         The window object.
- * @param  {Object} windowDimensions
+ * @param  {object} windowDimensions
  *         An object {width, height} representing the window's dimensions for the
  *         `window` given.
- * @return {Boolean} true if the <canvas> position was updated and false otherwise.
+ * @return {boolean} true if the <canvas> position was updated and false otherwise.
  */
-function updateCanvasPosition(canvasPosition, scrollPosition, window, windowDimensions) {
+function updateCanvasPosition(
+  canvasPosition,
+  scrollPosition,
+  window,
+  windowDimensions
+) {
   let { x: canvasX, y: canvasY } = canvasPosition;
-  let { x: scrollX, y: scrollY } = scrollPosition;
-  let cssCanvasSize = CANVAS_SIZE / window.devicePixelRatio;
-  let viewportSize = getViewportDimensions(window);
-  let { height, width } = windowDimensions;
-  let canvasWidth = cssCanvasSize;
-  let canvasHeight = cssCanvasSize;
+  const { x: scrollX, y: scrollY } = scrollPosition;
+  const cssCanvasSize = CANVAS_SIZE / window.devicePixelRatio;
+  const viewportSize = getViewportDimensions(window);
+  const { height, width } = windowDimensions;
+  const canvasWidth = cssCanvasSize;
+  const canvasHeight = cssCanvasSize;
   let hasUpdated = false;
 
   // Those values indicates the relative horizontal and vertical space the page can
@@ -498,20 +541,22 @@ function updateCanvasPosition(canvasPosition, scrollPosition, window, windowDime
   // sides (top/bottom, left/right; so 1/2 for each side) and also we don't want to
   // shown the edges of the canvas in case of fast scrolling (to avoid showing undraw
   // areas, therefore another 1/2 here).
-  let bufferSizeX = (canvasWidth - viewportSize.width) >> 2;
-  let bufferSizeY = (canvasHeight - viewportSize.height) >> 2;
+  const bufferSizeX = (canvasWidth - viewportSize.width) >> 2;
+  const bufferSizeY = (canvasHeight - viewportSize.height) >> 2;
 
   // Defines the boundaries for the canvas.
-  let leftBoundary = 0;
-  let rightBoundary = width - canvasWidth;
-  let topBoundary = 0;
-  let bottomBoundary = height - canvasHeight;
+  const leftBoundary = 0;
+  const rightBoundary = width - canvasWidth;
+  const topBoundary = 0;
+  const bottomBoundary = height - canvasHeight;
 
   // Defines the thresholds that triggers the canvas' position to be updated.
-  let leftThreshold = scrollX - bufferSizeX;
-  let rightThreshold = scrollX - canvasWidth + viewportSize.width + bufferSizeX;
-  let topThreshold = scrollY - bufferSizeY;
-  let bottomThreshold = scrollY - canvasHeight + viewportSize.height + bufferSizeY;
+  const leftThreshold = scrollX - bufferSizeX;
+  const rightThreshold =
+    scrollX - canvasWidth + viewportSize.width + bufferSizeX;
+  const topThreshold = scrollY - bufferSizeY;
+  const bottomThreshold =
+    scrollY - canvasHeight + viewportSize.height + bufferSizeY;
 
   if (canvasX < rightBoundary && canvasX < rightThreshold) {
     canvasX = Math.min(leftThreshold, rightBoundary);
