@@ -50,20 +50,20 @@ function isFileChannel(channel) {
  * @constructor
  * @param {DevToolsServerConnection} conn
  *        The connection into which this Actor will be added.
- * @param {Object} sessionContext
+ * @param {object} sessionContext
  *        The Session Context to help know what is debugged.
  *        See devtools/server/actors/watcher/session-context.js
- * @param {Object} options
+ * @param {object} options
  *        Dictionary object with the following attributes:
  *        - onNetworkEventUpdate: optional function
  *          Callback for updates for the network event
  *        - onNetworkEventDestroy: optional function
  *          Callback for the destruction of the network event
- * @param {Object} networkEventOptions
+ * @param {object} networkEventOptions
  *        Object describing the network event or the configuration of the
  *        network observer, and which cannot be easily inferred from the raw
  *        channel.
- *        - blockingExtension: optional string
+ *        - extension: optional object with `blocking` or `blocked` extension IDs
  *          id of the blocking webextension if any
  *        - blockedReason: optional number or string
  *        - discardRequestBody: boolean
@@ -109,7 +109,7 @@ class NetworkEventActor extends Actor {
     };
 
     if (isDataChannel(channel) || isFileChannel(channel)) {
-      this._innerWindowId = null;
+      this._innerWindowId = lazy.NetworkUtils.getChannelInnerWindowId(channel);
       this._isNavigationRequest = false;
 
       this._request = {
@@ -223,7 +223,7 @@ class NetworkEventActor extends Actor {
       resourceId: this._channelId,
       resourceType: NETWORK_EVENT,
       blockedReason,
-      blockingExtension: networkEventOptions.blockingExtension,
+      extension: networkEventOptions.extension,
       browsingContextID,
       cause,
       // This is used specifically in the browser toolbox console to distinguish privileged
@@ -290,7 +290,6 @@ class NetworkEventActor extends Actor {
       headersSize = this._request.rawHeaders.length;
       rawHeaders = this._createLongStringActor(this._request.rawHeaders);
     }
-
     return {
       headers: this._request.headers.map(header => ({
         name: header.name,
@@ -446,8 +445,25 @@ class NetworkEventActor extends Actor {
    *         The response packet - network response content.
    */
   getResponseContent() {
+    const content = { ...this._response.content };
+    if (this._response.contentLongStringActor) {
+      // Remove the old actor from the pool as new actor will be created
+      // with updated content.
+      this.unmanage(this._response.contentLongStringActor);
+    }
+    this._response.contentLongStringActor = new LongStringActor(
+      this.conn,
+      // When trying to fetch content on a previous page load, or a cancelled request,
+      // `response.content` can be an empty object
+      content.text || ""
+    );
+    // bug 1462561 - Use "json" type and manually manage/marshall actors to workaround
+    // protocol.js performance issue
+    this.manage(this._response.contentLongStringActor);
+    content.text = this._response.contentLongStringActor.form();
+
     return {
-      content: this._response.content,
+      content,
       contentDiscarded: this._discardResponseBody,
     };
   }
@@ -468,9 +484,9 @@ class NetworkEventActor extends Actor {
     };
   }
 
-  /** ****************************************************************
+  /******************************************************************
    * Listeners for new network event data coming from NetworkMonitor.
-   ******************************************************************/
+   *****************************************************************/
 
   addCacheDetails({ fromCache, fromServiceWorker }) {
     this._resource.fromCache = fromCache;
@@ -643,7 +659,7 @@ class NetworkEventActor extends Actor {
    *
    * @param object
    */
-  addResponseContentComplete({ blockedReason, blockingExtension }) {
+  addResponseContentComplete({ blockedReason, extension }) {
     // Ignore calls when this actor is already destroyed
     if (this.isDestroyed()) {
       return;
@@ -653,7 +669,7 @@ class NetworkEventActor extends Actor {
       lazy.NetworkUtils.NETWORK_EVENT_TYPES.RESPONSE_CONTENT_COMPLETE,
       {
         blockedReason,
-        blockingExtension,
+        extension,
       }
     );
   }
@@ -664,25 +680,23 @@ class NetworkEventActor extends Actor {
    * @param object content
    *        The response content.
    */
-  addResponseContent(content) {
+  addResponseContent(content, data) {
+    const { blockedReason, extension } = data || {};
+
     // Ignore calls when this actor is already destroyed
     if (this.isDestroyed()) {
       return;
     }
 
     this._response.content = content;
-    content.text = new LongStringActor(this.conn, content.text);
-    // bug 1462561 - Use "json" type and manually manage/marshall actors to workaround
-    // protocol.js performance issue
-    this.manage(content.text);
-    content.text = content.text.form();
-
     this._onEventUpdate(
       lazy.NetworkUtils.NETWORK_EVENT_TYPES.RESPONSE_CONTENT,
       {
         mimeType: content.mimeType,
         contentSize: content.size,
         transferredSize: content.transferredSize,
+        blockedReason,
+        extension,
       }
     );
   }
