@@ -12,7 +12,6 @@ const {
 const {
   InspectorCSSParserWrapper,
 } = require("resource://devtools/shared/css/lexer.js");
-const TrackChangeEmitter = require("resource://devtools/server/actors/utils/track-change-emitter.js");
 const {
   getRuleText,
   getTextAtLineColumn,
@@ -28,6 +27,12 @@ loader.lazyRequireGetter(
   "resource://devtools/server/actors/inspector/css-logic.js",
   true
 );
+loader.lazyRequireGetter(
+  this,
+  "getNodeDisplayName",
+  "resource://devtools/server/actors/inspector/utils.js",
+  true
+);
 loader.lazyRequireGetter(
   this,
   "SharedCssLogic",
@@ -77,11 +82,11 @@ const XHTML_NS = "http://www.w3.org/1999/xhtml";
 class StyleRuleActor extends Actor {
   /**
    *
-   * @param {Object} options
+   * @param {object} options
    * @param {PageStyleActor} options.pageStyle
    * @param {CSSStyleRule|Element} options.item
-   * @param {Boolean} options.userAdded: Optional boolean to distinguish rules added by the user.
-   * @param {String} options.pseudoElement An optional pseudo-element type in cases when
+   * @param {boolean} options.userAdded: Optional boolean to distinguish rules added by the user.
+   * @param {string} options.pseudoElement An optional pseudo-element type in cases when
    *        the CSS rule applies to a pseudo-element.
    */
   constructor({ pageStyle, item, userAdded = false, pseudoElement = null }) {
@@ -89,7 +94,11 @@ class StyleRuleActor extends Actor {
     this.pageStyle = pageStyle;
     this.rawStyle = item.style;
     this._userAdded = userAdded;
+    this._pseudoElements = new Set();
     this._pseudoElement = pseudoElement;
+    if (pseudoElement) {
+      this._pseudoElements.add(pseudoElement);
+    }
     this._parentSheet = null;
     // Parsed CSS declarations from this.form().declarations used to check CSS property
     // names and values before tracking changes. Using cached values instead of accessing
@@ -144,6 +153,10 @@ class StyleRuleActor extends Actor {
     this.rawNode = null;
     this.rawRule = null;
     this._declarations = null;
+    if (this._pseudoElements) {
+      this._pseudoElements.clear();
+      this._pseudoElements = null;
+    }
   }
 
   // Objects returned by this actor are owned by the PageStyleActor
@@ -183,7 +196,7 @@ class StyleRuleActor extends Actor {
     let rule = this.rawRule;
 
     while (rule.parentRule) {
-      ancestors.unshift(this.pageStyle._styleRef(rule.parentRule));
+      ancestors.unshift(this.pageStyle.styleRef(rule.parentRule));
       rule = rule.parentRule;
     }
 
@@ -200,7 +213,7 @@ class StyleRuleActor extends Actor {
    * - the rule's ancestor rules (@media, @supports, @keyframes), if any
    * - the rule's position within its ancestor tree, if any
    *
-   * @return {Object}
+   * @return {object}
    */
   get metadata() {
     const data = {};
@@ -292,6 +305,25 @@ class StyleRuleActor extends Actor {
     return data;
   }
 
+  /**
+   * Returns true if the pseudo element anonymous node (e.g. ::before, ::marker, …) is selected.
+   * Returns false if a non pseudo element node is selected and we're looking into its pseudo
+   * elements rules (i.e. this is for the "Pseudo-elements" section in the Rules view")
+   */
+  get isPseudoElementAnonymousNodeSelected() {
+    if (!this._pseudoElement) {
+      return false;
+    }
+
+    // `this._pseudoElement` is the returned value by getNodeDisplayName, i.e that does
+    // differ from this.pageStyle.selectedElement.implementedPseudoElement (e.g. for
+    // view transition element, it will be `::view-transition-group(root)`, while
+    // implementedPseudoElement will be `::view-transition-group`).
+    return (
+      this._pseudoElement === getNodeDisplayName(this.pageStyle.selectedElement)
+    );
+  }
+
   /**
    * StyleRuleActor is spawned once per CSS Rule, but will be refreshed based on the
    * currently selected DOM Element, which is updated when PageStyleActor.getApplied
@@ -299,15 +331,20 @@ class StyleRuleActor extends Actor {
    */
   get currentlySelectedElement() {
     let { selectedElement } = this.pageStyle;
-    if (!this._pseudoElement) {
+    // If we're not handling a pseudo element, or if the pseudo element node
+    // (e.g. ::before, ::marker, …) is the one selected in the markup view, we can
+    // directly return selected element.
+    if (!this._pseudoElement || this.isPseudoElementAnonymousNodeSelected) {
       return selectedElement;
     }
 
-    // Otherwise, we can be in one of two cases:
-    // - we are selecting a pseudo element, and that pseudo element is referenced
-    //   by `selectedElement`
-    // - we are selecting the pseudo element "parent", we need to walk down the tree
-    //   from `selectedElemnt` to find the pseudo element.
+    // Otherwise we are selecting the pseudo element "parent" (binding), and we need to
+    // walk down the tree from `selectedElement` to find the pseudo element.
+
+    // FIXME: ::view-transition pseudo elements don't have a _moz_generated_content_ prefixed
+    // nodename, but have specific type and name attribute.
+    // At the moment this isn't causing any issues because we don't display the view
+    // transition rules in the pseudo element section, but this should be fixed in Bug 1998345.
     const pseudo = this._pseudoElement.replaceAll(":", "");
     const nodeName = `_moz_generated_content_${pseudo}`;
 
@@ -335,23 +372,22 @@ class StyleRuleActor extends Actor {
 
     const { selectedElement } = this.pageStyle;
 
-    // We can be in one of two cases:
-    // - we are selecting a pseudo element, and that pseudo element is referenced
-    //   by `selectedElement`
-    // - we are selecting the pseudo element "parent".
-    // implementPseudoElement returns the pseudo-element string if this element represents
-    // a pseudo-element, or null otherwise. See https://searchfox.org/mozilla-central/rev/1b90936792b2c71ef931cb1b8d6baff9d825592e/dom/webidl/Element.webidl#102-107
-    const isPseudoElementParentSelected =
-      selectedElement.implementedPseudoElement !== this._pseudoElement;
-
     return selectedElement.ownerGlobal.getComputedStyle(
       selectedElement,
       // If we are selecting the pseudo element parent, we need to pass the pseudo element
       // to getComputedStyle to actually get the computed style of the pseudo element.
-      isPseudoElementParentSelected ? this._pseudoElement : null
+      !this.isPseudoElementAnonymousNodeSelected ? this._pseudoElement : null
     );
   }
 
+  get pseudoElements() {
+    return this._pseudoElements;
+  }
+
+  addPseudo(pseudoElement) {
+    this._pseudoElements.add(pseudoElement);
+  }
+
   getDocument(sheet) {
     if (!sheet.associatedDocument) {
       throw new Error(
@@ -370,6 +406,7 @@ class StyleRuleActor extends Actor {
     const form = {
       actor: this.actorID,
       type: this.type,
+      className: this.ruleClassName,
       line: this.line || undefined,
       column: this.column,
       traits: {
@@ -407,7 +444,7 @@ class StyleRuleActor extends Actor {
         form.selectors = [];
         form.selectorsSpecificity = [];
         break;
-      case "CSSStyleRule":
+      case "CSSStyleRule": {
         form.selectors = [];
         form.selectorsSpecificity = [];
 
@@ -427,7 +464,8 @@ class StyleRuleActor extends Actor {
           form.selectorWarnings = selectorWarnings;
         }
         break;
-      case ELEMENT_STYLE:
+      }
+      case ELEMENT_STYLE: {
         // Elements don't have a parent stylesheet, and therefore
         // don't have an associated URI.  Provide a URI for
         // those.
@@ -435,6 +473,7 @@ class StyleRuleActor extends Actor {
         form.href = doc.location ? doc.location.href : "";
         form.authoredText = this.rawNode.getAttribute("style");
         break;
+      }
       case PRES_HINTS:
         form.href = "";
         break;
@@ -445,6 +484,7 @@ class StyleRuleActor extends Actor {
         form.href = this.rawRule.href;
         break;
       case "CSSKeyframesRule":
+      case "CSSPositionTryRule":
         form.name = this.rawRule.name;
         break;
       case "CSSKeyframeRule":
@@ -507,10 +547,19 @@ class StyleRuleActor extends Actor {
           // In such case InspectorUtils.supports() would return false, but that would be
           // odd to show "invalid" pres hints declaration in the UI.
           this.ruleClassName === PRES_HINTS ||
-          InspectorUtils.supports(
+          (InspectorUtils.supports(
             `${decl.name}:${decl.value}`,
             supportsOptions
-          );
+          ) &&
+            // !important values are not valid in @position-try and @keyframes
+            // TODO: We might extend InspectorUtils.supports to take the actual rule
+            // so we wouldn't have to hardcode this, but this does come with some
+            // challenges (see Bug 2004379).
+            !(
+              decl.priority === "important" &&
+              (this.ruleClassName === "CSSPositionTryRule" ||
+                this.ruleClassName === "CSSKeyframesRule")
+            ));
         const inactiveCssData = getInactiveCssDataForProperty(
           el,
           style,
@@ -598,11 +647,12 @@ class StyleRuleActor extends Actor {
   /**
    * Return the rule cssText if applicable, null otherwise
    *
-   * @returns {String|null}
+   * @returns {string | null}
    */
   _getCssText() {
     switch (this.ruleClassName) {
       case "CSSNestedDeclarations":
+      case "CSSPositionTryRule":
       case "CSSStyleRule":
       case ELEMENT_STYLE:
       case PRES_HINTS:
@@ -617,8 +667,8 @@ class StyleRuleActor extends Actor {
   /**
    * Parse the rule declarations from its text.
    *
-   * @param {Object} options
-   * @param {Boolean} options.parseComments
+   * @param {object} options
+   * @param {boolean} options.parseComments
    * @returns {Array} @see parseNamedDeclarations
    */
   parseRuleDeclarations({ parseComments }) {
@@ -639,7 +689,7 @@ class StyleRuleActor extends Actor {
 
   /**
    *
-   * @returns {Array<Object>} ancestorData: An array of ancestor item data
+   * @returns {Array<object>} ancestorData: An array of ancestor item data
    */
   _getAncestorDataForForm() {
     const ancestorData = [];
@@ -754,8 +804,8 @@ class StyleRuleActor extends Actor {
    * Send an event notifying that the location of the rule has
    * changed.
    *
-   * @param {Number} line the new line number
-   * @param {Number} column the new column number
+   * @param {number} line the new line number
+   * @param {number} column the new column number
    */
   _notifyLocationChanged(line, column) {
     this.emit("location-changed", line, column);
@@ -832,6 +882,7 @@ class StyleRuleActor extends Actor {
     "CSSLayerBlockRule",
     "CSSMediaRule",
     "CSSNestedDeclarations",
+    "CSSPositionTryRule",
     "CSSStyleRule",
     "CSSSupportsRule",
   ]);
@@ -855,7 +906,7 @@ class StyleRuleActor extends Actor {
    * The authored text will include invalid and otherwise ignored
    * properties.
    *
-   * @param {Boolean} skipCache
+   * @param {boolean} skipCache
    *        If a value for authoredText was previously found and cached,
    *        ignore it and parse the stylehseet again. The authoredText
    *        may be outdated if a descendant of this rule has changed.
@@ -907,7 +958,7 @@ class StyleRuleActor extends Actor {
    * selector that uniquely identifies the element and with the rule body consisting of
    * the element's style attribute.
    *
-   * @return {String}
+   * @return {string}
    */
   async getRuleText() {
     // Bail out if the rule is not supported or not an element inline style.
@@ -950,7 +1001,7 @@ class StyleRuleActor extends Actor {
    * Set the contents of the rule.  This rewrites the rule in the
    * stylesheet and causes it to be re-evaluated.
    *
-   * @param {String} newText
+   * @param {string} newText
    *        The new text of the rule
    * @param {Array} modifications
    *        Array with modifications applied to the rule. Contains objects like:
@@ -1098,9 +1149,9 @@ class StyleRuleActor extends Actor {
    * current rule. Returns the newly inserted css rule or null if the rule is
    * unsuccessfully inserted to the parent style sheet.
    *
-   * @param {String} value
+   * @param {string} value
    *        The new selector value
-   * @param {Boolean} editAuthored
+   * @param {boolean} editAuthored
    *        True if the selector should be updated by editing the
    *        authored text; false if the selector should be updated via
    *        CSSOM.
@@ -1180,11 +1231,11 @@ class StyleRuleActor extends Actor {
    * Take an object with instructions to modify a CSS declaration and log an object with
    * normalized metadata which describes the change in the context of this rule.
    *
-   * @param {Object} change
+   * @param {object} change
    *        Data about a modification to a declaration. @see |modifyProperties()|
-   * @param {Object} newDeclarations
+   * @param {object} newDeclarations
    *        The current declarations array to get the latest values, names...
-   * @param {Object} oldDeclarations
+   * @param {object} oldDeclarations
    *        The previous declarations array to use to fetch old values, names...
    */
   logDeclarationChange(change, newDeclarations, oldDeclarations) {
@@ -1211,7 +1262,7 @@ class StyleRuleActor extends Actor {
     const data = this.metadata;
 
     switch (change.type) {
-      case "set":
+      case "set": {
         data.type = prevValue ? "declaration-add" : "declaration-update";
         // If `change.newName` is defined, use it because the property is being renamed.
         // Otherwise, a new declaration is being created or the value of an existing
@@ -1249,6 +1300,7 @@ class StyleRuleActor extends Actor {
         }
 
         break;
+      }
 
       case "remove":
         data.type = "declaration-remove";
@@ -1263,20 +1315,20 @@ class StyleRuleActor extends Actor {
         break;
     }
 
-    TrackChangeEmitter.trackChange(data);
+    this.pageStyle.inspector.targetActor.emit("track-css-change", data);
   }
 
   /**
    * Helper method for tracking CSS changes. Logs the change of this rule's selector as
    * two operations: a removal using the old selector and an addition using the new one.
    *
-   * @param {String} oldSelector
+   * @param {string} oldSelector
    *        This rule's previous selector.
-   * @param {String} newSelector
+   * @param {string} newSelector
    *        This rule's new selector.
    */
   logSelectorChange(oldSelector, newSelector) {
-    TrackChangeEmitter.trackChange({
+    this.pageStyle.inspector.targetActor.emit("track-css-change", {
       ...this.metadata,
       type: "selector-remove",
       add: null,
@@ -1284,7 +1336,7 @@ class StyleRuleActor extends Actor {
       selector: oldSelector,
     });
 
-    TrackChangeEmitter.trackChange({
+    this.pageStyle.inspector.targetActor.emit("track-css-change", {
       ...this.metadata,
       type: "selector-add",
       add: null,
@@ -1304,75 +1356,65 @@ class StyleRuleActor extends Actor {
    *
    * @param {DOMNode} node
    *        The current selected element
-   * @param {String} value
+   * @param {string} value
    *        The new selector value
-   * @param {Boolean} editAuthored
+   * @param {boolean} editAuthored
    *        True if the selector should be updated by editing the
    *        authored text; false if the selector should be updated via
    *        CSSOM.
-   * @returns {Object}
+   * @returns {Promise<object>}
    *        Returns an object that contains the applied style properties of the
    *        new rule and a boolean indicating whether or not the new selector
    *        matches the current selected element
    */
-  modifySelector(node, value, editAuthored = false) {
+  async modifySelector(node, value, editAuthored = false) {
     if (this.type === ELEMENT_STYLE || this.rawRule.selectorText === value) {
       return { ruleProps: null, isMatching: true };
     }
 
     // The rule's previous selector is lost after calling _addNewSelector(). Save it now.
     const oldValue = this.rawRule.selectorText;
-    let selectorPromise = this._addNewSelector(value, editAuthored);
+    const newCssRule = await this._addNewSelector(value, editAuthored);
 
-    if (editAuthored) {
-      selectorPromise = selectorPromise.then(newCssRule => {
-        if (newCssRule) {
-          this.logSelectorChange(oldValue, value);
-          const style = this.pageStyle._styleRef(newCssRule);
-          // See the comment in |form| to understand this.
-          return style.getAuthoredCssText().then(() => newCssRule);
-        }
-        return newCssRule;
-      });
+    if (editAuthored && newCssRule) {
+      this.logSelectorChange(oldValue, value);
+      const style = this.pageStyle.styleRef(newCssRule);
+      // See the comment in |form| to understand this.
+      await style.getAuthoredCssText();
     }
 
-    return selectorPromise.then(newCssRule => {
-      let entries = null;
-      let isMatching = false;
+    let entries = null;
+    let isMatching = false;
 
-      if (newCssRule) {
-        const ruleEntry = this.pageStyle.findEntryMatchingRule(
-          node,
-          newCssRule
-        );
-        if (ruleEntry.length === 1) {
-          entries = this.pageStyle.getAppliedProps(node, ruleEntry, {
-            matchedSelectors: true,
-          });
-        } else {
-          entries = this.pageStyle.getNewAppliedProps(node, newCssRule);
-        }
-
-        isMatching = entries.some(
-          ruleProp => !!ruleProp.matchedSelectorIndexes.length
-        );
+    if (newCssRule) {
+      const ruleEntry = this.pageStyle.findEntryMatchingRule(node, newCssRule);
+      if (ruleEntry) {
+        entries = this.pageStyle.getAppliedProps(node, [ruleEntry], {
+          matchedSelectors: true,
+        });
+      } else {
+        entries = this.pageStyle.getNewAppliedProps(node, newCssRule);
       }
 
-      const result = { isMatching };
-      if (entries) {
-        result.ruleProps = { entries };
-      }
+      isMatching = entries.some(
+        ruleProp => !!ruleProp.matchedSelectorIndexes.length
+      );
+    }
 
-      return result;
-    });
+    const result = { isMatching };
+    if (entries) {
+      result.ruleProps = { entries };
+    }
+
+    return result;
   }
 
   /**
    * Get the eligible query container for a given @container rule and a given node
    *
-   * @param {Number} ancestorRuleIndex: The index of the @container rule in this.ancestorRules
+   * @param {number} ancestorRuleIndex: The index of the @container rule in this.ancestorRules
    * @param {NodeActor} nodeActor: The nodeActor for which we want to retrieve the query container
-   * @returns {Object} An object with the following properties:
+   * @returns {object} An object with the following properties:
    *          - node: {NodeActor|null} The nodeActor representing the query container,
    *            null if none were found
    *          - containerType: {string} The computed `containerType` value of the query container
@@ -1415,7 +1457,7 @@ class StyleRuleActor extends Actor {
    * If any have changed their used/unused state, potentially as a result of changes in
    * another rule, fire a "rule-updated" event with this rule actor in its latest state.
    *
-   * @param {Boolean} forceRefresh: Set to true to emit "rule-updated", even if the state
+   * @param {boolean} forceRefresh: Set to true to emit "rule-updated", even if the state
    *        of the declarations didn't change.
    */
   maybeRefresh(forceRefresh) {
@@ -1461,10 +1503,11 @@ exports.StyleRuleActor = StyleRuleActor;
 /**
  * Compute the start and end offsets of a rule's selector text, given
  * the CSS text and the line and column at which the rule begins.
- * @param {String} initialText
- * @param {Number} line (1-indexed)
- * @param {Number} column (1-indexed)
- * @return {array} An array with two elements: [startOffset, endOffset].
+ *
+ * @param {string} initialText
+ * @param {number} line (1-indexed)
+ * @param {number} column (1-indexed)
+ * @return {Array} An array with two elements: [startOffset, endOffset].
  *                 The elements mark the bounds in |initialText| of
  *                 the CSS rule's selector.
  */
