@@ -373,7 +373,7 @@ class WindowGlobalTargetActor extends BaseTargetActor {
       writable: true,
     });
 
-    // When this target tracks only one WindowGlobal, set a fixed innerWindowId,
+    // When this target tracks only one WindowGlobal, set a fixed innerWindowId and window,
     // so that it can easily be read safely while the related WindowGlobal is being destroyed.
     if (this.followWindowGlobalLifeCycle) {
       Object.defineProperty(this, "innerWindowId", {
@@ -381,6 +381,16 @@ class WindowGlobalTargetActor extends BaseTargetActor {
         configurable: false,
         writable: false,
       });
+      Object.defineProperty(this, "window", {
+        value: this.window,
+        configurable: true,
+        writable: false,
+      });
+      Object.defineProperty(this, "chromeEventHandler", {
+        value: this.chromeEventHandler,
+        configurable: false,
+        writable: false,
+      });
     }
 
     // Save references to the original document we attached to
@@ -461,27 +471,15 @@ class WindowGlobalTargetActor extends BaseTargetActor {
   _targetScopedActorPool = null;
 
   /**
-   * An object on which listen for DOMWindowCreated and pageshow events.
+   * A EventTarget object on which to listen for 'DOMWindowCreated' and 'pageshow' events.
    */
   get chromeEventHandler() {
     return getDocShellChromeEventHandler(this.docShell);
   }
 
-  /**
-   * Getter for the nsIMessageManager associated to the window global.
-   */
-  get messageManager() {
-    try {
-      return this.docShell.messageManager;
-    } catch (e) {
-      // In some cases we can't get a docshell.  We just have no message manager
-      // then,
-      return null;
-    }
-  }
-
   /**
    * Getter for the list of all `docShell`s in the window global.
+   *
    * @return {Array}
    */
   get docShells() {
@@ -538,6 +536,7 @@ class WindowGlobalTargetActor extends BaseTargetActor {
 
   /**
    * Getter for the list of all content DOM windows in the window global.
+   *
    * @return {Array}
    */
   get windows() {
@@ -762,10 +761,10 @@ class WindowGlobalTargetActor extends BaseTargetActor {
   /**
    * Called when the actor is removed from the connection.
    *
-   * @params {Object} options
-   * @params {Boolean} options.isTargetSwitching: Set to true when this is called during
+   * @params {object} options
+   * @params {boolean} options.isTargetSwitching: Set to true when this is called during
    *         a target switch.
-   * @params {Boolean} options.isModeSwitching: Set to true true when this is called as the
+   * @params {boolean} options.isModeSwitching: Set to true true when this is called as the
    *         result of a change to the devtools.browsertoolbox.scope pref.
    */
   destroy({ isTargetSwitching = false, isModeSwitching = false } = {}) {
@@ -776,6 +775,17 @@ class WindowGlobalTargetActor extends BaseTargetActor {
     }
     this.destroying = true;
 
+    // In case the window already navigated to another origin,
+    // which is possibly in another process, nullify window
+    // as most, if not all attributes would throw.
+    if (Cu.isRemoteProxy(this.window)) {
+      Object.defineProperty(this, "window", {
+        value: null,
+        configurable: true,
+        writable: false,
+      });
+    }
+
     // Force flushing pending resources if the actor isn't already destroyed.
     // This helps notify the client about pending resources on navigation.
     if (!this.isDestroyed()) {
