@@ -83,9 +83,9 @@ exports.WatcherActor = class WatcherActor extends Actor {
    * @param {object} sessionContext
    *        The Session Context to help know what is debugged.
    *        See devtools/server/actors/watcher/session-context.js
-   * @param {Number} sessionContext.browserId: If this is a "browser-element" context type,
+   * @param {number} sessionContext.browserId: If this is a "browser-element" context type,
    *        the "browserId" of the <browser> element we would like to debug.
-   * @param {Boolean} sessionContext.isServerTargetSwitchingEnabled: Flag to to know if we should
+   * @param {boolean} sessionContext.isServerTargetSwitchingEnabled: Flag to to know if we should
    *        spawn new top level targets for the debugged context.
    */
   constructor(conn, sessionContext) {
@@ -218,7 +218,7 @@ exports.WatcherActor = class WatcherActor extends Actor {
     super.destroy();
   }
 
-  /*
+  /**
    * Get the list of the currently watched resources for this watcher.
    *
    * @return Array<String>
@@ -339,6 +339,7 @@ exports.WatcherActor = class WatcherActor extends Actor {
   /**
    * Flush any early iframe targets relating to this top level
    * window target.
+   *
    * @param {number} topInnerWindowID
    */
   _flushIframeTargets(topInnerWindowID) {
@@ -577,7 +578,7 @@ exports.WatcherActor = class WatcherActor extends Actor {
     );
 
     switch (this.sessionContext.type) {
-      case "all":
+      case "all": {
         const parentProcessTargetActor = actors.find(
           actor => actor.typeName === "parentProcessTarget"
         );
@@ -585,6 +586,7 @@ exports.WatcherActor = class WatcherActor extends Actor {
           return new Set([parentProcessTargetActor]);
         }
         return new Set();
+      }
       case "browser-element":
       case "webextension":
         // All target actors for browser-element and webextension sessions
@@ -755,7 +757,7 @@ exports.WatcherActor = class WatcherActor extends Actor {
   /**
    * Returns the network actor.
    *
-   * @return {Object} actor
+   * @return {object} actor
    *        The network actor.
    */
   getNetworkParentActor() {
@@ -769,7 +771,7 @@ exports.WatcherActor = class WatcherActor extends Actor {
   /**
    * Returns the blackboxing actor.
    *
-   * @return {Object} actor
+   * @return {object} actor
    *        The blackboxing actor.
    */
   getBlackboxingActor() {
@@ -783,7 +785,7 @@ exports.WatcherActor = class WatcherActor extends Actor {
   /**
    * Returns the breakpoint list actor.
    *
-   * @return {Object} actor
+   * @return {object} actor
    *        The breakpoint list actor.
    */
   getBreakpointListActor() {
@@ -797,7 +799,7 @@ exports.WatcherActor = class WatcherActor extends Actor {
   /**
    * Returns the target configuration actor.
    *
-   * @return {Object} actor
+   * @return {object} actor
    *        The configuration actor.
    */
   getTargetConfigurationActor() {
@@ -810,7 +812,7 @@ exports.WatcherActor = class WatcherActor extends Actor {
   /**
    * Returns the thread configuration actor.
    *
-   * @return {Object} actor
+   * @return {object} actor
    *        The configuration actor.
    */
   getThreadConfigurationActor() {
@@ -825,11 +827,11 @@ exports.WatcherActor = class WatcherActor extends Actor {
    * Used to agrement some new entries for a given data type (watchers target, resources,
    * breakpoints,...)
    *
-   * @param {String} type
+   * @param {string} type
    *        Data type to contribute to.
    * @param {Array<*>} entries
    *        List of values to add or set for this data type.
-   * @param {String} updateType
+   * @param {string} updateType
    *        "add" will only add the new entries in the existing data set.
    *        "set" will update the data set with the new entries.
    */
@@ -892,7 +894,7 @@ exports.WatcherActor = class WatcherActor extends Actor {
    * Used to remve some existing entries for a given data type (watchers target, resources,
    * breakpoints,...)
    *
-   * @param {String} type
+   * @param {string} type
    *        Data type to modify.
    * @param {Array<*>} entries
    *        List of values to remove from this data type.
@@ -920,7 +922,7 @@ exports.WatcherActor = class WatcherActor extends Actor {
   /**
    * Retrieve the current watched data for the provided type.
    *
-   * @param {String} type
+   * @param {string} type
    *        Data type to retrieve.
    */
   getSessionDataForType(type) {
@@ -932,7 +934,7 @@ exports.WatcherActor = class WatcherActor extends Actor {
    * This will notify the Service Worker JS Process Actors about the new top level page domain.
    * So that we start tracking that domain's workers.
    *
-   * @param {String} newTargetUrl
+   * @param {string} newTargetUrl
    */
   async updateDomainSessionDataForServiceWorkers(newTargetUrl) {
     // If the url could not be parsed the host defaults to an empty string.
