@@ -4,11 +4,28 @@
 
 "use strict";
 
-const { Arg, generateActorSpec, RetVal } = require("devtools/shared/protocol");
+const {
+  Arg,
+  generateActorSpec,
+  RetVal,
+} = require("resource://devtools/shared/protocol.js");
 
 const flexboxSpec = generateActorSpec({
   typeName: "flexbox",
 
+  methods: {
+    getFlexItems: {
+      request: {},
+      response: {
+        flexitems: RetVal("array:flexitem"),
+      },
+    },
+  },
+});
+
+const flexItemSpec = generateActorSpec({
+  typeName: "flexitem",
+
   methods: {},
 });
 
@@ -22,27 +39,37 @@ const layoutSpec = generateActorSpec({
   typeName: "layout",
 
   methods: {
-    getAllFlexbox: {
+    getCurrentFlexbox: {
       request: {
-        rootNode: Arg(0, "domnode"),
-        traverseFrames: Arg(1, "nullable:boolean")
+        node: Arg(0, "domnode"),
+        onlyLookAtParents: Arg(1, "nullable:boolean"),
       },
       response: {
-        flexboxes: RetVal("array:flexbox")
-      }
+        flexbox: RetVal("nullable:flexbox"),
+      },
+    },
+
+    getCurrentGrid: {
+      request: {
+        node: Arg(0, "domnode"),
+      },
+      response: {
+        grid: RetVal("nullable:grid"),
+      },
     },
 
     getGrids: {
       request: {
-        rootNode: Arg(0, "domnode")
+        rootNode: Arg(0, "domnode"),
       },
       response: {
-        grids: RetVal("array:grid")
-      }
+        grids: RetVal("array:grid"),
+      },
     },
   },
 });
 
 exports.flexboxSpec = flexboxSpec;
+exports.flexItemSpec = flexItemSpec;
 exports.gridSpec = gridSpec;
 exports.layoutSpec = layoutSpec;
