@@ -17,6 +17,7 @@
 #include "mozilla/dom/WindowGlobalActorsBinding.h"
 #include "mozilla/gfx/DrawEventRecorder.h"
 #include "mozilla/gfx/InlineTranslator.h"
+#include "mozilla/gfx/RecordedEvent.h"
 #include "mozilla/Logging.h"
 #include "mozilla/PresShell.h"
 
@@ -147,11 +148,12 @@ PaintFragment PaintFragment::Record(dom::BrowsingContext* aBc,
     thebes.SetMatrix(Matrix::Scaling(aScale, aScale));
     thebes.SetCrossProcessPaintScale(aScale);
     RefPtr<PresShell> presShell = presContext->PresShell();
-    Unused << presShell->RenderDocument(r, renderDocFlags, aBackgroundColor,
-                                        &thebes);
+    (void)presShell->RenderDocument(r, renderDocFlags, aBackgroundColor,
+                                    &thebes);
   }
 
   if (!recorder->mOutputStream.mValid) {
+    recorder->DetachResources();
     return PaintFragment{};
   }
 
@@ -162,11 +164,14 @@ PaintFragment PaintFragment::Record(dom::BrowsingContext* aBc,
   recorder->mOutputStream.mLength = 0;
   recorder->mOutputStream.mCapacity = 0;
 
-  return PaintFragment{
+  PaintFragment fragment{
       surfaceSize.ToUnknownSize(),
       std::move(recording),
       std::move(recorder->TakeDependentSurfaces()),
   };
+
+  recorder->DetachResources();
+  return fragment;
 }
 
 bool PaintFragment::IsEmpty() const {
@@ -323,7 +328,7 @@ CrossProcessPaint::CrossProcessPaint(float aScale, dom::TabId aRoot,
                                      CrossProcessPaintFlags aFlags)
     : mRoot{aRoot}, mScale{aScale}, mPendingFragments{0}, mFlags{aFlags} {}
 
-CrossProcessPaint::~CrossProcessPaint() = default;
+CrossProcessPaint::~CrossProcessPaint() { Clear(NS_ERROR_ABORT); }
 
 void CrossProcessPaint::ReceiveFragment(dom::WindowGlobalParent* aWGP,
                                         PaintFragment&& aFragment) {
