@@ -5,6 +5,7 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "AnimationHelper.h"
+#include "CompositorAnimationStorage.h"
 #include "base/process_util.h"
 #include "gfx2DGlue.h"                 // for ThebesRect
 #include "gfxLineSegment.h"            // for gfxLineSegment
@@ -15,21 +16,18 @@
 #include "mozilla/ServoStyleConsts.h"  // for StyleComputedTimingFunction
 #include "mozilla/dom/AnimationEffectBinding.h"  // for dom::FillMode
 #include "mozilla/dom/KeyframeEffectBinding.h"   // for dom::IterationComposite
-#include "mozilla/dom/KeyframeEffect.h"       // for dom::KeyFrameEffectReadOnly
-#include "mozilla/dom/Nullable.h"             // for dom::Nullable
-#include "mozilla/layers/APZSampler.h"        // for APZSampler
-#include "mozilla/layers/CompositorThread.h"  // for CompositorThreadHolder
-#include "mozilla/LayerAnimationInfo.h"       // for GetCSSPropertiesFor()
-#include "mozilla/Maybe.h"                    // for Maybe<>
-#include "mozilla/MotionPathUtils.h"          // for ResolveMotionPath()
-#include "mozilla/ServoBindings.h"  // for Servo_ComposeAnimationSegment, etc
+#include "mozilla/dom/KeyframeEffect.h"  // for dom::KeyFrameEffectReadOnly
+#include "mozilla/dom/Nullable.h"        // for dom::Nullable
+#include "mozilla/layers/APZSampler.h"   // for APZSampler
+#include "mozilla/CSSPropertyId.h"
+#include "mozilla/LayerAnimationInfo.h"   // for GetCSSPropertiesFor()
+#include "mozilla/Maybe.h"                // for Maybe<>
+#include "mozilla/MotionPathUtils.h"      // for ResolveMotionPath()
 #include "mozilla/StyleAnimationValue.h"  // for StyleAnimationValue, etc
-#include "nsCSSPropertyID.h"              // for eCSSProperty_offset_path, etc
-#include "nsDeviceContext.h"              // for AppUnitsPerCSSPixel
+#include "NonCustomCSSPropertyId.h"       // for eCSSProperty_offset_path, etc
 #include "nsDisplayList.h"                // for nsDisplayTransform, etc
 
-namespace mozilla {
-namespace layers {
+namespace mozilla::layers {
 
 static dom::Nullable<TimeDuration> CalculateElapsedTimeForScrollTimeline(
     const Maybe<APZSampler::ScrollOffsetAndRange> aScrollMeta,
@@ -320,7 +318,7 @@ AnimationHelper::SampleResult AnimationHelper::SampleAnimationForEachNode(
     const MutexAutoLock& aProofOfMapLock, TimeStamp aPreviousFrameTime,
     TimeStamp aCurrentFrameTime, const AnimatedValue* aPreviousValue,
     nsTArray<PropertyAnimationGroup>& aPropertyAnimationGroups,
-    nsTArray<RefPtr<StyleAnimationValue>>& aAnimationValues /* out */) {
+    SampledAnimationArray& aAnimationValues /* output */) {
   MOZ_ASSERT(!aPropertyAnimationGroups.IsEmpty(),
              "Should be called with animation data");
   MOZ_ASSERT(aAnimationValues.IsEmpty(),
@@ -448,11 +446,13 @@ static bool HasTransformLikeAnimations(const AnimationArray& aAnimations) {
 #endif
 
 AnimationStorageData AnimationHelper::ExtractAnimations(
-    const LayersId& aLayersId, const AnimationArray& aAnimations) {
+    const LayersId& aLayersId, const AnimationArray& aAnimations,
+    const CompositorAnimationStorage* aStorage,
+    const TimeStamp& aPreviousSampleTime) {
   AnimationStorageData storageData;
   storageData.mLayersId = aLayersId;
 
-  nsCSSPropertyID prevID = eCSSProperty_UNKNOWN;
+  NonCustomCSSPropertyId prevId = eCSSProperty_UNKNOWN;
   PropertyAnimationGroup* currData = nullptr;
   DebugOnly<const layers::Animatable*> currBaseStyle = nullptr;
 
@@ -460,7 +460,7 @@ AnimationStorageData AnimationHelper::ExtractAnimations(
     // Animations with same property are grouped together, so we can just
     // check if the current property is the same as the previous one for
     // knowing this is a new group.
-    if (prevID != animation.property()) {
+    if (prevId != animation.property()) {
       // Got a different group, we should create a different array.
       currData = storageData.mAnimation.AppendElement();
       currData->mProperty = animation.property();
@@ -470,7 +470,7 @@ AnimationStorageData AnimationHelper::ExtractAnimations(
         storageData.mTransformData = animation.transformData();
       }
 
-      prevID = animation.property();
+      prevId = animation.property();
 
       // Reset the debug pointer.
       currBaseStyle = nullptr;
@@ -540,12 +540,37 @@ AnimationStorageData AnimationHelper::ExtractAnimations(
     propertyAnimation->mScrollTimelineOptions =
         animation.scrollTimelineOptions();
 
+    RefPtr<StyleAnimationValue> startValue;
+    if (animation.replacedTransitionId()) {
+      if (const auto* animatedValue =
+              aStorage->GetAnimatedValue(*animation.replacedTransitionId())) {
+        startValue = animatedValue->AsAnimationValue(animation.property());
+        // Basically, the timeline time is increasing monotonically, so it may
+        // not make sense to have a negative start time (i.e. the case when
+        // aPreviousSampleTime is behind the origin time). Therefore, if the
+        // previous sample time is less than the origin time, we skip the
+        // replacement of the start time.
+        if (!aPreviousSampleTime.IsNull() &&
+            (aPreviousSampleTime >= animation.originTime())) {
+          propertyAnimation->mStartTime =
+              Some(aPreviousSampleTime - animation.originTime());
+        }
+
+        MOZ_ASSERT(animation.segments().Length() == 1,
+                   "The CSS Transition only has one segement");
+      }
+    }
+
     nsTArray<PropertyAnimation::SegmentData>& segmentData =
         propertyAnimation->mSegments;
     for (const AnimationSegment& segment : animation.segments()) {
       segmentData.AppendElement(PropertyAnimation::SegmentData{
-          AnimationValue::FromAnimatable(animation.property(),
-                                         segment.startState()),
+          // Note that even though we re-compute the start value on the main
+          // thread, we still replace it with the last sampled value, to avoid
+          // any possible lag.
+          startValue ? startValue
+                     : AnimationValue::FromAnimatable(animation.property(),
+                                                      segment.startState()),
           AnimationValue::FromAnimatable(animation.property(),
                                          segment.endState()),
           segment.sampleFn(), segment.startPortion(), segment.endPortion(),
@@ -560,7 +585,7 @@ AnimationStorageData AnimationHelper::ExtractAnimations(
   if (!storageData.mAnimation.IsEmpty()) {
     nsCSSPropertyIDSet seenProperties;
     for (const auto& group : storageData.mAnimation) {
-      nsCSSPropertyID id = group.mProperty;
+      NonCustomCSSPropertyId id = group.mProperty;
 
       MOZ_ASSERT(!seenProperties.HasProperty(id), "Should be a new property");
       seenProperties.AddProperty(id);
@@ -603,8 +628,8 @@ uint64_t AnimationHelper::GetNextCompositorAnimationsId() {
 }
 
 gfx::Matrix4x4 AnimationHelper::ServoAnimationValueToMatrix4x4(
-    const nsTArray<RefPtr<StyleAnimationValue>>& aValues,
-    const TransformData& aTransformData, gfx::Path* aCachedMotionPath) {
+    const SampledAnimationArray& aValues, const TransformData& aTransformData,
+    gfx::Path* aCachedMotionPath) {
   using nsStyleTransformMatrix::TransformReferenceBox;
 
   // This is a bit silly just to avoid the transform list copy from the
@@ -626,8 +651,9 @@ gfx::Matrix4x4 AnimationHelper::ServoAnimationValueToMatrix4x4(
 
   for (const auto& value : aValues) {
     MOZ_ASSERT(value);
-    nsCSSPropertyID id = Servo_AnimationValue_GetPropertyId(value);
-    switch (id) {
+    CSSPropertyId property(eCSSProperty_UNKNOWN);
+    Servo_AnimationValue_GetPropertyId(value, &property);
+    switch (property.mId) {
       case eCSSProperty_transform:
         MOZ_ASSERT(!transform);
         transform = Servo_AnimationValue_GetTransform(value);
@@ -841,5 +867,4 @@ bool AnimationHelper::ShouldBeJank(const LayoutDeviceRect& aPrerenderedRect,
                                prerenderedQuad.mPoints[3].y, clipRect);
 }
 
-}  // namespace layers
-}  // namespace mozilla
+}  // namespace mozilla::layers
