@@ -15,15 +15,14 @@
 #include "mozilla/EffectSet.h"
 #include "mozilla/MotionPathUtils.h"
 #include "mozilla/PresShell.h"
-#include "mozilla/StaticPrefs_layout.h"
+#include "mozilla/ScrollContainerFrame.h"
 #include "nsIContent.h"
 #include "nsLayoutUtils.h"
-#include "nsPresContextInlines.h"
+#include "nsRefreshDriver.h"
 #include "nsStyleTransformMatrix.h"
 #include "PuppetWidget.h"
 
-namespace mozilla {
-namespace layers {
+namespace mozilla::layers {
 
 using TransformReferenceBox = nsStyleTransformMatrix::TransformReferenceBox;
 
@@ -84,33 +83,28 @@ void AnimationInfo::ClearAnimationsForNextTransaction() {
   mPendingAnimations->Clear();
 }
 
-bool AnimationInfo::StartPendingAnimations(const TimeStamp& aReadyTime) {
-  bool updated = false;
-  for (size_t animIdx = 0, animEnd = mAnimations.Length(); animIdx < animEnd;
-       animIdx++) {
-    Animation& anim = mAnimations[animIdx];
+void AnimationInfo::MaybeStartPendingAnimation(Animation& aAnimation,
+                                               const TimeStamp& aReadyTime) {
+  // If the animation is doing an async update of its playback rate, then we
+  // want to match whatever its current time would be at *aReadyTime*.
+  if (!std::isnan(aAnimation.previousPlaybackRate()) &&
+      aAnimation.startTime().isSome() && !aAnimation.originTime().IsNull() &&
+      !aAnimation.isNotPlaying()) {
+    TimeDuration readyTime = aReadyTime - aAnimation.originTime();
+    aAnimation.holdTime() = dom::Animation::CurrentTimeFromTimelineTime(
+        readyTime, aAnimation.startTime().ref(),
+        aAnimation.previousPlaybackRate());
+    // Make start time null so that we know to update it below.
+    aAnimation.startTime() = Nothing();
+  }
 
-    // If the animation is doing an async update of its playback rate, then we
-    // want to match whatever its current time would be at *aReadyTime*.
-    if (!std::isnan(anim.previousPlaybackRate()) && anim.startTime().isSome() &&
-        !anim.originTime().IsNull() && !anim.isNotPlaying()) {
-      TimeDuration readyTime = aReadyTime - anim.originTime();
-      anim.holdTime() = dom::Animation::CurrentTimeFromTimelineTime(
-          readyTime, anim.startTime().ref(), anim.previousPlaybackRate());
-      // Make start time null so that we know to update it below.
-      anim.startTime() = Nothing();
-    }
-
-    // If the animation is play-pending, resolve the start time.
-    if (anim.startTime().isNothing() && !anim.originTime().IsNull() &&
-        !anim.isNotPlaying()) {
-      TimeDuration readyTime = aReadyTime - anim.originTime();
-      anim.startTime() = Some(dom::Animation::StartTimeFromTimelineTime(
-          readyTime, anim.holdTime(), anim.playbackRate()));
-      updated = true;
-    }
+  // If the aAnimationation is play-pending, resolve the start time.
+  if (aAnimation.startTime().isNothing() && !aAnimation.originTime().IsNull() &&
+      !aAnimation.isNotPlaying()) {
+    const TimeDuration readyTime = aReadyTime - aAnimation.originTime();
+    aAnimation.startTime() = Some(dom::Animation::StartTimeFromTimelineTime(
+        readyTime, aAnimation.holdTime(), aAnimation.playbackRate()));
   }
-  return updated;
 }
 
 bool AnimationInfo::ApplyPendingUpdatesForThisTransaction() {
@@ -126,8 +120,8 @@ bool AnimationInfo::ApplyPendingUpdatesForThisTransaction() {
 bool AnimationInfo::HasTransformAnimation() const {
   const nsCSSPropertyIDSet& transformSet =
       LayerAnimationInfo::GetCSSPropertiesFor(DisplayItemType::TYPE_TRANSFORM);
-  for (uint32_t i = 0; i < mAnimations.Length(); i++) {
-    if (transformSet.HasProperty(mAnimations[i].property())) {
+  for (const auto& animation : mAnimations) {
+    if (transformSet.HasProperty(animation.property())) {
       return true;
     }
   }
@@ -333,7 +327,7 @@ static Maybe<ScrollTimelineOptions> GetScrollTimelineOptions(
   return Some(ScrollTimelineOptions(source, timeline->Axis()));
 }
 
-static void SetAnimatable(nsCSSPropertyID aProperty,
+static void SetAnimatable(NonCustomCSSPropertyId aProperty,
                           const AnimationValue& aAnimationValue,
                           nsIFrame* aFrame, TransformReferenceBox& aRefBox,
                           layers::Animatable& aAnimatable) {
@@ -397,8 +391,7 @@ void AnimationInfo::AddAnimationForProperty(
     Send aSendFlag) {
   MOZ_ASSERT(aAnimation->GetEffect(),
              "Should not be adding an animation without an effect");
-  MOZ_ASSERT(!aAnimation->GetCurrentOrPendingStartTime().IsNull() ||
-                 !aAnimation->IsPlaying() ||
+  MOZ_ASSERT(!aAnimation->GetStartTime().IsNull() || !aAnimation->IsPlaying() ||
                  (aAnimation->GetTimeline() &&
                   aAnimation->GetTimeline()->TracksWallclockTime()),
              "If the animation has an unresolved start time it should either"
@@ -406,9 +399,9 @@ void AnimationInfo::AddAnimationForProperty(
              " timeline capable of converting TimeStamps (so we can calculate"
              " one later");
 
-  layers::Animation* animation = (aSendFlag == Send::NextTransaction)
-                                     ? AddAnimationForNextTransaction()
-                                     : AddAnimation();
+  Animation* animation = (aSendFlag == Send::NextTransaction)
+                             ? AddAnimationForNextTransaction()
+                             : AddAnimation();
 
   const TimingParams& timing = aAnimation->GetEffect()->NormalizedTiming();
 
@@ -425,9 +418,16 @@ void AnimationInfo::AddAnimationForProperty(
   // since after generating the new transition other requestAnimationFrame
   // callbacks may run that introduce further lag between the main thread and
   // the compositor.
-  dom::CSSTransition* cssTransition = aAnimation->AsCSSTransition();
-  if (cssTransition) {
-    cssTransition->UpdateStartValueFromReplacedTransition();
+  //
+  // Note that we will replace the start value with the last sampled animation
+  // value on the compositor.
+  // The computation here is for updating the keyframe values, to make sure the
+  // computed values on the main thread don't behind the rendering result on the
+  // compositor too much.
+  bool needReplaceTransition = false;
+  if (dom::CSSTransition* cssTransition = aAnimation->AsCSSTransition()) {
+    needReplaceTransition =
+        cssTransition->UpdateStartValueFromReplacedTransition();
   }
 
   animation->originTime() =
@@ -435,8 +435,7 @@ void AnimationInfo::AddAnimationForProperty(
           ? TimeStamp()
           : aAnimation->GetTimeline()->ToTimeStamp(TimeDuration());
 
-  dom::Nullable<TimeDuration> startTime =
-      aAnimation->GetCurrentOrPendingStartTime();
+  dom::Nullable<TimeDuration> startTime = aAnimation->GetStartTime();
   if (startTime.IsNull()) {
     animation->startTime() = Nothing();
   } else {
@@ -455,7 +454,9 @@ void AnimationInfo::AddAnimationForProperty(
       static_cast<float>(computedTiming.mIterationStart);
   animation->direction() = static_cast<uint8_t>(timing.Direction());
   animation->fillMode() = static_cast<uint8_t>(computedTiming.mFill);
-  animation->property() = aProperty.mProperty;
+  MOZ_ASSERT(!aProperty.mProperty.IsCustom(),
+             "We don't animate custom properties in the compositor");
+  animation->property() = aProperty.mProperty.mId;
   animation->playbackRate() =
       static_cast<float>(aAnimation->CurrentOrPendingPlaybackRate());
   animation->previousPlaybackRate() =
@@ -470,6 +471,12 @@ void AnimationInfo::AddAnimationForProperty(
   animation->isNotAnimating() = false;
   animation->scrollTimelineOptions() =
       GetScrollTimelineOptions(aAnimation->GetTimeline());
+  // We set this flag to let the compositor know that the start value of this
+  // transition is replaced. The compositor may replace the start value with its
+  // last sampled animation value, instead of using the segment.mFromValue we
+  // send to the compositor, to avoid any potential lag.
+  animation->replacedTransitionId() =
+      needReplaceTransition ? Some(GetCompositorAnimationsId()) : Nothing();
 
   TransformReferenceBox refBox(aFrame);
 
@@ -480,19 +487,17 @@ void AnimationInfo::AddAnimationForProperty(
       aAnimation->GetEffect()->AsKeyframeEffect()->BaseStyle(
           aProperty.mProperty);
   if (!baseStyle.IsNull()) {
-    SetAnimatable(aProperty.mProperty, baseStyle, aFrame, refBox,
+    SetAnimatable(aProperty.mProperty.mId, baseStyle, aFrame, refBox,
                   animation->baseStyle());
   } else {
     animation->baseStyle() = null_t();
   }
 
-  for (uint32_t segIdx = 0; segIdx < aProperty.mSegments.Length(); segIdx++) {
-    const AnimationPropertySegment& segment = aProperty.mSegments[segIdx];
-
+  for (const AnimationPropertySegment& segment : aProperty.mSegments) {
     AnimationSegment* animSegment = animation->segments().AppendElement();
-    SetAnimatable(aProperty.mProperty, segment.mFromValue, aFrame, refBox,
+    SetAnimatable(aProperty.mProperty.mId, segment.mFromValue, aFrame, refBox,
                   animSegment->startState());
-    SetAnimatable(aProperty.mProperty, segment.mToValue, aFrame, refBox,
+    SetAnimatable(aProperty.mProperty.mId, segment.mToValue, aFrame, refBox,
                   animSegment->endState());
 
     animSegment->startPortion() = segment.mFromKey;
@@ -502,6 +507,18 @@ void AnimationInfo::AddAnimationForProperty(
     animSegment->endComposite() = static_cast<uint8_t>(segment.mToComposite);
     animSegment->sampleFn() = segment.mTimingFunction;
   }
+
+  if (aAnimation->Pending()) {
+    TimeStamp readyTime = aAnimation->GetPendingReadyTime();
+    if (readyTime.IsNull()) {
+      // TODO(emilio): This should generally not happen anymore, can we remove
+      // this SetPendingReadyTime call?
+      readyTime = aFrame->PresContext()->RefreshDriver()->MostRecentRefresh();
+      MOZ_ASSERT(!readyTime.IsNull());
+      aAnimation->SetPendingReadyTime(readyTime);
+    }
+    MaybeStartPendingAnimation(*animation, readyTime);
+  }
 }
 
 // Let's use an example to explain this function:
@@ -535,22 +552,25 @@ void AnimationInfo::AddAnimationForProperty(
 // ]
 //
 // And then, for each transaction, we send this list to the compositor thread.
-static HashMap<nsCSSPropertyID, nsTArray<RefPtr<dom::Animation>>>
+static HashMap<NonCustomCSSPropertyId, nsTArray<RefPtr<dom::Animation>>>
 GroupAnimationsByProperty(const nsTArray<RefPtr<dom::Animation>>& aAnimations,
                           const nsCSSPropertyIDSet& aPropertySet) {
-  HashMap<nsCSSPropertyID, nsTArray<RefPtr<dom::Animation>>> groupedAnims;
+  HashMap<NonCustomCSSPropertyId, nsTArray<RefPtr<dom::Animation>>>
+      groupedAnims;
   for (const RefPtr<dom::Animation>& anim : aAnimations) {
     const dom::KeyframeEffect* effect = anim->GetEffect()->AsKeyframeEffect();
     MOZ_ASSERT(effect);
     for (const AnimationProperty& property : effect->Properties()) {
+      // TODO(zrhoffman, bug 1869475): Handle custom properties
       if (!aPropertySet.HasProperty(property.mProperty)) {
         continue;
       }
 
-      auto animsForPropertyPtr = groupedAnims.lookupForAdd(property.mProperty);
+      auto animsForPropertyPtr =
+          groupedAnims.lookupForAdd(property.mProperty.mId);
       if (!animsForPropertyPtr) {
         DebugOnly<bool> rv =
-            groupedAnims.add(animsForPropertyPtr, property.mProperty,
+            groupedAnims.add(animsForPropertyPtr, property.mProperty.mId,
                              nsTArray<RefPtr<dom::Animation>>());
         MOZ_ASSERT(rv, "Should have enough memory");
       }
@@ -563,8 +583,9 @@ GroupAnimationsByProperty(const nsTArray<RefPtr<dom::Animation>>& aAnimations,
 bool AnimationInfo::AddAnimationsForProperty(
     nsIFrame* aFrame, const EffectSet* aEffects,
     const nsTArray<RefPtr<dom::Animation>>& aCompositorAnimations,
-    const Maybe<TransformData>& aTransformData, nsCSSPropertyID aProperty,
-    Send aSendFlag, WebRenderLayerManager* aLayerManager) {
+    const Maybe<TransformData>& aTransformData,
+    NonCustomCSSPropertyId aProperty, Send aSendFlag,
+    WebRenderLayerManager* aLayerManager) {
   bool addedAny = false;
   // Add from first to last (since last overrides)
   for (dom::Animation* anim : aCompositorAnimations) {
@@ -572,12 +593,12 @@ bool AnimationInfo::AddAnimationsForProperty(
       continue;
     }
 
-    dom::KeyframeEffect* keyframeEffect =
-        anim->GetEffect() ? anim->GetEffect()->AsKeyframeEffect() : nullptr;
-    MOZ_ASSERT(keyframeEffect,
+    MOZ_ASSERT(anim->GetEffect() && anim->GetEffect()->AsKeyframeEffect(),
                "A playing animation should have a keyframe effect");
+    dom::KeyframeEffect* keyframeEffect = anim->GetEffect()->AsKeyframeEffect();
     const AnimationProperty* property =
-        keyframeEffect->GetEffectiveAnimationOfProperty(aProperty, *aEffects);
+        keyframeEffect->GetEffectiveAnimationOfProperty(
+            CSSPropertyId(aProperty), *aEffects);
     if (!property) {
       continue;
     }
@@ -592,9 +613,10 @@ bool AnimationInfo::AddAnimationsForProperty(
         "GetEffectiveAnimationOfProperty already tested the property "
         "is not overridden by !important rules");
 
-    // Don't add animations that are pending if their timeline does not
-    // track wallclock time. This is because any pending animations on layers
-    // will have their start time updated with the current wallclock time.
+    // Don't add animations that are pending if their timeline does not track
+    // wallclock time. This is because any pending animations on layers will
+    // have their start time updated with the current wallclock time.
+    //
     // If we can't convert that wallclock time back to an equivalent timeline
     // time, we won't be able to update the content animation and it will end
     // up being out of sync with the layer animation.
@@ -602,8 +624,8 @@ bool AnimationInfo::AddAnimationsForProperty(
     // Currently this only happens when the timeline is driven by a refresh
     // driver under test control. In this case, the next time the refresh
     // driver is advanced it will trigger any pending animations.
-    if (anim->Pending() &&
-        (anim->GetTimeline() && !anim->GetTimeline()->TracksWallclockTime())) {
+    if (anim->Pending() && anim->GetTimeline() &&
+        !anim->GetTimeline()->TracksWallclockTime()) {
       continue;
     }
 
@@ -646,27 +668,28 @@ static SideBits GetOverflowedSides(const nsRect& aOverflow,
 static std::pair<ParentLayerRect, gfx::Matrix4x4>
 GetClipRectAndTransformForPartialPrerender(
     const nsIFrame* aFrame, int32_t aDevPixelsToAppUnits,
-    const nsIFrame* aClipFrame, const nsIScrollableFrame* aScrollFrame) {
+    const nsIFrame* aClipFrame,
+    const ScrollContainerFrame* aScrollContainerFrame) {
   MOZ_ASSERT(aClipFrame);
 
   gfx::Matrix4x4 transformInClip =
       nsLayoutUtils::GetTransformToAncestor(RelativeTo{aFrame->GetParent()},
                                             RelativeTo{aClipFrame})
           .GetMatrix();
-  if (aScrollFrame) {
+  if (aScrollContainerFrame) {
     transformInClip.PostTranslate(
-        LayoutDevicePoint::FromAppUnits(aScrollFrame->GetScrollPosition(),
-                                        aDevPixelsToAppUnits)
+        LayoutDevicePoint::FromAppUnits(
+            aScrollContainerFrame->GetScrollPosition(), aDevPixelsToAppUnits)
             .ToUnknownPoint());
   }
 
   // We don't necessarily use nsLayoutUtils::CalculateCompositionSizeForFrame
   // since this is a case where we don't use APZ at all.
   return std::make_pair(
-      LayoutDeviceRect::FromAppUnits(aScrollFrame
-                                         ? aScrollFrame->GetScrollPortRect()
-                                         : aClipFrame->GetRectRelativeToSelf(),
-                                     aDevPixelsToAppUnits) *
+      LayoutDeviceRect::FromAppUnits(
+          aScrollContainerFrame ? aScrollContainerFrame->GetScrollPortRect()
+                                : aClipFrame->GetRectRelativeToSelf(),
+          aDevPixelsToAppUnits) *
           LayoutDeviceToLayerScale2D() * LayerToParentLayerScale(),
       transformInClip);
 }
@@ -680,14 +703,14 @@ static PartialPrerenderData GetPartialPrerenderData(
 
   const nsIFrame* clipFrame =
       nsLayoutUtils::GetNearestOverflowClipFrame(aFrame->GetParent());
-  const nsIScrollableFrame* scrollFrame = do_QueryFrame(clipFrame);
+  const ScrollContainerFrame* scrollContainerFrame = do_QueryFrame(clipFrame);
 
   if (!clipFrame) {
     // If there is no suitable clip frame in the same document, use the
     // root one.
-    scrollFrame = aFrame->PresShell()->GetRootScrollFrameAsScrollable();
-    if (scrollFrame) {
-      clipFrame = do_QueryFrame(scrollFrame);
+    scrollContainerFrame = aFrame->PresShell()->GetRootScrollContainerFrame();
+    if (scrollContainerFrame) {
+      clipFrame = scrollContainerFrame;
     } else {
       // If there is no root scroll frame, use the viewport frame.
       clipFrame = aFrame->PresShell()->GetRootFrame();
@@ -695,32 +718,33 @@ static PartialPrerenderData GetPartialPrerenderData(
   }
 
   // If the scroll frame is asyncronously scrollable, try to find the scroll id.
-  if (scrollFrame &&
-      !scrollFrame->GetScrollStyles().IsHiddenInBothDirections() &&
+  if (scrollContainerFrame &&
+      !scrollContainerFrame->GetScrollStyles().IsHiddenInBothDirections() &&
       nsLayoutUtils::AsyncPanZoomEnabled(aFrame)) {
     const bool isInPositionFixed =
         nsLayoutUtils::IsInPositionFixedSubtree(aFrame);
-    const ActiveScrolledRoot* asr = aItem->GetActiveScrolledRoot();
-    const nsIFrame* asrScrollableFrame =
-        asr ? do_QueryFrame(asr->mScrollableFrame) : nullptr;
+    // We need to find asynchronously scrollable ASRs, therefore we should
+    // ignore ASRs for pos:sticky display items.
+    const ActiveScrolledRoot* asr = aItem->GetNearestScrollASR();
     if (!isInPositionFixed && asr &&
-        aFrame->PresContext() == asrScrollableFrame->PresContext()) {
+        aFrame->PresContext() == asr->ScrollFrame()->PresContext()) {
       scrollId = asr->GetViewId();
-      MOZ_ASSERT(clipFrame == asrScrollableFrame);
+      MOZ_ASSERT(clipFrame == asr->ScrollFrame());
     } else {
       // Use the root scroll id in the same document if the target frame is in
       // position:fixed subtree or there is no ASR or the ASR is in a different
       // ancestor document.
       scrollId =
           nsLayoutUtils::ScrollIdForRootScrollFrame(aFrame->PresContext());
-      MOZ_ASSERT(clipFrame == aFrame->PresShell()->GetRootScrollFrame());
+      MOZ_ASSERT(clipFrame ==
+                 aFrame->PresShell()->GetRootScrollContainerFrame());
     }
   }
 
   int32_t devPixelsToAppUnits = aFrame->PresContext()->AppUnitsPerDevPixel();
 
   auto [clipRect, transformInClip] = GetClipRectAndTransformForPartialPrerender(
-      aFrame, devPixelsToAppUnits, clipFrame, scrollFrame);
+      aFrame, devPixelsToAppUnits, clipFrame, scrollContainerFrame);
 
   return PartialPrerenderData{
       LayoutDeviceRect::FromAppUnits(partialPrerenderedRect,
@@ -822,7 +846,7 @@ static Maybe<TransformData> CreateAnimationData(
 void AnimationInfo::AddNonAnimatingTransformLikePropertiesStyles(
     const nsCSSPropertyIDSet& aNonAnimatingProperties, nsIFrame* aFrame,
     Send aSendFlag) {
-  auto appendFakeAnimation = [this, aSendFlag](nsCSSPropertyID aProperty,
+  auto appendFakeAnimation = [this, aSendFlag](NonCustomCSSPropertyId aProperty,
                                                Animatable&& aBaseStyle) {
     layers::Animation* animation = (aSendFlag == Send::NextTransaction)
                                        ? AddAnimationForNextTransaction()
@@ -840,7 +864,7 @@ void AnimationInfo::AddNonAnimatingTransformLikePropertiesStyles(
       !display->mOffsetPath.IsNone() ||
       !aNonAnimatingProperties.HasProperty(eCSSProperty_offset_path);
 
-  for (nsCSSPropertyID id : aNonAnimatingProperties) {
+  for (NonCustomCSSPropertyId id : aNonAnimatingProperties) {
     switch (id) {
       case eCSSProperty_transform:
         if (!display->mTransform.IsNone()) {
@@ -942,7 +966,7 @@ void AnimationInfo::AddAnimationsForDisplayItem(
     return;
   }
 
-  const HashMap<nsCSSPropertyID, nsTArray<RefPtr<dom::Animation>>>
+  const HashMap<NonCustomCSSPropertyId, nsTArray<RefPtr<dom::Animation>>>
       compositorAnimations =
           GroupAnimationsByProperty(matchedAnimations, propertySet);
   Maybe<TransformData> transformData =
@@ -952,11 +976,7 @@ void AnimationInfo::AddAnimationsForDisplayItem(
                               ? AnimationDataType::WithMotionPath
                               : AnimationDataType::WithoutMotionPath,
                           aPosition);
-  // Bug 1424900: Drop this pref check after shipping individual transforms.
-  // Bug 1582554: Drop this pref check after shipping motion path.
   const bool hasMultipleTransformLikeProperties =
-      (StaticPrefs::layout_css_individual_transform_enabled() ||
-       StaticPrefs::layout_css_motion_path_enabled()) &&
       aType == DisplayItemType::TYPE_TRANSFORM;
   nsCSSPropertyIDSet nonAnimatingProperties =
       nsCSSPropertyIDSet::TransformLikeProperties();
@@ -1000,5 +1020,4 @@ void AnimationInfo::AddAnimationsForDisplayItem(
   }
 }
 
-}  // namespace layers
-}  // namespace mozilla
+}  // namespace mozilla::layers
