@@ -15,12 +15,9 @@
 #include "gtest/gtest.h"
 #include "gmock/gmock.h"
 
-#include "mozilla/Attributes.h"
-#include "mozilla/gfx/gfxVars.h"
-#include "mozilla/layers/AsyncCompositionManager.h"  // for ViewTransform
 #include "mozilla/layers/GeckoContentController.h"
 #include "mozilla/layers/CompositorBridgeParent.h"
-#include "mozilla/layers/LayerMetricsWrapper.h"
+#include "mozilla/layers/DoubleTapToZoom.h"
 #include "mozilla/layers/APZThreadUtils.h"
 #include "mozilla/layers/MatrixMessage.h"
 #include "mozilla/StaticPrefs_layout.h"
@@ -30,8 +27,8 @@
 #include "apz/src/AsyncPanZoomController.h"
 #include "apz/src/HitTestingTreeNode.h"
 #include "base/task.h"
-#include "Layers.h"
-#include "TestLayers.h"
+#include "gfxPlatform.h"
+#include "TestWRScrollData.h"
 #include "UnitTransforms.h"
 
 using namespace mozilla;
@@ -45,7 +42,7 @@ using ::testing::MockFunction;
 using ::testing::NiceMock;
 typedef mozilla::layers::GeckoContentController::TapType TapType;
 
-static TimeStamp GetStartupTime() {
+inline TimeStamp GetStartupTime() {
   static TimeStamp sStartupTime = TimeStamp::Now();
   return sStartupTime;
 }
@@ -76,9 +73,9 @@ inline PinchGestureInput CreatePinchGestureInput(
     PinchGestureInput::PinchGestureType aType, const ScreenPoint& aFocus,
     float aCurrentSpan, float aPreviousSpan, TimeStamp timestamp) {
   ParentLayerPoint localFocus(aFocus.x, aFocus.y);
-  PinchGestureInput result(
-      aType, PinchGestureInput::UNKNOWN, MillisecondsSinceStartup(timestamp),
-      timestamp, ExternalPoint(0, 0), aFocus, aCurrentSpan, aPreviousSpan, 0);
+  PinchGestureInput result(aType, PinchGestureInput::UNKNOWN, timestamp,
+                           ExternalPoint(0, 0), aFocus, aCurrentSpan,
+                           aPreviousSpan, 0);
   return result;
 }
 
@@ -99,6 +96,10 @@ class ScopedGfxSetting {
   Storage mOldVal;
 };
 
+static inline constexpr auto kDefaultTouchBehavior =
+    AllowedTouchBehavior::VERTICAL_PAN | AllowedTouchBehavior::HORIZONTAL_PAN |
+    AllowedTouchBehavior::PINCH_ZOOM | AllowedTouchBehavior::ANIMATING_ZOOM;
+
 #define FRESH_PREF_VAR_PASTE(id, line) id##line
 #define FRESH_PREF_VAR_EXPAND(id, line) FRESH_PREF_VAR_PASTE(id, line)
 #define FRESH_PREF_VAR FRESH_PREF_VAR_EXPAND(pref, __LINE__)
@@ -121,16 +122,13 @@ class ScopedGfxSetting {
       [=](float aPrefValue) { Preferences::SetFloat(prefName, aPrefValue); }, \
       prefValue)
 
-#define SCOPED_GFX_VAR(varBase, varType, varValue)         \
-  ScopedGfxSetting<const varType&, varType> var_##varBase( \
-      &(gfxVars::varBase), &(gfxVars::Set##varBase), varValue)
-
 class MockContentController : public GeckoContentController {
  public:
   MOCK_METHOD1(NotifyLayerTransforms, void(nsTArray<MatrixMessage>&&));
   MOCK_METHOD1(RequestContentRepaint, void(const RepaintRequest&));
-  MOCK_METHOD5(HandleTap, void(TapType, const LayoutDevicePoint&, Modifiers,
-                               const ScrollableLayerGuid&, uint64_t));
+  MOCK_METHOD6(HandleTap, void(TapType, const LayoutDevicePoint&, Modifiers,
+                               const ScrollableLayerGuid&, uint64_t,
+                               const Maybe<DoubleTapToZoomMetrics>&));
   MOCK_METHOD5(NotifyPinchGesture,
                void(PinchGestureInput::PinchGestureType,
                     const ScrollableLayerGuid&, const LayoutDevicePoint&,
@@ -143,8 +141,9 @@ class MockContentController : public GeckoContentController {
   void DispatchToRepaintThread(already_AddRefed<Runnable> aTask) {
     NS_DispatchToMainThread(std::move(aTask));
   }
-  MOCK_METHOD3(NotifyAPZStateChange, void(const ScrollableLayerGuid& aGuid,
-                                          APZStateChange aChange, int aArg));
+  MOCK_METHOD4(NotifyAPZStateChange,
+               void(const ScrollableLayerGuid& aGuid, APZStateChange aChange,
+                    int aArg, Maybe<uint64_t> aInputBlockId));
   MOCK_METHOD0(NotifyFlushComplete, void());
   MOCK_METHOD3(NotifyAsyncScrollbarDragInitiated,
                void(uint64_t, const ScrollableLayerGuid::ViewID&,
@@ -154,6 +153,12 @@ class MockContentController : public GeckoContentController {
   MOCK_METHOD1(NotifyAsyncAutoscrollRejected,
                void(const ScrollableLayerGuid::ViewID&));
   MOCK_METHOD1(CancelAutoscroll, void(const ScrollableLayerGuid&));
+  MOCK_METHOD2(NotifyScaleGestureComplete,
+               void(const ScrollableLayerGuid&, float aScale));
+  MOCK_METHOD4(UpdateOverscrollVelocity,
+               void(const ScrollableLayerGuid&, float, float, bool));
+  MOCK_METHOD4(UpdateOverscrollOffset,
+               void(const ScrollableLayerGuid&, float, float, bool));
 };
 
 class MockContentControllerDelayed : public MockContentController {
@@ -228,8 +233,11 @@ class MockContentControllerDelayed : public MockContentController {
 
 class TestAPZCTreeManager : public APZCTreeManager {
  public:
-  explicit TestAPZCTreeManager(MockContentControllerDelayed* aMcc)
-      : APZCTreeManager(LayersId{0}, gfx::gfxVars::UseWebRender()), mcc(aMcc) {}
+  explicit TestAPZCTreeManager(MockContentControllerDelayed* aMcc,
+                               UniquePtr<IAPZHitTester> aHitTester = nullptr)
+      : APZCTreeManager(LayersId{0}, std::move(aHitTester)), mcc(aMcc) {
+    Init();
+  }
 
   RefPtr<InputQueue> GetInputQueue() const { return mInputQueue; }
 
@@ -241,13 +249,36 @@ class TestAPZCTreeManager : public APZCTreeManager {
    **/
   void CancelAnimation() { EXPECT_TRUE(false); }
 
+  bool AdvanceAnimations(const SampleTime& aSampleTime) {
+    MutexAutoLock lock(mMapLock);
+    return AdvanceAnimationsInternal(lock, aSampleTime);
+  }
+
+  APZEventResult ReceiveInputEvent(
+      InputData& aEvent,
+      InputBlockCallback&& aCallback = InputBlockCallback()) override {
+    APZEventResult result =
+        APZCTreeManager::ReceiveInputEvent(aEvent, std::move(aCallback));
+    if (aEvent.mInputType == PANGESTURE_INPUT &&
+        // In the APZCTreeManager::ReceiveInputEvent some type of pan gesture
+        // events are marked as `mHandledByAPZ = false` (e.g. with Ctrl key
+        // modifier which causes reflow zoom), in such cases the events will
+        // never be processed by InputQueue so we shouldn't try to invoke
+        // AllowsSwipe() here.
+        aEvent.AsPanGestureInput().mHandledByAPZ &&
+        aEvent.AsPanGestureInput().AllowsSwipe()) {
+      SetBrowserGestureResponse(result.mInputBlockId,
+                                BrowserGestureResponse::NotConsumed);
+    }
+    return result;
+  }
+
  protected:
-  AsyncPanZoomController* NewAPZCInstance(
+  already_AddRefed<AsyncPanZoomController> NewAPZCInstance(
       LayersId aLayersId, GeckoContentController* aController) override;
 
   SampleTime GetFrameTime() override { return mcc->GetSampleTime(); }
 
- private:
   RefPtr<MockContentControllerDelayed> mcc;
 };
 
@@ -262,21 +293,23 @@ class TestAsyncPanZoomController : public AsyncPanZoomController {
         mWaitForMainThread(false),
         mcc(aMcc) {}
 
-  APZEventResult ReceiveInputEvent(const InputData& aEvent) {
+  APZEventResult ReceiveInputEvent(
+      InputData& aEvent,
+      const Maybe<nsTArray<uint32_t>>& aTouchBehaviors = Nothing()) {
     // This is a function whose signature matches exactly the ReceiveInputEvent
     // on APZCTreeManager. This allows us to templates for functions like
     // TouchDown, TouchUp, etc so that we can reuse the code for dispatching
     // events into both APZC and APZCTM.
-    APZEventResult result;
-    result.mStatus = ReceiveInputEvent(aEvent, &result.mInputBlockId);
-    return result;
-  }
-
-  nsEventStatus ReceiveInputEvent(const InputData& aEvent,
-                                  uint64_t* aOutInputBlockId) {
-    return GetInputQueue()->ReceiveInputEvent(
+    APZEventResult result = GetInputQueue()->ReceiveInputEvent(
         this, TargetConfirmationFlags{!mWaitForMainThread}, aEvent,
-        aOutInputBlockId);
+        aTouchBehaviors);
+
+    if (aEvent.mInputType == PANGESTURE_INPUT &&
+        aEvent.AsPanGestureInput().AllowsSwipe()) {
+      GetInputQueue()->SetBrowserGestureResponse(
+          result.mInputBlockId, BrowserGestureResponse::NotConsumed);
+    }
+    return result;
   }
 
   void ContentReceivedInputBlock(uint64_t aInputBlockId, bool aPreventDefault) {
@@ -298,6 +331,11 @@ class TestAsyncPanZoomController : public AsyncPanZoomController {
     Metrics() = metrics;
   }
 
+  void SetScrollMetadata(const ScrollMetadata& aMetadata) {
+    RecursiveMutexAutoLock lock(mRecursiveMutex);
+    mScrollMetadata = aMetadata;
+  }
+
   FrameMetrics& GetFrameMetrics() {
     RecursiveMutexAutoLock lock(mRecursiveMutex);
     return mScrollMetadata.GetMetrics();
@@ -313,6 +351,7 @@ class TestAsyncPanZoomController : public AsyncPanZoomController {
     return mScrollMetadata.GetMetrics();
   }
 
+  using AsyncPanZoomController::GetOverscrollAmount;
   using AsyncPanZoomController::GetVelocityVector;
 
   void AssertStateIsReset() const {
@@ -325,25 +364,77 @@ class TestAsyncPanZoomController : public AsyncPanZoomController {
     EXPECT_EQ(FLING, mState);
   }
 
-  void AssertStateIsSmoothMsdScroll() const {
+  void AssertInSmoothScroll() const {
     RecursiveMutexAutoLock lock(mRecursiveMutex);
-    EXPECT_EQ(SMOOTHMSD_SCROLL, mState);
+    EXPECT_TRUE(InScrollAnimation(ScrollAnimationKind::Smooth));
   }
 
-  void AssertNotAxisLocked() const {
+  void AssertInSmoothMsdScroll() const {
+    RecursiveMutexAutoLock lock(mRecursiveMutex);
+    EXPECT_TRUE(InScrollAnimation(ScrollAnimationKind::SmoothMsd));
+  }
+
+  void AssertStateIsPanningLockedY() {
+    RecursiveMutexAutoLock lock(mRecursiveMutex);
+    EXPECT_EQ(PANNING_LOCKED_Y, mState);
+  }
+
+  void AssertStateIsPanningLockedX() {
+    RecursiveMutexAutoLock lock(mRecursiveMutex);
+    EXPECT_EQ(PANNING_LOCKED_X, mState);
+  }
+
+  void AssertStateIsPanning() {
     RecursiveMutexAutoLock lock(mRecursiveMutex);
     EXPECT_EQ(PANNING, mState);
   }
 
-  void AssertAxisLocked(ScrollDirection aDirection) const {
+  void AssertStateIsPanMomentum() {
     RecursiveMutexAutoLock lock(mRecursiveMutex);
+    EXPECT_EQ(PAN_MOMENTUM, mState);
+  }
+
+  void AssertInWheelScroll() {
+    RecursiveMutexAutoLock lock(mRecursiveMutex);
+    EXPECT_TRUE(InScrollAnimation(ScrollAnimationKind::Wheel));
+  }
+
+  void AssertInKeyboardScroll() {
+    RecursiveMutexAutoLock lock(mRecursiveMutex);
+    EXPECT_TRUE(InScrollAnimation(ScrollAnimationKind::Keyboard));
+  }
+
+  void AssertStateIsAutoscroll() {
+    RecursiveMutexAutoLock lock(mRecursiveMutex);
+    EXPECT_EQ(AUTOSCROLL, mState);
+  }
+
+  void SetAxisLocked(ScrollDirections aDirections, bool aLockValue) {
+    if (aDirections.contains(ScrollDirection::eVertical)) {
+      mY.SetAxisLocked(aLockValue);
+    }
+    if (aDirections.contains(ScrollDirection::eHorizontal)) {
+      mX.SetAxisLocked(aLockValue);
+    }
+  }
+
+  void AssertNotAxisLocked() const {
+    EXPECT_FALSE(mY.IsAxisLocked());
+    EXPECT_FALSE(mX.IsAxisLocked());
+  }
+
+  void AssertAxisLocked(ScrollDirection aDirection) const {
     switch (aDirection) {
       case ScrollDirection::eHorizontal:
-        EXPECT_EQ(PANNING_LOCKED_X, mState);
+        EXPECT_TRUE(mY.IsAxisLocked());
+        EXPECT_FALSE(mX.IsAxisLocked());
         break;
       case ScrollDirection::eVertical:
-        EXPECT_EQ(PANNING_LOCKED_Y, mState);
+        EXPECT_TRUE(mX.IsAxisLocked());
+        EXPECT_FALSE(mY.IsAxisLocked());
         break;
+      default:
+        FAIL() << "input direction must be either vertical or horizontal";
     }
   }
 
@@ -361,10 +452,10 @@ class TestAsyncPanZoomController : public AsyncPanZoomController {
     bool ret = AdvanceAnimations(mcc->GetSampleTime());
     if (aOutTransform) {
       *aOutTransform =
-          GetCurrentAsyncTransform(AsyncPanZoomController::eForHitTesting);
+          GetCurrentAsyncTransform(AsyncPanZoomController::eForEventHandling);
     }
     aScrollOffset =
-        GetCurrentAsyncScrollOffset(AsyncPanZoomController::eForHitTesting);
+        GetCurrentAsyncScrollOffset(AsyncPanZoomController::eForEventHandling);
     return ret;
   }
 
@@ -376,6 +467,14 @@ class TestAsyncPanZoomController : public AsyncPanZoomController {
 
   void SetWaitForMainThread() { mWaitForMainThread = true; }
 
+  bool IsOverscrollAnimationRunning() const {
+    return mState == PanZoomState::OVERSCROLL_ANIMATION;
+  }
+
+  bool IsWheelScrollAnimationRunning() const {
+    return InScrollAnimation(ScrollAnimationKind::Wheel);
+  }
+
  private:
   bool mWaitForMainThread;
   MockContentControllerDelayed* mcc;
@@ -385,7 +484,18 @@ class APZCTesterBase : public ::testing::Test {
  public:
   APZCTesterBase() { mcc = new NiceMock<MockContentControllerDelayed>(); }
 
-  virtual void SetUp() { gfxPlatform::GetPlatform(); }
+  void SetUp() override {
+    gfxPlatform::GetPlatform();
+    // This pref is changed in Pan() without using SCOPED_GFX_PREF
+    // because the modified value needs to be in place until the touch
+    // events are processed, which may not happen until the input queue
+    // is flushed in TearDown(). So, we save and restore its value here.
+    mTouchStartTolerance = StaticPrefs::apz_touch_start_tolerance();
+  }
+
+  void TearDown() override {
+    Preferences::SetFloat("apz.touch_start_tolerance", mTouchStartTolerance);
+  }
 
   enum class PanOptions {
     None = 0,
@@ -400,7 +510,7 @@ class APZCTesterBase : public ::testing::Test {
     NoFling = 0x4
   };
 
-  enum class PinchOptions {
+  enum class PinchFlags {
     None = 0,
     LiftFinger1 = 0x1,
     LiftFinger2 = 0x2,
@@ -414,10 +524,9 @@ class APZCTesterBase : public ::testing::Test {
   };
 
   template <class InputReceiver>
-  void Tap(const RefPtr<InputReceiver>& aTarget, const ScreenIntPoint& aPoint,
-           TimeDuration aTapLength,
-           nsEventStatus (*aOutEventStatuses)[2] = nullptr,
-           uint64_t* aOutInputBlockId = nullptr);
+  APZEventResult Tap(const RefPtr<InputReceiver>& aTarget,
+                     const ScreenIntPoint& aPoint, TimeDuration aTapLength,
+                     nsEventStatus (*aOutEventStatuses)[2] = nullptr);
 
   template <class InputReceiver>
   void TapAndCheckStatus(const RefPtr<InputReceiver>& aTarget,
@@ -464,24 +573,60 @@ class APZCTesterBase : public ::testing::Test {
                                const ScreenIntPoint& aPoint,
                                uint64_t (*aOutInputBlockIds)[2] = nullptr);
 
-  template <class InputReceiver>
-  void PinchWithTouchInput(
-      const RefPtr<InputReceiver>& aTarget, const ScreenIntPoint& aFocus,
-      const ScreenIntPoint& aSecondFocus, float aScale, int& inputId,
-      nsTArray<uint32_t>* aAllowedTouchBehaviors = nullptr,
-      nsEventStatus (*aOutEventStatuses)[4] = nullptr,
-      uint64_t* aOutInputBlockId = nullptr,
-      PinchOptions aOptions = PinchOptions::LiftBothFingers);
+  struct PinchOptions {
+    nsTArray<uint32_t>* mAllowedTouchBehaviors = nullptr;
+    nsEventStatus (*mOutEventStatuses)[4] = nullptr;
+    uint64_t* mOutInputBlockId = nullptr;
+    PinchFlags mFlags = PinchFlags::LiftBothFingers;
+    bool mVertical = false;
+    int* mInputId = nullptr;
+    Maybe<ScreenIntPoint> mSecondFocus;
+    TimeDuration mTimeBetweenTouchEvents = TimeDuration::FromMilliseconds(20);
+
+    // Workaround for https://github.com/llvm/llvm-project/issues/36032
+    PinchOptions() {}
+
+    // Fluent interface
+    PinchOptions& AllowedTouchBehaviors(
+        nsTArray<uint32_t>* aAllowedTouchBehaviors) {
+      mAllowedTouchBehaviors = aAllowedTouchBehaviors;
+      return *this;
+    }
+    PinchOptions& OutEventStatuses(nsEventStatus (*aOutEventStatuses)[4]) {
+      mOutEventStatuses = aOutEventStatuses;
+      return *this;
+    }
+    PinchOptions& OutInputBlockId(uint64_t* aOutInputBlockId) {
+      mOutInputBlockId = aOutInputBlockId;
+      return *this;
+    }
+    PinchOptions& Flags(PinchFlags aFlags) {
+      mFlags = aFlags;
+      return *this;
+    }
+    PinchOptions& Vertical(bool aVertical) {
+      mVertical = aVertical;
+      return *this;
+    }
+    PinchOptions& InputId(int& aInputId) {
+      mInputId = &aInputId;
+      return *this;
+    }
+    PinchOptions& SecondFocus(const ScreenIntPoint& aSecondFocus) {
+      mSecondFocus = Some(aSecondFocus);
+      return *this;
+    }
+    PinchOptions& TimeBetweenTouchEvents(const TimeDuration& aDuration) {
+      mTimeBetweenTouchEvents = aDuration;
+      return *this;
+    }
+  };
 
   // Pinch with one focus point. Zooms in place with no panning
   template <class InputReceiver>
-  void PinchWithTouchInput(
-      const RefPtr<InputReceiver>& aTarget, const ScreenIntPoint& aFocus,
-      float aScale, int& inputId,
-      nsTArray<uint32_t>* aAllowedTouchBehaviors = nullptr,
-      nsEventStatus (*aOutEventStatuses)[4] = nullptr,
-      uint64_t* aOutInputBlockId = nullptr,
-      PinchOptions aOptions = PinchOptions::LiftBothFingers);
+  void PinchWithTouchInput(const RefPtr<InputReceiver>& aTarget,
+                           const ScreenIntPoint& aFocus, float aScale,
+                           PinchOptions aOptions = PinchOptions());
 
   template <class InputReceiver>
   void PinchWithTouchInputAndCheckStatus(
@@ -503,36 +648,36 @@ class APZCTesterBase : public ::testing::Test {
 
  protected:
   RefPtr<MockContentControllerDelayed> mcc;
+
+ private:
+  float mTouchStartTolerance;
 };
 
 MOZ_MAKE_ENUM_CLASS_BITWISE_OPERATORS(APZCTesterBase::PanOptions)
-MOZ_MAKE_ENUM_CLASS_BITWISE_OPERATORS(APZCTesterBase::PinchOptions)
+MOZ_MAKE_ENUM_CLASS_BITWISE_OPERATORS(APZCTesterBase::PinchFlags)
 
 template <class InputReceiver>
-void APZCTesterBase::Tap(const RefPtr<InputReceiver>& aTarget,
-                         const ScreenIntPoint& aPoint, TimeDuration aTapLength,
-                         nsEventStatus (*aOutEventStatuses)[2],
-                         uint64_t* aOutInputBlockId) {
-  APZEventResult result = TouchDown(aTarget, aPoint, mcc->Time());
+APZEventResult APZCTesterBase::Tap(const RefPtr<InputReceiver>& aTarget,
+                                   const ScreenIntPoint& aPoint,
+                                   TimeDuration aTapLength,
+                                   nsEventStatus (*aOutEventStatuses)[2]) {
+  APZEventResult touchDownResult = TouchDown(aTarget, aPoint, mcc->Time());
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[0] = result.mStatus;
-  }
-  if (aOutInputBlockId) {
-    *aOutInputBlockId = result.mInputBlockId;
+    (*aOutEventStatuses)[0] = touchDownResult.GetStatus();
   }
   mcc->AdvanceBy(aTapLength);
 
   // If touch-action is enabled then simulate the allowed touch behaviour
   // notification that the main thread is supposed to deliver.
-  if (StaticPrefs::layout_css_touch_action_enabled() &&
-      result.mStatus != nsEventStatus_eConsumeNoDefault) {
-    SetDefaultAllowedTouchBehavior(aTarget, result.mInputBlockId);
+  if (touchDownResult.GetStatus() != nsEventStatus_eConsumeNoDefault) {
+    SetDefaultAllowedTouchBehavior(aTarget, touchDownResult.mInputBlockId);
   }
 
-  result.mStatus = TouchUp(aTarget, aPoint, mcc->Time());
+  APZEventResult touchUpResult = TouchUp(aTarget, aPoint, mcc->Time());
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[1] = result.mStatus;
+    (*aOutEventStatuses)[1] = touchUpResult.GetStatus();
   }
+  return touchDownResult;
 }
 
 template <class InputReceiver>
@@ -552,11 +697,13 @@ void APZCTesterBase::Pan(const RefPtr<InputReceiver>& aTarget,
                          nsTArray<uint32_t>* aAllowedTouchBehaviors,
                          nsEventStatus (*aOutEventStatuses)[4],
                          uint64_t* aOutInputBlockId) {
-  // Reduce the touch start and move tolerance to a tiny value.
+  // Reduce the move tolerance to a tiny value.
   // We can't use a scoped pref because this value might be read at some later
   // time when the events are actually processed, rather than when we deliver
   // them.
-  Preferences::SetFloat("apz.touch_start_tolerance", 1.0f / 1000.0f);
+  const float touchStartTolerance = 0.1f;
+  const float panThreshold = touchStartTolerance * aTarget->GetDPI();
+  Preferences::SetFloat("apz.touch_start_tolerance", touchStartTolerance);
   Preferences::SetFloat("apz.touch_move_tolerance", 0.0f);
   int overcomeTouchToleranceX = 0;
   int overcomeTouchToleranceY = 0;
@@ -564,11 +711,17 @@ void APZCTesterBase::Pan(const RefPtr<InputReceiver>& aTarget,
     // Have the direction of the adjustment to overcome the touch tolerance
     // match the direction of the entire gesture, otherwise we run into
     // trouble such as accidentally activating the axis lock.
-    if (aTouchStart.x != aTouchEnd.x) {
-      overcomeTouchToleranceX = 1;
-    }
-    if (aTouchStart.y != aTouchEnd.y) {
-      overcomeTouchToleranceY = 1;
+    if (aTouchStart.x != aTouchEnd.x && aTouchStart.y != aTouchEnd.y) {
+      // Tests that need to avoid rounding error here can arrange for
+      // panThreshold to be 10 (by setting the DPI to 100), which makes sure
+      // that these are the legs in a Pythagorean triple where panThreshold is
+      // the hypotenuse. Watch out for changes of APZCPinchTester::mDPI.
+      overcomeTouchToleranceX = panThreshold / 10 * 6;
+      overcomeTouchToleranceY = panThreshold / 10 * 8;
+    } else if (aTouchStart.x != aTouchEnd.x) {
+      overcomeTouchToleranceX = panThreshold;
+    } else if (aTouchStart.y != aTouchEnd.y) {
+      overcomeTouchToleranceY = panThreshold;
     }
   }
 
@@ -592,25 +745,25 @@ void APZCTesterBase::Pan(const RefPtr<InputReceiver>& aTarget,
     *aOutInputBlockId = result.mInputBlockId;
   }
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[0] = result.mStatus;
+    (*aOutEventStatuses)[0] = result.GetStatus();
   }
 
   mcc->AdvanceBy(TIME_BETWEEN_TOUCH_EVENT);
 
   // Allowed touch behaviours must be set after sending touch-start.
-  if (result.mStatus != nsEventStatus_eConsumeNoDefault) {
+  if (result.GetStatus() != nsEventStatus_eConsumeNoDefault) {
     if (aAllowedTouchBehaviors) {
       EXPECT_EQ(1UL, aAllowedTouchBehaviors->Length());
       aTarget->SetAllowedTouchBehavior(*aOutInputBlockId,
                                        *aAllowedTouchBehaviors);
-    } else if (StaticPrefs::layout_css_touch_action_enabled()) {
+    } else {
       SetDefaultAllowedTouchBehavior(aTarget, *aOutInputBlockId);
     }
   }
 
-  result.mStatus = TouchMove(aTarget, aTouchStart, mcc->Time());
+  result = TouchMove(aTarget, aTouchStart, mcc->Time());
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[1] = result.mStatus;
+    (*aOutEventStatuses)[1] = result.GetStatus();
   }
 
   mcc->AdvanceBy(TIME_BETWEEN_TOUCH_EVENT);
@@ -619,25 +772,25 @@ void APZCTesterBase::Pan(const RefPtr<InputReceiver>& aTarget,
   auto stepVector = (aTouchEnd - aTouchStart) / numSteps;
   for (int k = 1; k < numSteps; k++) {
     auto stepPoint = aTouchStart + stepVector * k;
-    Unused << TouchMove(aTarget, stepPoint, mcc->Time());
+    (void)TouchMove(aTarget, stepPoint, mcc->Time());
 
     mcc->AdvanceBy(TIME_BETWEEN_TOUCH_EVENT);
   }
 
-  result.mStatus = TouchMove(aTarget, aTouchEnd, mcc->Time());
+  result = TouchMove(aTarget, aTouchEnd, mcc->Time());
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[2] = result.mStatus;
+    (*aOutEventStatuses)[2] = result.GetStatus();
   }
 
   mcc->AdvanceBy(TIME_BETWEEN_TOUCH_EVENT);
 
   if (!(aOptions & PanOptions::KeepFingerDown)) {
-    result.mStatus = TouchUp(aTarget, aTouchEnd, mcc->Time());
+    result = TouchUp(aTarget, aTouchEnd, mcc->Time());
   } else {
-    result.mStatus = nsEventStatus_eIgnore;
+    result.SetStatusAsIgnore();
   }
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[3] = result.mStatus;
+    (*aOutEventStatuses)[3] = result.GetStatus();
   }
 
   if ((aOptions & PanOptions::NoFling)) {
@@ -687,7 +840,7 @@ void APZCTesterBase::DoubleTap(const RefPtr<InputReceiver>& aTarget,
                                uint64_t (*aOutInputBlockIds)[2]) {
   APZEventResult result = TouchDown(aTarget, aPoint, mcc->Time());
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[0] = result.mStatus;
+    (*aOutEventStatuses)[0] = result.GetStatus();
   }
   if (aOutInputBlockIds) {
     (*aOutInputBlockIds)[0] = result.mInputBlockId;
@@ -696,33 +849,31 @@ void APZCTesterBase::DoubleTap(const RefPtr<InputReceiver>& aTarget,
 
   // If touch-action is enabled then simulate the allowed touch behaviour
   // notification that the main thread is supposed to deliver.
-  if (StaticPrefs::layout_css_touch_action_enabled() &&
-      result.mStatus != nsEventStatus_eConsumeNoDefault) {
+  if (result.GetStatus() != nsEventStatus_eConsumeNoDefault) {
     SetDefaultAllowedTouchBehavior(aTarget, result.mInputBlockId);
   }
 
-  result.mStatus = TouchUp(aTarget, aPoint, mcc->Time());
+  result = TouchUp(aTarget, aPoint, mcc->Time());
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[1] = result.mStatus;
+    (*aOutEventStatuses)[1] = result.GetStatus();
   }
   mcc->AdvanceByMillis(10);
   result = TouchDown(aTarget, aPoint, mcc->Time());
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[2] = result.mStatus;
+    (*aOutEventStatuses)[2] = result.GetStatus();
   }
   if (aOutInputBlockIds) {
     (*aOutInputBlockIds)[1] = result.mInputBlockId;
   }
   mcc->AdvanceByMillis(10);
 
-  if (StaticPrefs::layout_css_touch_action_enabled() &&
-      result.mStatus != nsEventStatus_eConsumeNoDefault) {
+  if (result.GetStatus() != nsEventStatus_eConsumeNoDefault) {
     SetDefaultAllowedTouchBehavior(aTarget, result.mInputBlockId);
   }
 
-  result.mStatus = TouchUp(aTarget, aPoint, mcc->Time());
+  result = TouchUp(aTarget, aPoint, mcc->Time());
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[3] = result.mStatus;
+    (*aOutEventStatuses)[3] = result.GetStatus();
   }
 }
 
@@ -739,61 +890,61 @@ void APZCTesterBase::DoubleTapAndCheckStatus(
 }
 
 template <class InputReceiver>
-void APZCTesterBase::PinchWithTouchInput(
-    const RefPtr<InputReceiver>& aTarget, const ScreenIntPoint& aFocus,
-    float aScale, int& inputId, nsTArray<uint32_t>* aAllowedTouchBehaviors,
-    nsEventStatus (*aOutEventStatuses)[4], uint64_t* aOutInputBlockId,
-    PinchOptions aOptions) {
-  // Perform a pinch gesture with the same start & end focus point
-  PinchWithTouchInput(aTarget, aFocus, aFocus, aScale, inputId,
-                      aAllowedTouchBehaviors, aOutEventStatuses,
-                      aOutInputBlockId, aOptions);
-}
-
-template <class InputReceiver>
-void APZCTesterBase::PinchWithTouchInput(
-    const RefPtr<InputReceiver>& aTarget, const ScreenIntPoint& aFocus,
-    const ScreenIntPoint& aSecondFocus, float aScale, int& inputId,
-    nsTArray<uint32_t>* aAllowedTouchBehaviors,
-    nsEventStatus (*aOutEventStatuses)[4], uint64_t* aOutInputBlockId,
-    PinchOptions aOptions) {
+void APZCTesterBase::PinchWithTouchInput(const RefPtr<InputReceiver>& aTarget,
+                                         const ScreenIntPoint& aFocus,
+                                         float aScale, PinchOptions aOptions) {
   // Having pinch coordinates in float type may cause problems with
   // high-precision scale values since SingleTouchData accepts integer value.
   // But for trivial tests it should be ok.
-  float pinchLength = 100.0;
-  float pinchLengthScaled = pinchLength * aScale;
+  const float pinchLength = 100.0;
+  const float pinchLengthScaled = pinchLength * aScale;
+
+  const float pinchLengthX = aOptions.mVertical ? 0 : pinchLength;
+  const float pinchLengthScaledX = aOptions.mVertical ? 0 : pinchLengthScaled;
+  const float pinchLengthY = aOptions.mVertical ? pinchLength : 0;
+  const float pinchLengthScaledY = aOptions.mVertical ? pinchLengthScaled : 0;
 
   // Even if the caller doesn't care about the block id, we need it to set the
   // allowed touch behaviour below, so make sure aOutInputBlockId is non-null.
   uint64_t blockId;
-  if (!aOutInputBlockId) {
-    aOutInputBlockId = &blockId;
+  if (!aOptions.mOutInputBlockId) {
+    aOptions.mOutInputBlockId = &blockId;
   }
 
-  const TimeDuration TIME_BETWEEN_TOUCH_EVENT =
-      TimeDuration::FromMilliseconds(20);
+  int inputId = aOptions.mInputId ? *aOptions.mInputId : 0;
+
+  // If a second focus point is not specified in the pinch options, use the
+  // same focus point throughout the gesture.
+  ScreenIntPoint secondFocus =
+      aOptions.mSecondFocus.isSome() ? *aOptions.mSecondFocus : aFocus;
 
   MultiTouchInput mtiStart =
       MultiTouchInput(MultiTouchInput::MULTITOUCH_START, 0, mcc->Time(), 0);
   mtiStart.mTouches.AppendElement(CreateSingleTouchData(inputId, aFocus));
   mtiStart.mTouches.AppendElement(CreateSingleTouchData(inputId + 1, aFocus));
-  nsEventStatus status = aTarget->ReceiveInputEvent(mtiStart, aOutInputBlockId);
-  if (aOutEventStatuses) {
-    (*aOutEventStatuses)[0] = status;
+  APZEventResult result;
+  result = aTarget->ReceiveInputEvent(mtiStart);
+  if (aOptions.mOutInputBlockId) {
+    *aOptions.mOutInputBlockId = result.mInputBlockId;
+  }
+  if (aOptions.mOutEventStatuses) {
+    (*aOptions.mOutEventStatuses)[0] = result.GetStatus();
   }
 
-  mcc->AdvanceBy(TIME_BETWEEN_TOUCH_EVENT);
-
-  if (aAllowedTouchBehaviors) {
-    EXPECT_EQ(2UL, aAllowedTouchBehaviors->Length());
-    aTarget->SetAllowedTouchBehavior(*aOutInputBlockId,
-                                     *aAllowedTouchBehaviors);
-  } else if (StaticPrefs::layout_css_touch_action_enabled()) {
-    SetDefaultAllowedTouchBehavior(aTarget, *aOutInputBlockId, 2);
+  if (aOptions.mAllowedTouchBehaviors) {
+    EXPECT_EQ(2UL, aOptions.mAllowedTouchBehaviors->Length());
+    aTarget->SetAllowedTouchBehavior(*aOptions.mOutInputBlockId,
+                                     *aOptions.mAllowedTouchBehaviors);
+  } else {
+    SetDefaultAllowedTouchBehavior(aTarget, *aOptions.mOutInputBlockId, 2);
   }
 
-  ScreenIntPoint pinchStartPoint1(aFocus.x - int32_t(pinchLength), aFocus.y);
-  ScreenIntPoint pinchStartPoint2(aFocus.x + int32_t(pinchLength), aFocus.y);
+  mcc->AdvanceBy(aOptions.mTimeBetweenTouchEvents);
+
+  ScreenIntPoint pinchStartPoint1(aFocus.x - int32_t(pinchLengthX),
+                                  aFocus.y - int32_t(pinchLengthY));
+  ScreenIntPoint pinchStartPoint2(aFocus.x + int32_t(pinchLengthX),
+                                  aFocus.y + int32_t(pinchLengthY));
 
   MultiTouchInput mtiMove1 =
       MultiTouchInput(MultiTouchInput::MULTITOUCH_MOVE, 0, mcc->Time(), 0);
@@ -801,37 +952,37 @@ void APZCTesterBase::PinchWithTouchInput(
       CreateSingleTouchData(inputId, pinchStartPoint1));
   mtiMove1.mTouches.AppendElement(
       CreateSingleTouchData(inputId + 1, pinchStartPoint2));
-  status = aTarget->ReceiveInputEvent(mtiMove1, nullptr);
-  if (aOutEventStatuses) {
-    (*aOutEventStatuses)[1] = status;
+  result = aTarget->ReceiveInputEvent(mtiMove1);
+  if (aOptions.mOutEventStatuses) {
+    (*aOptions.mOutEventStatuses)[1] = result.GetStatus();
   }
 
-  mcc->AdvanceBy(TIME_BETWEEN_TOUCH_EVENT);
+  mcc->AdvanceBy(aOptions.mTimeBetweenTouchEvents);
 
   // Pinch instantly but move in steps.
   const int numSteps = 3;
-  auto stepVector = (aSecondFocus - aFocus) / numSteps;
+  auto stepVector = (secondFocus - aFocus) / numSteps;
   for (int k = 1; k < numSteps; k++) {
     ScreenIntPoint stepFocus = aFocus + stepVector * k;
-    ScreenIntPoint stepPoint1(stepFocus.x - int32_t(pinchLengthScaled),
-                              stepFocus.y);
-    ScreenIntPoint stepPoint2(stepFocus.x + int32_t(pinchLengthScaled),
-                              stepFocus.y);
+    ScreenIntPoint stepPoint1(stepFocus.x - int32_t(pinchLengthScaledX),
+                              stepFocus.y - int32_t(pinchLengthScaledY));
+    ScreenIntPoint stepPoint2(stepFocus.x + int32_t(pinchLengthScaledX),
+                              stepFocus.y + int32_t(pinchLengthScaledY));
     MultiTouchInput mtiMoveStep =
         MultiTouchInput(MultiTouchInput::MULTITOUCH_MOVE, 0, mcc->Time(), 0);
     mtiMoveStep.mTouches.AppendElement(
         CreateSingleTouchData(inputId, stepPoint1));
     mtiMoveStep.mTouches.AppendElement(
         CreateSingleTouchData(inputId + 1, stepPoint2));
-    Unused << aTarget->ReceiveInputEvent(mtiMoveStep, nullptr);
+    (void)aTarget->ReceiveInputEvent(mtiMoveStep);
 
-    mcc->AdvanceBy(TIME_BETWEEN_TOUCH_EVENT);
+    mcc->AdvanceBy(aOptions.mTimeBetweenTouchEvents);
   }
 
-  ScreenIntPoint pinchEndPoint1(aSecondFocus.x - int32_t(pinchLengthScaled),
-                                aSecondFocus.y);
-  ScreenIntPoint pinchEndPoint2(aSecondFocus.x + int32_t(pinchLengthScaled),
-                                aSecondFocus.y);
+  ScreenIntPoint pinchEndPoint1(secondFocus.x - int32_t(pinchLengthScaledX),
+                                secondFocus.y - int32_t(pinchLengthScaledY));
+  ScreenIntPoint pinchEndPoint2(secondFocus.x + int32_t(pinchLengthScaledX),
+                                secondFocus.y + int32_t(pinchLengthScaledY));
 
   MultiTouchInput mtiMove2 =
       MultiTouchInput(MultiTouchInput::MULTITOUCH_MOVE, 0, mcc->Time(), 0);
@@ -839,31 +990,35 @@ void APZCTesterBase::PinchWithTouchInput(
       CreateSingleTouchData(inputId, pinchEndPoint1));
   mtiMove2.mTouches.AppendElement(
       CreateSingleTouchData(inputId + 1, pinchEndPoint2));
-  status = aTarget->ReceiveInputEvent(mtiMove2, nullptr);
-  if (aOutEventStatuses) {
-    (*aOutEventStatuses)[2] = status;
+  result = aTarget->ReceiveInputEvent(mtiMove2);
+  if (aOptions.mOutEventStatuses) {
+    (*aOptions.mOutEventStatuses)[2] = result.GetStatus();
   }
 
-  if (aOptions & (PinchOptions::LiftFinger1 | PinchOptions::LiftFinger2)) {
-    mcc->AdvanceBy(TIME_BETWEEN_TOUCH_EVENT);
+  if (aOptions.mFlags & (PinchFlags::LiftFinger1 | PinchFlags::LiftFinger2)) {
+    mcc->AdvanceBy(aOptions.mTimeBetweenTouchEvents);
 
     MultiTouchInput mtiEnd =
         MultiTouchInput(MultiTouchInput::MULTITOUCH_END, 0, mcc->Time(), 0);
-    if (aOptions & PinchOptions::LiftFinger1) {
+    if (aOptions.mFlags & PinchFlags::LiftFinger1) {
       mtiEnd.mTouches.AppendElement(
           CreateSingleTouchData(inputId, pinchEndPoint1));
     }
-    if (aOptions & PinchOptions::LiftFinger2) {
+    if (aOptions.mFlags & PinchFlags::LiftFinger2) {
       mtiEnd.mTouches.AppendElement(
           CreateSingleTouchData(inputId + 1, pinchEndPoint2));
     }
-    status = aTarget->ReceiveInputEvent(mtiEnd, nullptr);
-    if (aOutEventStatuses) {
-      (*aOutEventStatuses)[3] = status;
+    result = aTarget->ReceiveInputEvent(mtiEnd);
+    if (aOptions.mOutEventStatuses) {
+      (*aOptions.mOutEventStatuses)[3] = result.GetStatus();
     }
   }
 
   inputId += 2;
+
+  if (aOptions.mInputId) {
+    *aOptions.mInputId = inputId;
+  }
 }
 
 template <class InputReceiver>
@@ -872,8 +1027,11 @@ void APZCTesterBase::PinchWithTouchInputAndCheckStatus(
     float aScale, int& inputId, bool aShouldTriggerPinch,
     nsTArray<uint32_t>* aAllowedTouchBehaviors) {
   nsEventStatus statuses[4];  // down, move, move, up
-  PinchWithTouchInput(aTarget, aFocus, aScale, inputId, aAllowedTouchBehaviors,
-                      &statuses);
+  PinchWithTouchInput(aTarget, aFocus, aScale,
+                      PinchOptions()
+                          .AllowedTouchBehaviors(aAllowedTouchBehaviors)
+                          .OutEventStatuses(&statuses)
+                          .InputId(inputId));
 
   nsEventStatus expectedMoveStatus = aShouldTriggerPinch
                                          ? nsEventStatus_eConsumeDoDefault
@@ -891,30 +1049,29 @@ void APZCTesterBase::PinchWithPinchInput(
   const TimeDuration TIME_BETWEEN_PINCH_INPUT =
       TimeDuration::FromMilliseconds(50);
 
-  nsEventStatus actualStatus = aTarget->ReceiveInputEvent(
-      CreatePinchGestureInput(PinchGestureInput::PINCHGESTURE_START, aFocus,
-                              10.0, 10.0, mcc->Time()),
-      nullptr);
+  auto event = CreatePinchGestureInput(PinchGestureInput::PINCHGESTURE_START,
+                                       aFocus, 10.0, 10.0, mcc->Time());
+  APZEventResult actual = aTarget->ReceiveInputEvent(event);
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[0] = actualStatus;
+    (*aOutEventStatuses)[0] = actual.GetStatus();
   }
   mcc->AdvanceBy(TIME_BETWEEN_PINCH_INPUT);
 
-  actualStatus = aTarget->ReceiveInputEvent(
+  event =
       CreatePinchGestureInput(PinchGestureInput::PINCHGESTURE_SCALE,
-                              aSecondFocus, 10.0 * aScale, 10.0, mcc->Time()),
-      nullptr);
+                              aSecondFocus, 10.0 * aScale, 10.0, mcc->Time());
+  actual = aTarget->ReceiveInputEvent(event);
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[1] = actualStatus;
+    (*aOutEventStatuses)[1] = actual.GetStatus();
   }
   mcc->AdvanceBy(TIME_BETWEEN_PINCH_INPUT);
 
-  actualStatus = aTarget->ReceiveInputEvent(
+  event =
       CreatePinchGestureInput(PinchGestureInput::PINCHGESTURE_END, aSecondFocus,
-                              10.0 * aScale, 10.0 * aScale, mcc->Time()),
-      nullptr);
+                              10.0 * aScale, 10.0 * aScale, mcc->Time());
+  actual = aTarget->ReceiveInputEvent(event);
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[2] = actualStatus;
+    (*aOutEventStatuses)[2] = actual.GetStatus();
   }
 }
 
@@ -932,20 +1089,11 @@ void APZCTesterBase::PinchWithPinchInputAndCheckStatus(
   EXPECT_EQ(expectedStatus, statuses[1]);
 }
 
-AsyncPanZoomController* TestAPZCTreeManager::NewAPZCInstance(
-    LayersId aLayersId, GeckoContentController* aController) {
-  MockContentControllerDelayed* mcc =
-      static_cast<MockContentControllerDelayed*>(aController);
-  return new TestAsyncPanZoomController(
-      aLayersId, mcc, this, AsyncPanZoomController::USE_GESTURE_DETECTOR);
-}
-
 inline FrameMetrics TestFrameMetrics() {
   FrameMetrics fm;
 
   fm.SetDisplayPort(CSSRect(0, 0, 10, 10));
   fm.SetCompositionBounds(ParentLayerRect(0, 0, 10, 10));
-  fm.SetCriticalDisplayPort(CSSRect(0, 0, 10, 10));
   fm.SetScrollableRect(CSSRect(0, 0, 100, 100));
 
   return fm;
