@@ -15,8 +15,11 @@ macro_rules! debug {
 }
 
 #[repr(C)]
-struct LockedTexture { _private: [u8; 0] }
+struct LockedTexture {
+    _private: [u8; 0],
+}
 
+#[allow(dead_code)]
 extern "C" {
     fn ActiveTexture(texture: GLenum);
     fn BindTexture(target: GLenum, texture: GLuint);
@@ -62,19 +65,7 @@ extern "C" {
         level: GLint,
     );
     fn CheckFramebufferStatus(target: GLenum) -> GLenum;
-    fn InvalidateFramebuffer(
-        target: GLenum,
-        num_attachments: GLsizei,
-        attachments: *const GLenum,
-    );
-    fn TexStorage3D(
-        target: GLenum,
-        levels: GLint,
-        internal_format: GLenum,
-        width: GLsizei,
-        height: GLsizei,
-        depth: GLsizei,
-    );
+    fn InvalidateFramebuffer(target: GLenum, num_attachments: GLsizei, attachments: *const GLenum);
     fn TexImage2D(
         target: GLenum,
         level: GLint,
@@ -86,18 +77,6 @@ extern "C" {
         ty: GLenum,
         data: *const c_void,
     );
-    fn TexImage3D(
-        target: GLenum,
-        level: GLint,
-        internal_format: GLint,
-        width: GLsizei,
-        height: GLsizei,
-        depth: GLsizei,
-        border: GLint,
-        format: GLenum,
-        ty: GLenum,
-        data: *const c_void,
-    );
     fn TexSubImage2D(
         target: GLenum,
         level: GLint,
@@ -109,19 +88,6 @@ extern "C" {
         ty: GLenum,
         data: *const c_void,
     );
-    fn TexSubImage3D(
-        target: GLenum,
-        level: GLint,
-        xoffset: GLint,
-        yoffset: GLint,
-        zoffset: GLint,
-        width: GLsizei,
-        height: GLsizei,
-        depth: GLsizei,
-        format: GLenum,
-        ty: GLenum,
-        data: *const c_void,
-    );
     fn GenerateMipmap(target: GLenum);
     fn GetUniformLocation(program: GLuint, name: *const GLchar) -> GLint;
     fn BindAttribLocation(program: GLuint, index: GLuint, name: *const GLchar);
@@ -166,13 +132,6 @@ extern "C" {
     fn GetLinkStatus(program: GLuint) -> GLint;
     fn UseProgram(program: GLuint);
     fn SetViewport(x: GLint, y: GLint, width: GLsizei, height: GLsizei);
-    fn FramebufferTextureLayer(
-        target: GLenum,
-        attachment: GLenum,
-        texture: GLuint,
-        level: GLint,
-        layer: GLint,
-    );
     fn FramebufferRenderbuffer(
         target: GLenum,
         attachment: GLenum,
@@ -186,6 +145,31 @@ extern "C" {
     fn ClearColor(r: GLfloat, g: GLfloat, b: GLfloat, a: GLfloat);
     fn ClearDepth(depth: GLdouble);
     fn Clear(mask: GLbitfield);
+    fn ClearTexSubImage(
+        target: GLenum,
+        level: GLint,
+        xoffset: GLint,
+        yoffset: GLint,
+        zoffset: GLint,
+        width: GLsizei,
+        height: GLsizei,
+        depth: GLsizei,
+        format: GLenum,
+        ty: GLenum,
+        data: *const c_void,
+    );
+    fn ClearTexImage(target: GLenum, level: GLint, format: GLenum, ty: GLenum, data: *const c_void);
+    fn ClearColorRect(
+        fbo: GLuint,
+        xoffset: GLint,
+        yoffset: GLint,
+        width: GLsizei,
+        height: GLsizei,
+        r: GLfloat,
+        g: GLfloat,
+        b: GLfloat,
+        a: GLfloat,
+    );
     fn PixelStorei(name: GLenum, param: GLint);
     fn ReadPixels(
         x: GLint,
@@ -226,17 +210,6 @@ extern "C" {
         width: GLsizei,
         height: GLsizei,
     );
-    fn CopyTexSubImage3D(
-        target: GLenum,
-        level: GLint,
-        xoffset: GLint,
-        yoffset: GLint,
-        zoffset: GLint,
-        x: GLint,
-        y: GLint,
-        width: GLsizei,
-        height: GLsizei,
-    );
     fn BlitFramebuffer(
         src_x0: GLint,
         src_y0: GLint,
@@ -269,6 +242,7 @@ extern "C" {
         height: *mut i32,
         stride: *mut i32,
     ) -> *mut c_void;
+    fn ResolveFramebuffer(fbo: GLuint);
     fn SetTextureBuffer(
         tex: GLuint,
         internal_format: GLenum,
@@ -291,7 +265,7 @@ extern "C" {
     fn LockFramebuffer(fbo: GLuint) -> *mut LockedTexture;
     fn LockTexture(tex: GLuint) -> *mut LockedTexture;
     fn LockResource(resource: *mut LockedTexture);
-    fn UnlockResource(resource: *mut LockedTexture);
+    fn UnlockResource(resource: *mut LockedTexture) -> i32;
     fn GetResourceBuffer(
         resource: *mut LockedTexture,
         width: *mut i32,
@@ -310,7 +284,8 @@ extern "C" {
         dst_width: GLsizei,
         dst_height: GLsizei,
         opaque: GLboolean,
-        flip: GLboolean,
+        flip_x: GLboolean,
+        flip_y: GLboolean,
         filter: GLenum,
         clip_x: GLint,
         clip_y: GLint,
@@ -322,7 +297,7 @@ extern "C" {
         locked_y: *mut LockedTexture,
         locked_u: *mut LockedTexture,
         locked_v: *mut LockedTexture,
-        color_space: YUVColorSpace,
+        color_space: YuvRangedColorSpace,
         color_depth: GLuint,
         src_x: GLint,
         src_y: GLint,
@@ -332,16 +307,22 @@ extern "C" {
         dst_y: GLint,
         dst_width: GLsizei,
         dst_height: GLsizei,
-        flip: GLboolean,
+        flip_x: GLboolean,
+        flip_y: GLboolean,
         clip_x: GLint,
         clip_y: GLint,
         clip_width: GLsizei,
         clip_height: GLsizei,
     );
+    fn ApplyMask(
+        locked_dst: *mut LockedTexture,
+        locked_mask: *mut LockedTexture,
+    );
     fn CreateContext() -> *mut c_void;
     fn ReferenceContext(ctx: *mut c_void);
     fn DestroyContext(ctx: *mut c_void);
     fn MakeCurrent(ctx: *mut c_void);
+    fn ReportMemory(ctx: *mut c_void, size_of_op: unsafe extern "C" fn(ptr: *const c_void) -> usize) -> usize;
 }
 
 #[derive(Clone, Copy)]
@@ -389,11 +370,40 @@ impl Context {
             let mut width: i32 = 0;
             let mut height: i32 = 0;
             let mut stride: i32 = 0;
-            let data_ptr = GetColorBuffer(fbo, flush as GLboolean, &mut width, &mut height, &mut stride);
+            let data_ptr = GetColorBuffer(
+                fbo,
+                flush as GLboolean,
+                &mut width,
+                &mut height,
+                &mut stride,
+            );
             (data_ptr, width, height, stride)
         }
     }
 
+    pub fn resolve_framebuffer(&self, fbo: GLuint) {
+        unsafe {
+            ResolveFramebuffer(fbo);
+        }
+    }
+
+    pub fn clear_color_rect(
+        &self,
+        fbo: GLuint,
+        xoffset: GLint,
+        yoffset: GLint,
+        width: GLsizei,
+        height: GLsizei,
+        r: f32,
+        g: f32,
+        b: f32,
+        a: f32,
+    ) {
+        unsafe {
+            ClearColorRect(fbo, xoffset, yoffset, width, height, r, g, b, a);
+        }
+    }
+
     pub fn set_texture_buffer(
         &self,
         tex: GLuint,
@@ -428,7 +438,7 @@ impl Context {
     pub fn lock_framebuffer(&self, fbo: GLuint) -> Option<LockedResource> {
         unsafe {
             let resource = LockFramebuffer(fbo);
-            if resource != ptr::null_mut() {
+            if !resource.is_null() {
                 Some(LockedResource(resource))
             } else {
                 None
@@ -439,13 +449,17 @@ impl Context {
     pub fn lock_texture(&self, tex: GLuint) -> Option<LockedResource> {
         unsafe {
             let resource = LockTexture(tex);
-            if resource != ptr::null_mut() {
+            if !resource.is_null() {
                 Some(LockedResource(resource))
             } else {
                 None
             }
         }
     }
+
+    pub fn report_memory(&self, size_of_op: unsafe extern "C" fn(ptr: *const c_void) -> usize) -> usize {
+        unsafe { ReportMemory(self.0, size_of_op) }
+    }
 }
 
 impl From<*mut c_void> for Context {
@@ -484,7 +498,7 @@ fn calculate_length(width: GLsizei, height: GLsizei, format: GLenum, pixel_type:
         _ => panic!("unsupported pixel_type for read_pixels: {:?}", pixel_type),
     };
 
-    return (width * height * colors * depth) as usize;
+    (width * height * colors * depth) as usize
 }
 
 impl Gl for Context {
@@ -553,10 +567,10 @@ impl Gl for Context {
         //panic!();
         for s in strings {
             let u = str::from_utf8(s).unwrap();
-            const PREFIX: &'static str = "// shader: ";
+            const PREFIX: &str = "// shader: ";
             if let Some(start) = u.find(PREFIX) {
-                if let Some(end) = u[start ..].find('\n') {
-                    let name = u[start + PREFIX.len() .. start + end].trim();
+                if let Some(end) = u[start..].find('\n') {
+                    let name = u[start + PREFIX.len()..start + end].trim();
                     debug!("shader name: {}", name);
                     unsafe {
                         let c_string = CString::new(name).unwrap();
@@ -954,6 +968,16 @@ impl Gl for Context {
         }
     }
 
+    fn bind_vertex_buffer(
+        &self,
+        binding_index: GLuint,
+        buffer: GLuint,
+        offset: GLintptr,
+        stride: GLint,
+    ) {
+        unimplemented!("Not supported by SWGL");
+    }
+
     fn bind_texture(&self, target: GLenum, texture: GLuint) {
         //panic!();
         unsafe {
@@ -1025,7 +1049,6 @@ impl Gl for Context {
         panic!();
     }
 
-    // FIXME: Does not verify buffer size -- unsafe!
     fn tex_image_3d(
         &self,
         target: GLenum,
@@ -1039,24 +1062,7 @@ impl Gl for Context {
         ty: GLenum,
         opt_data: Option<&[u8]>,
     ) {
-        unsafe {
-            let pdata = match opt_data {
-                Some(data) => data.as_ptr() as *const GLvoid,
-                None => ptr::null(),
-            };
-            TexImage3D(
-                target,
-                level,
-                internal_format,
-                width,
-                height,
-                depth,
-                border,
-                format,
-                ty,
-                pdata,
-            );
-        }
+        panic!();
     }
 
     fn copy_tex_image_2d(
@@ -1101,11 +1107,7 @@ impl Gl for Context {
         width: GLsizei,
         height: GLsizei,
     ) {
-        unsafe {
-            CopyTexSubImage3D(
-                target, level, xoffset, yoffset, zoffset, x, y, width, height,
-            );
-        }
+        panic!();
     }
 
     fn tex_sub_image_2d(
@@ -1187,22 +1189,7 @@ impl Gl for Context {
         data: &[u8],
     ) {
         debug!("tex_sub_image_3d");
-        //panic!();
-        unsafe {
-            TexSubImage3D(
-                target,
-                level,
-                xoffset,
-                yoffset,
-                zoffset,
-                width,
-                height,
-                depth,
-                format,
-                ty,
-                data.as_ptr() as *const c_void,
-            );
-        }
+        panic!();
     }
 
     fn tex_sub_image_3d_pbo(
@@ -1219,21 +1206,7 @@ impl Gl for Context {
         ty: GLenum,
         offset: usize,
     ) {
-        unsafe {
-            TexSubImage3D(
-                target,
-                level,
-                xoffset,
-                yoffset,
-                zoffset,
-                width,
-                height,
-                depth,
-                format,
-                ty,
-                offset as *const c_void,
-            );
-        }
+        panic!();
     }
 
     fn tex_storage_2d(
@@ -1259,10 +1232,7 @@ impl Gl for Context {
         height: GLsizei,
         depth: GLsizei,
     ) {
-        //panic!();
-        unsafe {
-            TexStorage3D(target, levels, internal_format, width, height, depth);
-        }
+        panic!();
     }
 
     fn get_tex_image_into_buffer(
@@ -1422,10 +1392,7 @@ impl Gl for Context {
             "framebuffer_texture_layer {} {} {} {} {}",
             target, attachment, texture, level, layer
         );
-        //panic!();
-        unsafe {
-            FramebufferTextureLayer(target, attachment, texture, level, layer);
-        }
+        panic!();
     }
 
     fn blit_framebuffer(
@@ -1452,6 +1419,10 @@ impl Gl for Context {
         panic!();
     }
 
+    fn vertex_attrib_binding(&self, attrib_index: GLuint, binding_index: GLuint) {
+        unimplemented!("Not supported by SWGL");
+    }
+
     fn vertex_attrib_pointer_f32(
         &self,
         index: GLuint,
@@ -1516,6 +1487,31 @@ impl Gl for Context {
         }
     }
 
+    fn vertex_attrib_format(
+        &self,
+        attrib_index: GLuint,
+        size: GLint,
+        type_: GLenum,
+        normalized: bool,
+        relative_offset: GLuint,
+    ) {
+        unimplemented!("Not supported by SWGL");
+    }
+
+    fn vertex_attrib_i_format(
+        &self,
+        attrib_index: GLuint,
+        size: GLint,
+        type_: GLenum,
+        relative_offset: GLuint,
+    ) {
+        unimplemented!("Not supported by SWGL");
+    }
+
+    fn vertex_binding_divisor(&self, binding_index: GLuint, divisor: GLuint) {
+        unimplemented!("Not supported by SWGL");
+    }
+
     fn viewport(&self, x: GLint, y: GLint, width: GLsizei, height: GLsizei) {
         debug!("viewport {} {} {} {}", x, y, width, height);
         //panic!();
@@ -1548,13 +1544,7 @@ impl Gl for Context {
 
     fn draw_arrays(&self, mode: GLenum, first: GLint, count: GLsizei) {
         unsafe {
-            DrawElementsInstanced(
-                mode,
-                count,
-                NONE,
-                first as GLintptr,
-                1,
-            );
+            DrawElementsInstanced(mode, count, NONE, first as GLintptr, 1);
         }
     }
 
@@ -1566,13 +1556,7 @@ impl Gl for Context {
         primcount: GLsizei,
     ) {
         unsafe {
-            DrawElementsInstanced(
-                mode,
-                count,
-                NONE,
-                first as GLintptr,
-                primcount,
-            );
+            DrawElementsInstanced(mode, count, NONE, first as GLintptr, primcount);
         }
     }
 
@@ -1589,13 +1573,7 @@ impl Gl for Context {
         );
         //panic!();
         unsafe {
-            DrawElementsInstanced(
-                mode,
-                count,
-                element_type,
-                indices_offset as GLintptr,
-                1,
-            );
+            DrawElementsInstanced(mode, count, element_type, indices_offset as GLintptr, 1);
         }
     }
 
@@ -1971,9 +1949,9 @@ impl Gl for Context {
         unsafe {
             let llstr = GetString(which);
             if !llstr.is_null() {
-                return str::from_utf8_unchecked(CStr::from_ptr(llstr).to_bytes()).to_string();
+                str::from_utf8_unchecked(CStr::from_ptr(llstr).to_bytes()).to_string()
             } else {
-                return "".to_string();
+                "".to_string()
             }
         }
     }
@@ -2258,7 +2236,7 @@ impl Gl for Context {
 
     // GL_KHR_blend_equation_advanced
     fn blend_barrier_khr(&self) {
-        panic!();
+        // No barrier required, so nothing to do
     }
 
     // GL_CHROMIUM_copy_texture
@@ -2350,6 +2328,21 @@ impl Gl for Context {
     fn flush_mapped_buffer_range(&self, target: GLenum, offset: GLintptr, length: GLsizeiptr) {
         unimplemented!("Not supported by SWGL");
     }
+
+    fn start_tiling_qcom(
+        &self,
+        x: GLuint,
+        y: GLuint,
+        width: GLuint,
+        height: GLuint,
+        preserve_mask: GLbitfield,
+    ) {
+        unimplemented!("Not supported by SWGL");
+    }
+
+    fn end_tiling_qcom(&self, preserve_mask: GLbitfield) {
+        unimplemented!("Not supported by SWGL");
+    }
 }
 
 /// A resource that is intended for sharing between threads.
@@ -2362,12 +2355,15 @@ pub struct LockedResource(*mut LockedTexture);
 unsafe impl Send for LockedResource {}
 unsafe impl Sync for LockedResource {}
 
-#[repr(C)]
-pub enum YUVColorSpace {
-    Rec601 = 0,
-    Rec709,
-    Rec2020,
-    Identity,
+#[repr(u8)]
+pub enum YuvRangedColorSpace {
+    Rec601Narrow = 0,
+    Rec601Full,
+    Rec709Narrow,
+    Rec709Full,
+    Rec2020Narrow,
+    Rec2020Full,
+    GbrIdentity,
 }
 
 impl LockedResource {
@@ -2386,7 +2382,8 @@ impl LockedResource {
         dst_width: GLsizei,
         dst_height: GLsizei,
         opaque: bool,
-        flip: bool,
+        flip_x: bool,
+        flip_y: bool,
         filter: GLenum,
         clip_x: GLint,
         clip_y: GLint,
@@ -2406,7 +2403,8 @@ impl LockedResource {
                 dst_width,
                 dst_height,
                 opaque as GLboolean,
-                flip as GLboolean,
+                flip_x as GLboolean,
+                flip_y as GLboolean,
                 filter,
                 clip_x,
                 clip_y,
@@ -2422,7 +2420,7 @@ impl LockedResource {
         locked_y: &LockedResource,
         locked_u: &LockedResource,
         locked_v: &LockedResource,
-        color_space: YUVColorSpace,
+        color_space: YuvRangedColorSpace,
         color_depth: GLuint,
         src_x: GLint,
         src_y: GLint,
@@ -2432,7 +2430,8 @@ impl LockedResource {
         dst_y: GLint,
         dst_width: GLsizei,
         dst_height: GLsizei,
-        flip: bool,
+        flip_x: bool,
+        flip_y: bool,
         clip_x: GLint,
         clip_y: GLint,
         clip_width: GLsizei,
@@ -2454,7 +2453,8 @@ impl LockedResource {
                 dst_y,
                 dst_width,
                 dst_height,
-                flip as GLboolean,
+                flip_x as GLboolean,
+                flip_y as GLboolean,
                 clip_x,
                 clip_y,
                 clip_width,
@@ -2463,6 +2463,19 @@ impl LockedResource {
         }
     }
 
+    /// Apply an R8 alpha mask to this surface
+    pub fn apply_mask(
+        &self,
+        mask: &LockedResource,
+    ) {
+        unsafe {
+            ApplyMask(
+                self.0,
+                mask.0,
+            );
+        }
+    }
+
     /// Get the underlying buffer for a locked resource
     pub fn get_buffer(&self) -> (*mut c_void, i32, i32, i32) {
         unsafe {
@@ -2477,14 +2490,17 @@ impl LockedResource {
 
 impl Clone for LockedResource {
     fn clone(&self) -> Self {
-        unsafe { LockResource(self.0); }
+        unsafe {
+            LockResource(self.0);
+        }
         LockedResource(self.0)
     }
 }
 
 impl Drop for LockedResource {
     fn drop(&mut self) {
-        unsafe { UnlockResource(self.0); }
+        unsafe {
+            UnlockResource(self.0);
+        }
     }
 }
-
