@@ -16,6 +16,7 @@
 #include "nsIIOService.h"
 #include "nsIInputStream.h"
 #include "nsIObliviousHttp.h"
+#include "nsIOService.h"
 #include "nsISupports.h"
 #include "nsISupportsUtils.h"
 #include "nsITimedChannel.h"
@@ -33,22 +34,22 @@
 #include "TRRLoadInfo.h"
 
 #include "mozilla/Base64.h"
-#include "mozilla/DebugOnly.h"
 #include "mozilla/Logging.h"
+#include "mozilla/Maybe.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/StaticPrefs_network.h"
-#include "mozilla/Telemetry.h"
+#include "mozilla/glean/NetwerkDnsMetrics.h"
 #include "mozilla/TimeStamp.h"
 #include "mozilla/Tokenizer.h"
 #include "mozilla/UniquePtr.h"
 // Put DNSLogging.h at the end to avoid LOG being overwritten by other headers.
 #include "DNSLogging.h"
+#include "mozilla/glean/NetwerkMetrics.h"
 
 namespace mozilla {
 namespace net {
 
-NS_IMPL_ISUPPORTS(TRR, nsIHttpPushListener, nsIInterfaceRequestor,
-                  nsIStreamListener, nsIRunnable, nsITimerCallback)
+NS_IMPL_ISUPPORTS_INHERITED(TRR, Runnable, nsIStreamListener, nsITimerCallback)
 
 // when firing off a normal A or AAAA query
 TRR::TRR(AHostResolver* aResolver, nsHostRecord* aRec, enum TrrType aType)
@@ -78,13 +79,6 @@ TRR::TRR(AHostResolver* aResolver, nsHostRecord* aRec, nsCString& aHost,
                         "TRR must be in parent or socket process");
 }
 
-// used on push
-TRR::TRR(AHostResolver* aResolver, bool aPB)
-    : mozilla::Runnable("TRR"), mHostResolver(aResolver), mPB(aPB) {
-  MOZ_DIAGNOSTIC_ASSERT(XRE_IsParentProcess() || XRE_IsSocketProcess(),
-                        "TRR must be in parent or socket process");
-}
-
 // to verify a domain
 TRR::TRR(AHostResolver* aResolver, nsACString& aHost, enum TrrType aType,
          const nsACString& aOriginSuffix, bool aPB, bool aUseFreshConnection)
@@ -177,8 +171,8 @@ bool TRR::MaybeBlockRequest() {
                                                 true)) {
       if (mType == TRRTYPE_A) {
         // count only blocklist for A records to avoid double counts
-        Telemetry::Accumulate(Telemetry::DNS_TRR_BLACKLISTED3,
-                              TRRService::ProviderKey(), true);
+        glean::dns::trr_blacklisted.Get(TRRService::ProviderKey(), "true"_ns)
+            .Add();
       }
 
       RecordReason(TRRSkippedReason::TRR_HOST_BLOCKED_TEMPORARY);
@@ -192,8 +186,8 @@ bool TRR::MaybeBlockRequest() {
     }
 
     if (UseDefaultServer() && (mType == TRRTYPE_A)) {
-      Telemetry::Accumulate(Telemetry::DNS_TRR_BLACKLISTED3,
-                            TRRService::ProviderKey(), false);
+      glean::dns::trr_blacklisted.Get(TRRService::ProviderKey(), "false"_ns)
+          .Add();
     }
   }
 
@@ -301,9 +295,6 @@ nsresult TRR::SendHTTPRequest() {
   channel->SetLoadFlags(loadFlags);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = channel->SetNotificationCallbacks(this);
-  NS_ENSURE_SUCCESS(rv, rv);
-
   nsCOMPtr<nsIHttpChannel> httpChannel = do_QueryInterface(channel);
   if (!httpChannel) {
     return NS_ERROR_UNEXPECTED;
@@ -338,7 +329,10 @@ nsresult TRR::SendHTTPRequest() {
   rv = internalChannel->SetIsTRRServiceChannel(true);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  if (UseDefaultServer() && StaticPrefs::network_trr_async_connInfo()) {
+  // When using OHTTP, the we can't use cached connection info, since we
+  // need to connect to the relay, not the TRR server.
+  if (UseDefaultServer() && !useOHTTP &&
+      StaticPrefs::network_trr_async_connInfo()) {
     RefPtr<nsHttpConnectionInfo> trrConnInfo =
         TRRService::Get()->TRRConnectionInfo();
     if (trrConnInfo) {
@@ -349,7 +343,10 @@ nsresult TRR::SendHTTPRequest() {
         LOG(("TRR::SendHTTPRequest use conn info:%s\n",
              trrConnInfo->HashKey().get()));
       } else {
-        MOZ_DIAGNOSTIC_ASSERT(false);
+        // The connection info is inconsistent. Avoid using it and generate a
+        // new one.
+        TRRService::Get()->SetDefaultTRRConnectionInfo(nullptr);
+        TRRService::Get()->InitTRRConnectionInfo(true);
       }
     } else {
       TRRService::Get()->InitTRRConnectionInfo();
@@ -387,9 +384,8 @@ nsresult TRR::SendHTTPRequest() {
 
   // If the asyncOpen succeeded we can say that we actually attempted to
   // use the TRR connection.
-  RefPtr<AddrHostRecord> addrRec = do_QueryObject(mRec);
-  if (addrRec) {
-    addrRec->mResolverType = ResolverType();
+  if (mRec) {
+    mRec->mResolverType = ResolverType();
   }
 
   NS_NewTimerWithCallback(
@@ -438,186 +434,9 @@ nsresult TRR::SetupTRRServiceChannelInternal(nsIHttpChannel* aChannel,
     LOG(("TRR::SetupTRRServiceChannelInternal: couldn't set content-type!\n"));
   }
 
-  nsCOMPtr<nsITimedChannel> timedChan(do_QueryInterface(httpChannel));
-  if (timedChan) {
-    timedChan->SetTimingEnabled(true);
-  }
-
   return NS_OK;
 }
 
-NS_IMETHODIMP
-TRR::GetInterface(const nsIID& iid, void** result) {
-  if (!iid.Equals(NS_GET_IID(nsIHttpPushListener))) {
-    return NS_ERROR_NO_INTERFACE;
-  }
-
-  nsCOMPtr<nsIHttpPushListener> copy(this);
-  *result = copy.forget().take();
-  return NS_OK;
-}
-
-nsresult TRR::DohDecodeQuery(const nsCString& query, nsCString& host,
-                             enum TrrType& type) {
-  FallibleTArray<uint8_t> binary;
-  bool found_dns = false;
-  LOG(("TRR::DohDecodeQuery %s!\n", query.get()));
-
-  // extract "dns=" from the query string
-  nsAutoCString data;
-  for (const nsACString& token :
-       nsCCharSeparatedTokenizer(query, '&').ToRange()) {
-    nsDependentCSubstring dns = Substring(token, 0, 4);
-    nsAutoCString check(dns);
-    if (check.Equals("dns=")) {
-      nsDependentCSubstring q = Substring(token, 4, -1);
-      data = q;
-      found_dns = true;
-      break;
-    }
-  }
-  if (!found_dns) {
-    LOG(("TRR::DohDecodeQuery no dns= in pushed URI query string\n"));
-    return NS_ERROR_ILLEGAL_VALUE;
-  }
-
-  nsresult rv =
-      Base64URLDecode(data, Base64URLDecodePaddingPolicy::Ignore, binary);
-  NS_ENSURE_SUCCESS(rv, rv);
-  uint32_t avail = binary.Length();
-  if (avail < 12) {
-    return NS_ERROR_FAILURE;
-  }
-  // check the query bit and the opcode
-  if ((binary[2] & 0xf8) != 0) {
-    return NS_ERROR_FAILURE;
-  }
-  uint32_t qdcount = (binary[4] << 8) + binary[5];
-  if (!qdcount) {
-    return NS_ERROR_FAILURE;
-  }
-
-  uint32_t index = 12;
-  uint32_t length = 0;
-  host.Truncate();
-  do {
-    if (avail < (index + 1)) {
-      return NS_ERROR_UNEXPECTED;
-    }
-
-    length = binary[index];
-    if (length) {
-      if (host.Length()) {
-        host.Append(".");
-      }
-      if (avail < (index + 1 + length)) {
-        return NS_ERROR_UNEXPECTED;
-      }
-      host.Append((const char*)(&binary[0]) + index + 1, length);
-    }
-    index += 1 + length;  // skip length byte + label
-  } while (length);
-
-  LOG(("TRR::DohDecodeQuery host %s\n", host.get()));
-
-  if (avail < (index + 2)) {
-    return NS_ERROR_UNEXPECTED;
-  }
-  uint16_t i16 = 0;
-  i16 += binary[index] << 8;
-  i16 += binary[index + 1];
-  type = (enum TrrType)i16;
-
-  LOG(("TRR::DohDecodeQuery type %d\n", (int)type));
-
-  return NS_OK;
-}
-
-nsresult TRR::ReceivePush(nsIHttpChannel* pushed, nsHostRecord* pushedRec) {
-  if (!mHostResolver) {
-    return NS_ERROR_UNEXPECTED;
-  }
-
-  LOG(("TRR::ReceivePush: PUSH incoming!\n"));
-
-  nsCOMPtr<nsIURI> uri;
-  pushed->GetURI(getter_AddRefs(uri));
-  nsAutoCString query;
-  if (uri) {
-    uri->GetQuery(query);
-  }
-
-  if (NS_FAILED(DohDecodeQuery(query, mHost, mType)) ||
-      HostIsIPLiteral(mHost)) {  // literal
-    LOG(("TRR::ReceivePush failed to decode %s\n", mHost.get()));
-    return NS_ERROR_UNEXPECTED;
-  }
-
-  if ((mType != TRRTYPE_A) && (mType != TRRTYPE_AAAA) &&
-      (mType != TRRTYPE_TXT) && (mType != TRRTYPE_HTTPSSVC)) {
-    LOG(("TRR::ReceivePush unknown type %d\n", mType));
-    return NS_ERROR_UNEXPECTED;
-  }
-
-  if (TRRService::Get()->IsExcludedFromTRR(mHost)) {
-    return NS_ERROR_FAILURE;
-  }
-
-  uint32_t type = nsIDNSService::RESOLVE_TYPE_DEFAULT;
-  if (mType == TRRTYPE_TXT) {
-    type = nsIDNSService::RESOLVE_TYPE_TXT;
-  } else if (mType == TRRTYPE_HTTPSSVC) {
-    type = nsIDNSService::RESOLVE_TYPE_HTTPSSVC;
-  }
-
-  RefPtr<nsHostRecord> hostRecord;
-  nsresult rv;
-  rv = mHostResolver->GetHostRecord(
-      mHost, ""_ns, type, pushedRec->flags, pushedRec->af, pushedRec->pb,
-      pushedRec->originSuffix, getter_AddRefs(hostRecord));
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  // Since we don't ever call nsHostResolver::NameLookup for this record,
-  // we need to copy the trr mode from the previous record
-  if (hostRecord->mEffectiveTRRMode == nsIRequest::TRR_DEFAULT_MODE) {
-    hostRecord->mEffectiveTRRMode =
-        static_cast<nsIRequest::TRRMode>(pushedRec->mEffectiveTRRMode);
-  }
-
-  rv = mHostResolver->TrrLookup_unlocked(hostRecord, this);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  rv = pushed->AsyncOpen(this);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  // OK!
-  mChannel = pushed;
-  mRec.swap(hostRecord);
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-TRR::OnPush(nsIHttpChannel* associated, nsIHttpChannel* pushed) {
-  LOG(("TRR::OnPush entry\n"));
-  MOZ_ASSERT(associated == mChannel);
-  if (!mRec) {
-    return NS_ERROR_FAILURE;
-  }
-  if (!UseDefaultServer()) {
-    return NS_ERROR_FAILURE;
-  }
-
-  RefPtr<TRR> trr = new TRR(mHostResolver, mPB);
-  return trr->ReceivePush(pushed, mRec);
-}
-
 NS_IMETHODIMP
 TRR::OnStartRequest(nsIRequest* aRequest) {
   LOG(("TRR::OnStartRequest %p %s %d\n", this, mHost.get(), mType));
@@ -626,8 +445,10 @@ TRR::OnStartRequest(nsIRequest* aRequest) {
   aRequest->GetStatus(&status);
 
   if (NS_FAILED(status)) {
-    if (NS_IsOffline()) {
-      RecordReason(TRRSkippedReason::TRR_IS_OFFLINE);
+    if (gIOService->InSleepMode()) {
+      RecordReason(TRRSkippedReason::TRR_SYSTEM_SLEEP_MODE);
+    } else if (NS_IsOffline()) {
+      RecordReason(TRRSkippedReason::TRR_BROWSER_IS_OFFLINE);
     }
 
     switch (status) {
@@ -635,7 +456,7 @@ TRR::OnStartRequest(nsIRequest* aRequest) {
         RecordReason(TRRSkippedReason::TRR_CHANNEL_DNS_FAIL);
         break;
       case NS_ERROR_OFFLINE:
-        RecordReason(TRRSkippedReason::TRR_IS_OFFLINE);
+        RecordReason(TRRSkippedReason::TRR_BROWSER_IS_OFFLINE);
         break;
       case NS_ERROR_NET_RESET:
         RecordReason(TRRSkippedReason::TRR_NET_RESET);
@@ -753,6 +574,23 @@ void TRR::StoreIPHintAsDNSRecord(const struct SVCB& aSVCBRecord) {
 }
 
 nsresult TRR::ReturnData(nsIChannel* aChannel) {
+  Maybe<TimeDuration> trrFetchDuration;
+  Maybe<TimeDuration> trrFetchDurationNetworkOnly;
+  // Set timings.
+  nsCOMPtr<nsITimedChannel> timedChan = do_QueryInterface(aChannel);
+  if (timedChan) {
+    TimeStamp asyncOpen, start, end;
+    if (NS_SUCCEEDED(timedChan->GetAsyncOpen(&asyncOpen)) &&
+        !asyncOpen.IsNull()) {
+      trrFetchDuration = Some(TimeStamp::Now() - asyncOpen);
+    }
+    if (NS_SUCCEEDED(timedChan->GetRequestStart(&start)) &&
+        NS_SUCCEEDED(timedChan->GetResponseEnd(&end)) && !start.IsNull() &&
+        !end.IsNull()) {
+      trrFetchDurationNetworkOnly = Some(end - start);
+    }
+  }
+
   if (mType != TRRTYPE_TXT && mType != TRRTYPE_HTTPSSVC) {
     // create and populate an AddrInfo instance to pass on
     RefPtr<AddrInfo> ai(new AddrInfo(mHost, ResolverType(), mType,
@@ -760,33 +598,50 @@ nsresult TRR::ReturnData(nsIChannel* aChannel) {
     auto builder = ai->Build();
     builder.SetAddresses(std::move(mDNS.mAddresses));
     builder.SetCanonicalHostname(mCname);
-
-    // Set timings.
-    nsCOMPtr<nsITimedChannel> timedChan = do_QueryInterface(aChannel);
-    if (timedChan) {
-      TimeStamp asyncOpen, start, end;
-      if (NS_SUCCEEDED(timedChan->GetAsyncOpen(&asyncOpen)) &&
-          !asyncOpen.IsNull()) {
-        builder.SetTrrFetchDuration(
-            (TimeStamp::Now() - asyncOpen).ToMilliseconds());
-      }
-      if (NS_SUCCEEDED(timedChan->GetRequestStart(&start)) &&
-          NS_SUCCEEDED(timedChan->GetResponseEnd(&end)) && !start.IsNull() &&
-          !end.IsNull()) {
-        builder.SetTrrFetchDurationNetworkOnly((end - start).ToMilliseconds());
-      }
+    if (trrFetchDuration) {
+      builder.SetTrrFetchDuration((*trrFetchDuration).ToMilliseconds());
+    }
+    if (trrFetchDurationNetworkOnly) {
+      builder.SetTrrFetchDurationNetworkOnly(
+          (*trrFetchDurationNetworkOnly).ToMilliseconds());
     }
     ai = builder.Finish();
 
     if (!mHostResolver) {
       return NS_ERROR_FAILURE;
     }
+    RecordReason(TRRSkippedReason::TRR_OK);
     (void)mHostResolver->CompleteLookup(mRec, NS_OK, ai, mPB, mOriginSuffix,
                                         mTRRSkippedReason, this);
     mHostResolver = nullptr;
     mRec = nullptr;
   } else {
-    (void)mHostResolver->CompleteLookupByType(mRec, NS_OK, mResult, mTTL, mPB);
+    RecordReason(TRRSkippedReason::TRR_OK);
+    (void)mHostResolver->CompleteLookupByType(mRec, NS_OK, mResult,
+                                              mTRRSkippedReason, mTTL, mPB);
+  }
+
+  nsCOMPtr<nsIHttpChannel> httpChannel = do_QueryInterface(aChannel);
+  if (httpChannel) {
+    nsAutoCString version;
+    if (NS_SUCCEEDED(httpChannel->GetProtocolVersion(version))) {
+      nsAutoCString key("h1"_ns);
+      if (version.Equals("h3"_ns)) {
+        key.Assign("h3"_ns);
+      } else if (version.Equals("h2"_ns)) {
+        key.Assign("h2"_ns);
+      }
+
+      if (trrFetchDuration) {
+        glean::networking::trr_fetch_duration.Get(key).AccumulateRawDuration(
+            *trrFetchDuration);
+      }
+      if (trrFetchDurationNetworkOnly) {
+        key.Append("_network_only"_ns);
+        glean::networking::trr_fetch_duration.Get(key).AccumulateRawDuration(
+            *trrFetchDurationNetworkOnly);
+      }
+    }
   }
   return NS_OK;
 }
@@ -801,7 +656,8 @@ nsresult TRR::FailData(nsresult error) {
 
   if (mType == TRRTYPE_TXT || mType == TRRTYPE_HTTPSSVC) {
     TypeRecordResultType empty(Nothing{});
-    (void)mHostResolver->CompleteLookupByType(mRec, error, empty, 0, mPB);
+    (void)mHostResolver->CompleteLookupByType(mRec, error, empty,
+                                              mTRRSkippedReason, 0, mPB);
   } else {
     // create and populate an TRR AddrInfo instance to pass on to signal that
     // this comes from TRR
@@ -896,6 +752,7 @@ nsresult TRR::FollowCname(nsIChannel* aChannel) {
        mCnameLoop));
   RefPtr<TRR> trr =
       new TRR(mHostResolver, mRec, mCname, mType, mCnameLoop, mPB);
+  trr->SetPurpose(mPurpose);
   if (!TRRService::Get()) {
     return NS_ERROR_FAILURE;
   }
@@ -905,9 +762,11 @@ nsresult TRR::FollowCname(nsIChannel* aChannel) {
 nsresult TRR::On200Response(nsIChannel* aChannel) {
   // decode body and create an AddrInfo struct for the response
   nsClassHashtable<nsCStringHashKey, DOHresp> additionalRecords;
-  RefPtr<TypeHostRecord> typeRec = do_QueryObject(mRec);
-  if (typeRec && typeRec->mOriginHost) {
-    GetOrCreateDNSPacket()->SetOriginHost(typeRec->mOriginHost);
+  if (RefPtr<TypeHostRecord> typeRec = do_QueryObject(mRec)) {
+    MutexAutoLock lock(typeRec->mResultsLock);
+    if (typeRec->mOriginHost) {
+      GetOrCreateDNSPacket()->SetOriginHost(typeRec->mOriginHost);
+    }
   }
   nsresult rv = GetOrCreateDNSPacket()->Decode(
       mHost, mType, mCname, StaticPrefs::network_trr_allow_rfc1918(), mDNS,
@@ -954,7 +813,7 @@ void TRR::RecordProcessingTime(nsIChannel* aChannel) {
     return;
   }
 
-  Telemetry::AccumulateTimeDelta(Telemetry::DNS_TRR_PROCESSING_TIME, end);
+  glean::dns::trr_processing_time.AccumulateRawDuration(TimeStamp::Now() - end);
 
   LOG(("Processing DoH response took %f ms",
        (TimeStamp::Now() - end).ToMilliseconds()));
@@ -983,15 +842,14 @@ static void RecordHttpVersion(nsIHttpChannel* aHttpChannel) {
     return;
   }
 
-  auto label = Telemetry::LABELS_DNS_TRR_HTTP_VERSION2::h_1;
   if (major == 2) {
-    label = Telemetry::LABELS_DNS_TRR_HTTP_VERSION2::h_2;
+    glean::dns::trr_http_version.Get(TRRService::ProviderKey(), "h_2"_ns).Add();
   } else if (major == 3) {
-    label = Telemetry::LABELS_DNS_TRR_HTTP_VERSION2::h_3;
+    glean::dns::trr_http_version.Get(TRRService::ProviderKey(), "h_3"_ns).Add();
+  } else {
+    glean::dns::trr_http_version.Get(TRRService::ProviderKey(), "h_1"_ns).Add();
   }
 
-  Telemetry::AccumulateCategoricalKeyed(TRRService::ProviderKey(), label);
-
   LOG(("RecordHttpVersion: Provider responded using HTTP version: %d", major));
 }
 
@@ -1004,6 +862,13 @@ TRR::OnStopRequest(nsIRequest* aRequest, nsresult aStatusCode) {
   channel.swap(mChannel);
 
   mChannelStatus = aStatusCode;
+  if (NS_SUCCEEDED(aStatusCode)) {
+    nsCString label = "regular"_ns;
+    if (mPB) {
+      label = "private"_ns;
+    }
+    mozilla::glean::networking::trr_request_count.Get(label).Add(1);
+  }
 
   {
     // Cancel the timer since we don't need it anymore.
