@@ -7,9 +7,9 @@
 #include "nsComponentManagerUtils.h"
 #include "nsDirectoryServiceUtils.h"
 #include "nsHttpConnectionInfo.h"
+#include "nsHttpHandler.h"
 #include "nsICaptivePortalService.h"
 #include "nsIFile.h"
-#include "nsIParentalControlsService.h"
 #include "nsINetworkLinkService.h"
 #include "nsIObserverService.h"
 #include "nsIOService.h"
@@ -20,12 +20,14 @@
 
 #include "mozilla/Preferences.h"
 #include "mozilla/StaticPrefs_network.h"
+#include "mozilla/glean/NetwerkDnsMetrics.h"
 #include "mozilla/Telemetry.h"
-#include "mozilla/TelemetryComms.h"
 #include "mozilla/Tokenizer.h"
 #include "mozilla/dom/ContentParent.h"
+#include "mozilla/glean/NetwerkMetrics.h"
 #include "mozilla/net/NeckoParent.h"
 #include "mozilla/net/TRRServiceChild.h"
+#include "mozilla/ProfilerMarkers.h"
 // Put DNSLogging.h at the end to avoid LOG being overwritten by other headers.
 #include "DNSLogging.h"
 
@@ -100,6 +102,8 @@ void TRRService::SetProviderDomain(const nsACString& aTRRDomain) {
   }
 }
 
+const nsCString& TRRProviderKey() { return TRRService::ProviderKey(); }
+
 // static
 const nsCString& TRRService::ProviderKey() {
   return kTRRDomains[sCurrentTRRModeIndex][sDomainIndex];
@@ -114,7 +118,7 @@ NS_IMPL_RELEASE_USING_AGGREGATOR(TRRService::ConfirmationContext,
 NS_IMPL_QUERY_INTERFACE(TRRService::ConfirmationContext, nsITimerCallback,
                         nsINamed)
 
-TRRService::TRRService() : mLock("TRRService", this) {
+TRRService::TRRService() : mLock("TRRService") {
   MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
 }
 
@@ -164,13 +168,7 @@ bool TRRService::CheckCaptivePortalIsPassed() {
   return result;
 }
 
-static void EventTelemetryPrefChanged(const char* aPref, void* aData) {
-  Telemetry::SetEventRecordingEnabled(
-      "network.dns"_ns,
-      StaticPrefs::network_trr_confirmation_telemetry_enabled());
-}
-
-nsresult TRRService::Init() {
+nsresult TRRService::Init(bool aNativeHTTPSQueryEnabled) {
   MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
   if (mInitialized) {
     return NS_OK;
@@ -189,13 +187,14 @@ nsresult TRRService::Init() {
 
   sTRRServicePtr = this;
 
+  mNativeHTTPSQueryEnabled = aNativeHTTPSQueryEnabled;
   ReadPrefs(nullptr);
   mConfirmation.HandleEvent(ConfirmationEvent::Init);
 
   if (XRE_IsParentProcess()) {
     mCaptiveIsPassed = CheckCaptivePortalIsPassed();
 
-    mParentalControlEnabled = GetParentalControlEnabledInternal();
+    mParentalControlEnabled = GetParentalControlsEnabledInternal();
 
     mLinkService = do_GetService(NS_NETWORK_LINK_SERVICE_CONTRACTID);
     if (mLinkService) {
@@ -214,26 +213,19 @@ nsresult TRRService::Init() {
     sTRRBackgroundThread = thread;
   }
 
-  Preferences::RegisterCallbackAndCall(
-      EventTelemetryPrefChanged,
-      "network.trr.confirmation_telemetry_enabled"_ns);
-
   LOG(("Initialized TRRService\n"));
   return NS_OK;
 }
 
 // static
-bool TRRService::GetParentalControlEnabledInternal() {
-  nsCOMPtr<nsIParentalControlsService> pc =
-      do_CreateInstance("@mozilla.org/parental-controls-service;1");
-  if (pc) {
-    bool result = false;
-    pc->GetParentalControlsEnabled(&result);
-    LOG(("TRRService::GetParentalControlEnabledInternal=%d\n", result));
-    return result;
-  }
+bool TRRService::GetParentalControlsEnabledInternal() {
+  return nsHttpHandler::GetParentalControlsEnabled();
+}
 
-  return false;
+// static, for testing purposes only
+bool TRRService::ReloadParentalControlsEnabled() {
+  nsHttpHandler::UpdateParentalControlsEnabled(true /* wait for completion */);
+  return nsHttpHandler::GetParentalControlsEnabled();
 }
 
 void TRRService::SetDetectedTrrURI(const nsACString& aURI) {
@@ -312,7 +304,7 @@ bool TRRService::MaybeSetPrivateURI(const nsACString& aURI) {
 
   ProcessURITemplate(newURI);
   {
-    MutexSingleWriterAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
     if (mPrivateURI.Equals(newURI)) {
       return false;
     }
@@ -343,7 +335,7 @@ bool TRRService::MaybeSetPrivateURI(const nsACString& aURI) {
       if (!neckoParent) {
         continue;
       }
-      Unused << neckoParent->SendSetTRRDomain(host);
+      (void)neckoParent->SendSetTRRDomain(host);
     }
 
     AsyncCreateTRRConnectionInfo(mPrivateURI);
@@ -393,27 +385,28 @@ nsresult TRRService::ReadPrefs(const char* name) {
     OnTRRURIChange();
   }
   if (!name || !strcmp(name, TRR_PREF("credentials"))) {
-    MutexSingleWriterAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
     Preferences::GetCString(TRR_PREF("credentials"), mPrivateCred);
   }
   if (!name || !strcmp(name, TRR_PREF("confirmationNS"))) {
-    MutexSingleWriterAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
     Preferences::GetCString(TRR_PREF("confirmationNS"), mConfirmationNS);
     LOG(("confirmationNS = %s", mConfirmationNS.get()));
   }
   if (!name || !strcmp(name, TRR_PREF("bootstrapAddr"))) {
-    MutexSingleWriterAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
     Preferences::GetCString(TRR_PREF("bootstrapAddr"), mBootstrapAddr);
     clearEntireCache = true;
   }
   if (!name || !strcmp(name, TRR_PREF("excluded-domains")) ||
       !strcmp(name, TRR_PREF("builtin-excluded-domains"))) {
-    MutexSingleWriterAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
+
     mExcludedDomains.Clear();
 
-    auto parseExcludedDomains = [this](const char* aPrefName) {
+    auto parseExcludedDomains = [this](const char* aPrefName) MOZ_REQUIRES(
+                                    mLock) {
       nsAutoCString excludedDomains;
-      mLock.AssertCurrentThreadOwns();
       Preferences::GetCString(aPrefName, excludedDomains);
       if (excludedDomains.IsEmpty()) {
         return;
@@ -456,7 +449,7 @@ void TRRService::ClearEntireCache() {
 }
 
 void TRRService::AddEtcHosts(const nsTArray<nsCString>& aArray) {
-  MutexSingleWriterAutoLock lock(mLock);
+  MutexAutoLock lock(mLock);
   for (const auto& item : aArray) {
     LOG(("Adding %s from /etc/hosts to excluded domains", item.get()));
     mEtcHostsDomains.Insert(item);
@@ -478,12 +471,12 @@ void TRRService::ReadEtcHostsFile() {
 }
 
 void TRRService::GetURI(nsACString& result) {
-  MutexSingleWriterAutoLock lock(mLock);
+  MutexAutoLock lock(mLock);
   result = mPrivateURI;
 }
 
 nsresult TRRService::GetCredentials(nsCString& result) {
-  MutexSingleWriterAutoLock lock(mLock);
+  MutexAutoLock lock(mLock);
   result = mPrivateCred;
   return NS_OK;
 }
@@ -531,8 +524,7 @@ nsresult TRRService::DispatchTRRRequestInternal(TRR* aTrrRequest,
 }
 
 already_AddRefed<nsIThread> TRRService::MainThreadOrTRRThread(bool aWithLock) {
-  if (!StaticPrefs::network_trr_fetch_off_main_thread() ||
-      XRE_IsSocketProcess() || mDontUseTRRThread) {
+  if (XRE_IsSocketProcess() || mDontUseTRRThread) {
     return do_GetMainThread();
   }
 
@@ -541,7 +533,7 @@ already_AddRefed<nsIThread> TRRService::MainThreadOrTRRThread(bool aWithLock) {
 }
 
 already_AddRefed<nsIThread> TRRService::TRRThread() {
-  MutexSingleWriterAutoLock lock(mLock);
+  MutexAutoLock lock(mLock);
   return TRRThread_locked();
 }
 
@@ -553,7 +545,7 @@ already_AddRefed<nsIThread> TRRService::TRRThread_locked() {
 bool TRRService::IsOnTRRThread() {
   nsCOMPtr<nsIThread> thread;
   {
-    MutexSingleWriterAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
     thread = sTRRBackgroundThread;
   }
   if (!thread) {
@@ -574,7 +566,7 @@ TRRService::Observe(nsISupports* aSubject, const char* aTopic,
     mConfirmationTriggered = false;
     ReadPrefs(NS_ConvertUTF16toUTF8(aData).get());
     {
-      MutexSingleWriterAutoLock lock(mLock);
+      MutexAutoLock lock(mLock);
       mConfirmation.RecordEvent("pref-change", lock);
     }
 
@@ -624,9 +616,12 @@ TRRService::Observe(nsISupports* aSubject, const char* aTopic,
     }
 
     if (!strcmp(aTopic, NS_NETWORK_LINK_TOPIC)) {
-      if (NS_ConvertUTF16toUTF8(aData).EqualsLiteral(
-              NS_NETWORK_LINK_DATA_DOWN)) {
-        MutexSingleWriterAutoLock lock(mLock);
+      nsAutoCString converted = NS_ConvertUTF16toUTF8(aData);
+      if (converted.EqualsLiteral(NS_NETWORK_LINK_DATA_DOWN)) {
+        MutexAutoLock lock(mLock);
+        mConfirmation.RecordEvent("network-down", lock);
+      } else if (converted.EqualsLiteral(NS_NETWORK_LINK_DATA_CHANGED)) {
+        MutexAutoLock lock(mLock);
         mConfirmation.RecordEvent("network-change", lock);
       }
 
@@ -636,7 +631,7 @@ TRRService::Observe(nsISupports* aSubject, const char* aTopic,
         CheckURIPrefs();
       }
 
-      if (NS_ConvertUTF16toUTF8(aData).EqualsLiteral(NS_NETWORK_LINK_DATA_UP)) {
+      if (converted.EqualsLiteral(NS_NETWORK_LINK_DATA_UP)) {
         mConfirmation.HandleEvent(ConfirmationEvent::NetworkUp);
       }
     }
@@ -646,7 +641,7 @@ TRRService::Observe(nsISupports* aSubject, const char* aTopic,
     // Since there should be no more confirmations after this, the shutdown
     // reason would not really be recorded in telemetry.
     {
-      MutexSingleWriterAutoLock lock(mLock);
+      MutexAutoLock lock(mLock);
       mConfirmation.RecordEvent("shutdown", lock);
     }
 
@@ -666,7 +661,7 @@ void TRRService::RebuildSuffixList(nsTArray<nsCString>&& aSuffixList) {
     return;
   }
 
-  MutexSingleWriterAutoLock lock(mLock);
+  MutexAutoLock lock(mLock);
   mDNSSuffixDomains.Clear();
   for (const auto& item : aSuffixList) {
     LOG(("TRRService adding %s to suffix list", item.get()));
@@ -676,6 +671,7 @@ void TRRService::RebuildSuffixList(nsTArray<nsCString>&& aSuffixList) {
 
 void TRRService::ConfirmationContext::SetState(
     enum ConfirmationState aNewState) {
+  LOG(("ConfirmationContext::SetState %u", uint32_t(aNewState)));
   mState = aNewState;
 
   enum ConfirmationState state = mState;
@@ -720,18 +716,18 @@ void TRRService::ConfirmationContext::SetState(
   TRRServiceChild* child = TRRServiceChild::GetSingleton();
   if (child && child->CanSend()) {
     LOG(("TRRService::SendSetConfirmationState"));
-    Unused << child->SendSetConfirmationState(mState);
+    (void)child->SendSetConfirmationState(mState);
   }
 }
 
 bool TRRService::ConfirmationContext::HandleEvent(ConfirmationEvent aEvent) {
-  MutexSingleWriterAutoLock lock(OwningObject()->mLock);
+  MutexAutoLock lock(OwningObject()->mLock);
   return HandleEvent(aEvent, lock);
 }
 
 // We're protected by service->mLock
-bool TRRService::ConfirmationContext::HandleEvent(
-    ConfirmationEvent aEvent, const MutexSingleWriterAutoLock&) {
+bool TRRService::ConfirmationContext::HandleEvent(ConfirmationEvent aEvent,
+                                                  const MutexAutoLock&) {
   auto prevAddr = TaskAddr();
   TRRService* service = OwningObject();
   service->mLock.AssertCurrentThreadOwns();
@@ -807,9 +803,11 @@ bool TRRService::ConfirmationContext::HandleEvent(
 
     MOZ_ASSERT(mode == nsIDNSService::MODE_TRRFIRST,
                "Should only confirm in TRR first mode");
-    // Set aUseFreshConnection if TRR lookups are retried.
+    // Set aUseFreshConnection if TRR lookups are retried
+    // or if confirmation already failed.
     mTask = new TRR(service, service->mConfirmationNS, TRRTYPE_NS, ""_ns, false,
-                    StaticPrefs::network_trr_retry_on_recoverable_errors());
+                    mState == CONFIRM_TRYING_FAILED ||
+                        StaticPrefs::network_trr_retry_on_recoverable_errors());
     mTask->SetTimeout(StaticPrefs::network_trr_confirmation_timeout_ms());
     mTask->SetPurpose(TRR::Confirmation);
 
@@ -875,6 +873,8 @@ bool TRRService::ConfirmationContext::HandleEvent(
       }
       break;
     case ConfirmationEvent::ConfirmOK:
+      // Reset confirmation retry timeout to default
+      mRetryInterval = StaticPrefs::network_trr_retry_timeout_ms();
       SetState(CONFIRM_OK);
       mTask = nullptr;
       break;
@@ -884,12 +884,13 @@ bool TRRService::ConfirmationContext::HandleEvent(
       SetState(CONFIRM_FAILED);
       mTask = nullptr;
       // retry failed NS confirmation
-
+      LOG(("Setting timer to reconfirm %u", uint32_t(mRetryInterval)));
       NS_NewTimerWithCallback(getter_AddRefs(mTimer), this, mRetryInterval,
                               nsITimer::TYPE_ONE_SHOT);
-      if (mRetryInterval < 64000) {
-        // double the interval up to this point
-        mRetryInterval *= 2;
+      // double the interval up to this point
+      mRetryInterval *= 2;
+      if (mRetryInterval > StaticPrefs::network_trr_max_retry_timeout_ms()) {
+        mRetryInterval = StaticPrefs::network_trr_max_retry_timeout_ms();
       }
       break;
     default:
@@ -901,7 +902,7 @@ bool TRRService::ConfirmationContext::HandleEvent(
 
 bool TRRService::MaybeBootstrap(const nsACString& aPossible,
                                 nsACString& aResult) {
-  MutexSingleWriterAutoLock lock(mLock);
+  MutexAutoLock lock(mLock);
   if (mMode == nsIDNSService::MODE_TRROFF || mBootstrapAddr.IsEmpty()) {
     return false;
   }
@@ -1001,14 +1002,12 @@ bool TRRService::IsExcludedFromTRR(const nsACString& aHost) {
   // This method may be called off the main thread. We need to lock so
   // mExcludedDomains and mDNSSuffixDomains don't change while this code
   // is running.
-  MutexSingleWriterAutoLock lock(mLock);
+  MutexAutoLock lock(mLock);
 
   return IsExcludedFromTRR_unlocked(aHost);
 }
 
 bool TRRService::IsExcludedFromTRR_unlocked(const nsACString& aHost) {
-  mLock.AssertOnWritingThreadOrHeld();
-
   int32_t dot = 0;
   // iteratively check the sub-domain of |aHost|
   while (dot < static_cast<int32_t>(aHost.Length())) {
@@ -1021,7 +1020,9 @@ bool TRRService::IsExcludedFromTRR_unlocked(const nsACString& aHost) {
       return true;
     }
     if (mDNSSuffixDomains.Contains(subdomain)) {
-      LOG(("Subdomain [%s] of host [%s] Is Excluded From TRR via pref\n",
+      LOG(
+          ("Subdomain [%s] of host [%s] Is Excluded From TRR via DNSSuffix "
+           "domains\n",
            subdomain.BeginReading(), aHost.BeginReading()));
       return true;
     }
@@ -1088,7 +1089,7 @@ void TRRService::AddToBlocklist(const nsACString& aHost,
 
 NS_IMETHODIMP
 TRRService::ConfirmationContext::Notify(nsITimer* aTimer) {
-  MutexSingleWriterAutoLock lock(OwningObject()->mLock);
+  MutexAutoLock lock(OwningObject()->mLock);
   if (aTimer == mTimer) {
     HandleEvent(ConfirmationEvent::ConfirmationRetry, lock);
   }
@@ -1154,13 +1155,12 @@ void TRRService::RecordTRRStatus(TRR* aTrrRequest) {
 
   nsresult channelStatus = aTrrRequest->ChannelStatus();
 
-  Telemetry::AccumulateCategoricalKeyed(
-      ProviderKey(), NS_SUCCEEDED(channelStatus)
-                         ? Telemetry::LABELS_DNS_TRR_SUCCESS3::Fine
-                         : (channelStatus == NS_ERROR_NET_TIMEOUT_EXTERNAL
-                                ? Telemetry::LABELS_DNS_TRR_SUCCESS3::Timeout
-                                : Telemetry::LABELS_DNS_TRR_SUCCESS3::Bad));
-
+  glean::dns::trr_success.Get(
+      ProviderKey(),
+      NS_SUCCEEDED(channelStatus)
+          ? "Fine"_ns
+          : (channelStatus == NS_ERROR_NET_TIMEOUT_EXTERNAL ? "Timeout"_ns
+                                                            : "Bad"_ns));
   mConfirmation.RecordTRRStatus(aTrrRequest);
 }
 
@@ -1214,8 +1214,8 @@ void TRRService::ConfirmationContext::RecordTRRStatus(TRR* aTrrRequest) {
   }
 }
 
-void TRRService::ConfirmationContext::RecordEvent(
-    const char* aReason, const MutexSingleWriterAutoLock&) {
+void TRRService::ConfirmationContext::RecordEvent(const char* aReason,
+                                                  const MutexAutoLock&) {
   // Reset the confirmation context attributes
   // Only resets the attributes that we keep for telemetry purposes.
   auto reset = [&]() {
@@ -1238,9 +1238,6 @@ void TRRService::ConfirmationContext::RecordEvent(
     return;
   }
 
-  Telemetry::EventID eventType =
-      Telemetry::EventID::NetworkDns_Trrconfirmation_Context;
-
   nsAutoCString results;
   static_assert(RESULTS_SIZE < 64);
 
@@ -1260,32 +1257,23 @@ void TRRService::ConfirmationContext::RecordEvent(
     results.Append(nsDependentCSubstring(mResults, posInResults));
   }
 
-  auto extra = Some<nsTArray<mozilla::Telemetry::EventExtraEntry>>({
-      Telemetry::EventExtraEntry{"trigger"_ns, mTrigger},
-      Telemetry::EventExtraEntry{"contextReason"_ns, mContextChangeReason},
-      Telemetry::EventExtraEntry{"attemptCount"_ns,
-                                 nsPrintfCString("%u", mAttemptCount)},
-      Telemetry::EventExtraEntry{"results"_ns, results},
-      Telemetry::EventExtraEntry{
-          "time"_ns,
-          nsPrintfCString(
-              "%f",
-              !mFirstRequestTime.IsNull()
-                  ? (TimeStamp::Now() - mFirstRequestTime).ToMilliseconds()
-                  : 0.0)},
-      Telemetry::EventExtraEntry{"networkID"_ns, mNetworkId},
-      Telemetry::EventExtraEntry{"captivePortal"_ns,
-                                 nsPrintfCString("%i", mCaptivePortalStatus)},
-  });
-
-  if (mTrigger.Equals("failed-lookups"_ns)) {
-    extra.ref().AppendElement(
-        Telemetry::EventExtraEntry{"failedLookups"_ns, mFailedLookups});
-  }
-
-  enum ConfirmationState state = mState;
-  Telemetry::RecordEvent(eventType, mozilla::Some(nsPrintfCString("%u", state)),
-                         extra);
+  glean::network_dns::TrrConfirmationContextExtra extra = {
+      .attemptcount = Some(mAttemptCount),
+      .captiveportal = Some(nsPrintfCString("%i", mCaptivePortalStatus)),
+      .contextreason = Some(mContextChangeReason),
+      .failedlookups = mTrigger.Equals("failed-lookups"_ns)
+                           ? Some(mFailedLookups)
+                           : Nothing(),
+      .networkid = Some(mNetworkId),
+      .results = Some(results),
+      .time = Some(nsPrintfCString(
+          "%f", !mFirstRequestTime.IsNull()
+                    ? (TimeStamp::Now() - mFirstRequestTime).ToMilliseconds()
+                    : 0.0)),
+      .trigger = Some(mTrigger),
+      .value = Some(mState),
+  };
+  glean::network_dns::trr_confirmation_context.Record(Some(extra));
 
   reset();
 }
@@ -1300,7 +1288,7 @@ void TRRService::ConfirmationContext::RequestCompleted(
 void TRRService::ConfirmationContext::CompleteConfirmation(nsresult aStatus,
                                                            TRR* aTRRRequest) {
   {
-    MutexSingleWriterAutoLock lock(OwningObject()->mLock);
+    MutexAutoLock lock(OwningObject()->mLock);
     // Ignore confirmations that dont match the pending task.
     if (mTask != aTRRRequest) {
       return;
@@ -1317,8 +1305,12 @@ void TRRService::ConfirmationContext::CompleteConfirmation(nsresult aStatus,
 
     MOZ_ASSERT(mTask);
     if (NS_SUCCEEDED(aStatus)) {
+      profiler_add_marker("TRR Confirmation Success",
+                          geckoprofiler::category::NETWORK);
       HandleEvent(ConfirmationEvent::ConfirmOK, lock);
     } else {
+      profiler_add_marker("TRR Confirmation Failure",
+                          geckoprofiler::category::NETWORK);
       HandleEvent(ConfirmationEvent::ConfirmFail, lock);
     }
 
@@ -1339,8 +1331,10 @@ void TRRService::ConfirmationContext::CompleteConfirmation(nsresult aStatus,
     MOZ_ASSERT(State() == CONFIRM_FAILED);
   }
 
-  Telemetry::Accumulate(Telemetry::DNS_TRR_NS_VERFIFIED3,
-                        TRRService::ProviderKey(), (State() == CONFIRM_OK));
+  glean::dns::trr_ns_verfified
+      .Get(TRRService::ProviderKey(),
+           (State() == CONFIRM_OK) ? "true"_ns : "false"_ns)
+      .Add();
 }
 
 AHostResolver::LookupStatus TRRService::CompleteLookup(
@@ -1378,7 +1372,7 @@ AHostResolver::LookupStatus TRRService::CompleteLookup(
 
 AHostResolver::LookupStatus TRRService::CompleteLookupByType(
     nsHostRecord*, nsresult, mozilla::net::TypeRecordResultType& aResult,
-    uint32_t aTtl, bool aPb) {
+    mozilla::net::TRRSkippedReason aReason, uint32_t aTtl, bool aPb) {
   return LOOKUP_OK;
 }
 
@@ -1392,9 +1386,9 @@ NS_IMETHODIMP TRRService::OnProxyConfigChanged() {
   return NS_OK;
 }
 
-void TRRService::InitTRRConnectionInfo() {
+void TRRService::InitTRRConnectionInfo(bool aForceReinit) {
   if (XRE_IsParentProcess()) {
-    TRRServiceBase::InitTRRConnectionInfo();
+    TRRServiceBase::InitTRRConnectionInfo(aForceReinit);
     return;
   }
 
@@ -1404,7 +1398,7 @@ void TRRService::InitTRRConnectionInfo() {
   TRRServiceChild* child = TRRServiceChild::GetSingleton();
   if (child && child->CanSend()) {
     LOG(("TRRService::SendInitTRRConnectionInfo"));
-    Unused << child->SendInitTRRConnectionInfo();
+    (void)child->SendInitTRRConnectionInfo(aForceReinit);
   }
 }
 
