@@ -2,7 +2,6 @@
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
-#include <numeric>
 #include <vector>
 #include <algorithm>
 
@@ -26,13 +25,15 @@
 #include "nsCRT.h"
 #include "nsNetCID.h"
 #include "nsThreadUtils.h"
+#include "mozilla/AppShutdown.h"
+#include "mozilla/Components.h"
 #include "mozilla/Logging.h"
 #include "mozilla/StaticPrefs_network.h"
 #include "mozilla/SHA1.h"
 #include "mozilla/Base64.h"
 #include "mozilla/ScopeExit.h"
 #include "mozilla/Services.h"
-#include "mozilla/Telemetry.h"
+#include "mozilla/glean/NetwerkMetrics.h"
 #include "nsNetworkLinkService.h"
 #include "../../base/IPv6Utils.h"
 #include "../LinkServiceCommon.h"
@@ -75,7 +76,9 @@ static OSStatus getErrorCodeBool(Boolean success) {
 }
 
 // If given a NULL pointer, return the error code.
-static OSStatus getErrorCodePtr(const void* value) { return getErrorCodeBool(value != nullptr); }
+static OSStatus getErrorCodePtr(const void* value) {
+  return getErrorCodeBool(value != nullptr);
+}
 
 // Convenience function to allow NULL input.
 static void CFReleaseSafe(CFTypeRef cf) {
@@ -86,8 +89,10 @@ static void CFReleaseSafe(CFTypeRef cf) {
   }
 }
 
-NS_IMPL_ISUPPORTS(nsNetworkLinkService, nsINetworkLinkService, nsIObserver, nsITimerCallback,
-                  nsINamed)
+mozilla::Atomic<bool, mozilla::MemoryOrdering::Relaxed> sHasNonLocalIPv6{true};
+
+NS_IMPL_ISUPPORTS(nsNetworkLinkService, nsINetworkLinkService, nsIObserver,
+                  nsITimerCallback, nsINamed)
 
 nsNetworkLinkService::nsNetworkLinkService()
     : mLinkUp(true),
@@ -129,7 +134,8 @@ nsNetworkLinkService::GetNetworkID(nsACString& aNetworkID) {
 }
 
 NS_IMETHODIMP
-nsNetworkLinkService::GetPlatformDNSIndications(uint32_t* aPlatformDNSIndications) {
+nsNetworkLinkService::GetPlatformDNSIndications(
+    uint32_t* aPlatformDNSIndications) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
@@ -139,8 +145,9 @@ void nsNetworkLinkService::GetDnsSuffixListInternal() {
 
   auto sendNotification = mozilla::MakeScopeExit([self = RefPtr{this}] {
     NS_DispatchToMainThread(NS_NewRunnableFunction(
-        "nsNetworkLinkService::GetDnsSuffixListInternal",
-        [self]() { self->NotifyObservers(NS_DNS_SUFFIX_LIST_UPDATED_TOPIC, nullptr); }));
+        "nsNetworkLinkService::GetDnsSuffixListInternal", [self]() {
+          self->NotifyObservers(NS_DNS_SUFFIX_LIST_UPDATED_TOPIC, nullptr);
+        }));
   });
 
   nsTArray<nsCString> result;
@@ -176,15 +183,17 @@ nsNetworkLinkService::GetResolvers(nsTArray<RefPtr<nsINetAddr>>& aResolvers) {
 }
 
 NS_IMETHODIMP
-nsNetworkLinkService::GetNativeResolvers(nsTArray<mozilla::net::NetAddr>& aResolvers) {
+nsNetworkLinkService::GetNativeResolvers(
+    nsTArray<mozilla::net::NetAddr>& aResolvers) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 #ifndef SA_SIZE
-#  define SA_SIZE(sa)                                 \
-    ((!(sa) || ((struct sockaddr*)(sa))->sa_len == 0) \
-         ? sizeof(uint32_t)                           \
-         : 1 + ((((struct sockaddr*)(sa))->sa_len - 1) | (sizeof(uint32_t) - 1)))
+#  define SA_SIZE(sa)                                    \
+    ((!(sa) || ((struct sockaddr*)(sa))->sa_len == 0)    \
+         ? sizeof(uint32_t)                              \
+         : 1 + ((((struct sockaddr*)(sa))->sa_len - 1) | \
+                (sizeof(uint32_t) - 1)))
 #endif
 
 static bool getMac(struct sockaddr_dl* sdl, char* buf, size_t bufsize) {
@@ -196,14 +205,14 @@ static bool getMac(struct sockaddr_dl* sdl, char* buf, size_t bufsize) {
     return false;
   }
 
-  snprintf(buf, bufsize, "%02x:%02x:%02x:%02x:%02x:%02x", mac[0], mac[1], mac[2], mac[3], mac[4],
-           mac[5]);
+  snprintf(buf, bufsize, "%02x:%02x:%02x:%02x:%02x:%02x", mac[0], mac[1],
+           mac[2], mac[3], mac[4], mac[5]);
   return true;
 }
 
 /* If the IP matches, get the MAC and return true */
-static bool matchIp(struct sockaddr_dl* sdl, struct sockaddr_inarp* addr, char* ip, char* buf,
-                    size_t bufsize) {
+static bool matchIp(struct sockaddr_dl* sdl, struct sockaddr_inarp* addr,
+                    char* ip, char* buf, size_t bufsize) {
   if (sdl->sdl_alen) {
     if (!strcmp(inet_ntoa(addr->sin_addr), ip)) {
       if (getMac(sdl, buf, bufsize)) {
@@ -275,8 +284,10 @@ static bool scanArp(char* ip, char* mac, size_t maclen) {
   struct rt_msghdr* rtm;
   for (next = &buf[0]; next < lim; next += rtm->rtm_msglen) {
     rtm = reinterpret_cast<struct rt_msghdr*>(next);
-    struct sockaddr_inarp* sin2 = reinterpret_cast<struct sockaddr_inarp*>(rtm + 1);
-    struct sockaddr_dl* sdl = reinterpret_cast<struct sockaddr_dl*>((char*)sin2 + SA_SIZE(sin2));
+    struct sockaddr_inarp* sin2 =
+        reinterpret_cast<struct sockaddr_inarp*>(rtm + 1);
+    struct sockaddr_dl* sdl =
+        reinterpret_cast<struct sockaddr_dl*>((char*)sin2 + SA_SIZE(sin2));
     if (matchIp(sdl, sin2, ip, mac, maclen)) {
       return true;
     }
@@ -285,9 +296,10 @@ static bool scanArp(char* ip, char* mac, size_t maclen) {
   return false;
 }
 
-// Append the mac address of rtm to `stringsToHash`. If it's not in arp table, append
-// ifname and IP address.
-static bool parseHashKey(struct rt_msghdr* rtm, nsTArray<nsCString>& stringsToHash,
+// Append the mac address of rtm to `stringsToHash`. If it's not in arp table,
+// append ifname and IP address.
+static bool parseHashKey(struct rt_msghdr* rtm,
+                         nsTArray<nsCString>& stringsToHash,
                          bool skipDstCheck) {
   struct sockaddr* sa;
   struct sockaddr_in* sockin;
@@ -317,8 +329,8 @@ static bool parseHashKey(struct rt_msghdr* rtm, nsTArray<nsCString>& stringsToHa
     return false;
   }
 
-  struct sockaddr* gateway =
-      reinterpret_cast<struct sockaddr*>((char*)sa + RTAX_GATEWAY * SA_SIZE(sa));
+  struct sockaddr* gateway = reinterpret_cast<struct sockaddr*>(
+      (char*)sa + RTAX_GATEWAY * SA_SIZE(sa));
 
   if (!gateway) {
     return false;
@@ -414,6 +426,7 @@ bool nsNetworkLinkService::RoutingFromKernel(nsTArray<nsCString>& aHash) {
     LOG(("RoutingFromKernel: Can create a socket for network id"));
     return false;
   }
+  auto sockfd_guard = mozilla::MakeScopeExit([sockfd] { close(sockfd); });
 
   MOZ_ASSERT(!NS_IsMainThread());
 
@@ -440,7 +453,8 @@ bool nsNetworkLinkService::RoutingFromKernel(nsTArray<nsCString>& aHash) {
   sin->sin_addr = mRouteCheckIPv4;
 
   if (write(sockfd, rtm, rtm->rtm_msglen) == -1) {
-    LOG(("RoutingFromKernel: write() failed. No route to the predefine destincation"));
+    LOG(("RoutingFromKernel: write() failed. No route to the predefine "
+         "destincation"));
     return false;
   }
 
@@ -451,9 +465,11 @@ bool nsNetworkLinkService::RoutingFromKernel(nsTArray<nsCString>& aHash) {
       return false;
     }
 
-    LOG(("RoutingFromKernel: read() rtm_type: %d (%d), rtm_pid: %d (%d), rtm_seq: %d (%d)\n",
+    LOG(("RoutingFromKernel: read() rtm_type: %d (%d), rtm_pid: %d (%d), "
+         "rtm_seq: %d (%d)\n",
          rtm->rtm_type, RTM_GET, rtm->rtm_pid, pid, rtm->rtm_seq, seq));
-  } while (rtm->rtm_type != RTM_GET || rtm->rtm_pid != pid || rtm->rtm_seq != seq);
+  } while (rtm->rtm_type != RTM_GET || rtm->rtm_pid != pid ||
+           rtm->rtm_seq != seq);
 
   return parseHashKey(rtm, aHash, true);
 }
@@ -489,8 +505,9 @@ bool nsNetworkLinkService::IPv4NetworkId(SHA1Sum* aSHA1) {
 //
 void nsNetworkLinkService::HashSortedPrefixesAndNetmasks(
     std::vector<prefix_and_netmask> prefixAndNetmaskStore, SHA1Sum* sha1) {
-  // getifaddrs does not guarantee the interfaces will always be in the same order.
-  // We want to make sure the hash remains consistent Regardless of the interface order.
+  // getifaddrs does not guarantee the interfaces will always be in the same
+  // order. We want to make sure the hash remains consistent Regardless of the
+  // interface order.
   std::sort(prefixAndNetmaskStore.begin(), prefixAndNetmaskStore.end(),
             [](prefix_and_netmask a, prefix_and_netmask b) {
               // compare prefixStore
@@ -513,6 +530,7 @@ bool nsNetworkLinkService::IPv6NetworkId(SHA1Sum* sha1) {
   std::vector<prefix_and_netmask> prefixAndNetmaskStore;
 
   if (!getifaddrs(&ifap)) {
+    bool hasNonLocalIPv6 = false;
     struct ifaddrs* ifa;
     for (ifa = ifap; ifa; ifa = ifa->ifa_next) {
       if (ifa->ifa_addr == NULL) {
@@ -521,7 +539,9 @@ bool nsNetworkLinkService::IPv6NetworkId(SHA1Sum* sha1) {
       if ((AF_INET6 == ifa->ifa_addr->sa_family) &&
           !(ifa->ifa_flags & (IFF_POINTOPOINT | IFF_LOOPBACK))) {
         // only IPv6 interfaces that aren't pointtopoint or loopback
-        struct sockaddr_in6* sin_netmask = (struct sockaddr_in6*)ifa->ifa_netmask;
+        hasNonLocalIPv6 = true;
+        struct sockaddr_in6* sin_netmask =
+            (struct sockaddr_in6*)ifa->ifa_netmask;
         if (sin_netmask) {
           struct sockaddr_in6* sin_addr = (struct sockaddr_in6*)ifa->ifa_addr;
           int scope = net::utils::ipv6_scope(sin_addr->sin6_addr.s6_addr);
@@ -530,17 +550,20 @@ bool nsNetworkLinkService::IPv6NetworkId(SHA1Sum* sha1) {
             memset(&prefix, 0, sizeof(prefix));
             // Get the prefix by combining the address and netmask.
             for (size_t i = 0; i < sizeof(prefix); ++i) {
-              prefix.s6_addr[i] =
-                  sin_addr->sin6_addr.s6_addr[i] & sin_netmask->sin6_addr.s6_addr[i];
+              prefix.s6_addr[i] = sin_addr->sin6_addr.s6_addr[i] &
+                                  sin_netmask->sin6_addr.s6_addr[i];
             }
 
             // check if prefix and netmask was already found
-            auto prefixAndNetmask = std::make_pair(prefix, sin_netmask->sin6_addr);
+            auto prefixAndNetmask =
+                std::make_pair(prefix, sin_netmask->sin6_addr);
             auto foundPosition = std::find_if(
                 prefixAndNetmaskStore.begin(), prefixAndNetmaskStore.end(),
                 [&prefixAndNetmask](prefix_and_netmask current) {
-                  return memcmp(&prefixAndNetmask.first, &current.first, sizeof(in6_addr)) == 0 &&
-                         memcmp(&prefixAndNetmask.second, &current.second, sizeof(in6_addr)) == 0;
+                  return memcmp(&prefixAndNetmask.first, &current.first,
+                                sizeof(in6_addr)) == 0 &&
+                         memcmp(&prefixAndNetmask.second, &current.second,
+                                sizeof(in6_addr)) == 0;
                 });
             if (foundPosition != prefixAndNetmaskStore.end()) {
               continue;
@@ -550,6 +573,7 @@ bool nsNetworkLinkService::IPv6NetworkId(SHA1Sum* sha1) {
         }
       }
     }
+    sHasNonLocalIPv6 = hasNonLocalIPv6;
     freeifaddrs(ifap);
   }
   if (prefixAndNetmaskStore.empty()) {
@@ -557,7 +581,8 @@ bool nsNetworkLinkService::IPv6NetworkId(SHA1Sum* sha1) {
     return false;
   }
 
-  nsNetworkLinkService::HashSortedPrefixesAndNetmasks(prefixAndNetmaskStore, sha1);
+  nsNetworkLinkService::HashSortedPrefixesAndNetmasks(prefixAndNetmaskStore,
+                                                      sha1);
 
   return true;
 }
@@ -565,6 +590,10 @@ bool nsNetworkLinkService::IPv6NetworkId(SHA1Sum* sha1) {
 void nsNetworkLinkService::calculateNetworkIdWithDelay(uint32_t aDelay) {
   MOZ_ASSERT(NS_IsMainThread());
 
+  if (AppShutdown::IsInOrBeyond(ShutdownPhase::AppShutdownNetTeardown)) {
+    return;
+  }
+
   if (aDelay) {
     if (mNetworkIdTimer) {
       LOG(("Restart the network id timer."));
@@ -577,15 +606,17 @@ void nsNetworkLinkService::calculateNetworkIdWithDelay(uint32_t aDelay) {
     return;
   }
 
-  nsCOMPtr<nsIEventTarget> target = do_GetService(NS_STREAMTRANSPORTSERVICE_CONTRACTID);
+  nsCOMPtr<nsIEventTarget> target;
+  target = mozilla::components::StreamTransport::Service();
   if (!target) {
     return;
   }
 
-  MOZ_ALWAYS_SUCCEEDS(
-      target->Dispatch(NewRunnableMethod("nsNetworkLinkService::calculateNetworkIdInternal", this,
-                                         &nsNetworkLinkService::calculateNetworkIdInternal),
-                       NS_DISPATCH_NORMAL));
+  MOZ_ALWAYS_SUCCEEDS(target->Dispatch(
+      NewRunnableMethod("nsNetworkLinkService::calculateNetworkIdInternal",
+                        this,
+                        &nsNetworkLinkService::calculateNetworkIdInternal),
+      NS_DISPATCH_NORMAL));
 }
 
 NS_IMETHODIMP
@@ -615,7 +646,8 @@ void nsNetworkLinkService::calculateNetworkIdInternal(void) {
     SeedNetworkId(sha1);
     uint8_t digest[SHA1Sum::kHashSize];
     sha1.finish(digest);
-    nsAutoCString newString(reinterpret_cast<char*>(digest), SHA1Sum::kHashSize);
+    nsAutoCString newString(reinterpret_cast<char*>(digest),
+                            SHA1Sum::kHashSize);
     nsresult rv = Base64Encode(newString, output);
     MOZ_RELEASE_ASSERT(NS_SUCCEEDED(rv));
     LOG(("networkid: id %s\n", output.get()));
@@ -623,18 +655,18 @@ void nsNetworkLinkService::calculateNetworkIdInternal(void) {
     if (mNetworkId != output) {
       // new id
       if (found4 && !found6) {
-        Telemetry::Accumulate(Telemetry::NETWORK_ID2, 1);  // IPv4 only
+        glean::network::id.AccumulateSingleSample(1);  // IPv4 only
       } else if (!found4 && found6) {
-        Telemetry::Accumulate(Telemetry::NETWORK_ID2, 3);  // IPv6 only
+        glean::network::id.AccumulateSingleSample(3);  // IPv6 only
       } else {
-        Telemetry::Accumulate(Telemetry::NETWORK_ID2, 4);  // Both!
+        glean::network::id.AccumulateSingleSample(4);  // Both!
       }
       mNetworkId = output;
       idChanged = true;
     } else {
       // same id
       LOG(("Same network id"));
-      Telemetry::Accumulate(Telemetry::NETWORK_ID2, 2);
+      glean::network::id.AccumulateSingleSample(2);
     }
   } else {
     // no id
@@ -643,7 +675,7 @@ void nsNetworkLinkService::calculateNetworkIdInternal(void) {
     if (!mNetworkId.IsEmpty()) {
       mNetworkId.Truncate();
       idChanged = true;
-      Telemetry::Accumulate(Telemetry::NETWORK_ID2, 0);
+      glean::network::id.AccumulateSingleSample(0);
     }
   }
 
@@ -652,16 +684,17 @@ void nsNetworkLinkService::calculateNetworkIdInternal(void) {
   if (idChanged && !initialIDCalculation) {
     RefPtr<nsNetworkLinkService> self = this;
 
-    NS_DispatchToMainThread(
-        NS_NewRunnableFunction("nsNetworkLinkService::calculateNetworkIdInternal",
-                               [self]() { self->OnNetworkIdChanged(); }));
+    NS_DispatchToMainThread(NS_NewRunnableFunction(
+        "nsNetworkLinkService::calculateNetworkIdInternal",
+        [self]() { self->OnNetworkIdChanged(); }));
   }
 
   initialIDCalculation = false;
 }
 
 NS_IMETHODIMP
-nsNetworkLinkService::Observe(nsISupports* subject, const char* topic, const char16_t* data) {
+nsNetworkLinkService::Observe(nsISupports* subject, const char* topic,
+                              const char16_t* data) {
   if (!strcmp(topic, "xpcom-shutdown")) {
     Shutdown();
   }
@@ -671,14 +704,17 @@ nsNetworkLinkService::Observe(nsISupports* subject, const char* topic, const cha
 
 /* static */
 void nsNetworkLinkService::NetworkConfigChanged(SCDynamicStoreRef aStoreREf,
-                                                CFArrayRef aChangedKeys, void* aInfo) {
+                                                CFArrayRef aChangedKeys,
+                                                void* aInfo) {
   LOG(("nsNetworkLinkService::NetworkConfigChanged"));
 
   bool ipConfigChanged = false;
   bool dnsConfigChanged = false;
   for (CFIndex i = 0; i < CFArrayGetCount(aChangedKeys); ++i) {
-    CFStringRef key = static_cast<CFStringRef>(CFArrayGetValueAtIndex(aChangedKeys, i));
-    if (CFStringHasSuffix(key, kSCEntNetIPv4) || CFStringHasSuffix(key, kSCEntNetIPv6)) {
+    CFStringRef key =
+        static_cast<CFStringRef>(CFArrayGetValueAtIndex(aChangedKeys, i));
+    if (CFStringHasSuffix(key, kSCEntNetIPv4) ||
+        CFStringHasSuffix(key, kSCEntNetIPv6)) {
       ipConfigChanged = true;
     }
     if (CFStringHasSuffix(key, kSCEntNetDNS)) {
@@ -698,7 +734,8 @@ void nsNetworkLinkService::NetworkConfigChanged(SCDynamicStoreRef aStoreREf,
 
 void nsNetworkLinkService::DNSConfigChanged(uint32_t aDelayMs) {
   LOG(("nsNetworkLinkService::DNSConfigChanged"));
-  nsCOMPtr<nsIEventTarget> target = do_GetService(NS_STREAMTRANSPORTSERVICE_CONTRACTID);
+  nsCOMPtr<nsIEventTarget> target;
+  target = mozilla::components::StreamTransport::Service();
   if (!target) {
     return;
   }
@@ -714,20 +751,20 @@ void nsNetworkLinkService::DNSConfigChanged(uint32_t aDelayMs) {
           self->mDNSConfigChangedTimers.RemoveElement(aTimer);
         },
         TimeDuration::FromMilliseconds(aDelayMs), nsITimer::TYPE_ONE_SHOT,
-        "nsNetworkLinkService::GetDnsSuffixListInternal", target));
+        "nsNetworkLinkService::GetDnsSuffixListInternal"_ns, target));
     mDNSConfigChangedTimers.AppendElement(timer);
   } else {
-    MOZ_ALWAYS_SUCCEEDS(target->Dispatch(
-        NS_NewRunnableFunction("nsNetworkLinkService::GetDnsSuffixListInternal",
-                               [self = RefPtr{this}]() { self->GetDnsSuffixListInternal(); })));
+    MOZ_ALWAYS_SUCCEEDS(target->Dispatch(NS_NewRunnableFunction(
+        "nsNetworkLinkService::GetDnsSuffixListInternal",
+        [self = RefPtr{this}]() { self->GetDnsSuffixListInternal(); })));
   }
 }
 
 nsresult nsNetworkLinkService::Init(void) {
   nsresult rv;
 
-  nsCOMPtr<nsIObserverService> observerService =
-      do_GetService("@mozilla.org/observer-service;1", &rv);
+  nsCOMPtr<nsIObserverService> observerService;
+  observerService = mozilla::components::Observer::Service(&rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = observerService->AddObserver(this, "xpcom-shutdown", false);
@@ -735,7 +772,7 @@ nsresult nsNetworkLinkService::Init(void) {
 
   if (inet_pton(AF_INET, ROUTE_CHECK_IPV4, &mRouteCheckIPv4) != 1) {
     LOG(("Cannot parse address " ROUTE_CHECK_IPV4));
-    MOZ_DIAGNOSTIC_ASSERT(false, "Cannot parse address " ROUTE_CHECK_IPV4);
+    MOZ_DIAGNOSTIC_CRASH("Cannot parse address " ROUTE_CHECK_IPV4);
     return NS_ERROR_UNEXPECTED;
   }
 
@@ -745,13 +782,15 @@ nsresult nsNetworkLinkService::Init(void) {
   bzero(&addr, sizeof(addr));
   addr.sin_len = sizeof(addr);
   addr.sin_family = AF_INET;
-  mReachability = ::SCNetworkReachabilityCreateWithAddress(nullptr, (struct sockaddr*)&addr);
+  mReachability = ::SCNetworkReachabilityCreateWithAddress(
+      nullptr, (struct sockaddr*)&addr);
   if (!mReachability) {
     return NS_ERROR_NOT_AVAILABLE;
   }
 
   SCNetworkReachabilityContext context = {0, this, nullptr, nullptr, nullptr};
-  if (!::SCNetworkReachabilitySetCallback(mReachability, ReachabilityChanged, &context)) {
+  if (!::SCNetworkReachabilitySetCallback(mReachability, ReachabilityChanged,
+                                          &context)) {
     NS_WARNING("SCNetworkReachabilitySetCallback failed.");
     ::CFRelease(mReachability);
     mReachability = nullptr;
@@ -759,24 +798,25 @@ nsresult nsNetworkLinkService::Init(void) {
   }
 
   SCDynamicStoreContext storeContext = {0, this, nullptr, nullptr, nullptr};
-  mStoreRef = ::SCDynamicStoreCreate(nullptr, CFSTR("IPAndDNSChangeCallbackSCF"),
-                                     NetworkConfigChanged, &storeContext);
+  mStoreRef =
+      ::SCDynamicStoreCreate(nullptr, CFSTR("IPAndDNSChangeCallbackSCF"),
+                             NetworkConfigChanged, &storeContext);
 
   CFStringRef patterns[4] = {nullptr, nullptr, nullptr, nullptr};
   OSStatus err = getErrorCodePtr(mStoreRef);
   if (err == noErr) {
     // This pattern is "State:/Network/Service/[^/]+/IPv4".
-    patterns[0] = ::SCDynamicStoreKeyCreateNetworkServiceEntity(nullptr, kSCDynamicStoreDomainState,
-                                                                kSCCompAnyRegex, kSCEntNetIPv4);
+    patterns[0] = ::SCDynamicStoreKeyCreateNetworkServiceEntity(
+        nullptr, kSCDynamicStoreDomainState, kSCCompAnyRegex, kSCEntNetIPv4);
     // This pattern is "State:/Network/Service/[^/]+/IPv6".
-    patterns[1] = ::SCDynamicStoreKeyCreateNetworkServiceEntity(nullptr, kSCDynamicStoreDomainState,
-                                                                kSCCompAnyRegex, kSCEntNetIPv6);
+    patterns[1] = ::SCDynamicStoreKeyCreateNetworkServiceEntity(
+        nullptr, kSCDynamicStoreDomainState, kSCCompAnyRegex, kSCEntNetIPv6);
     // This pattern is "State:/Network/Service/[^/]+/DNS".
-    patterns[2] = ::SCDynamicStoreKeyCreateNetworkServiceEntity(nullptr, kSCDynamicStoreDomainState,
-                                                                kSCCompAnyRegex, kSCEntNetDNS);
+    patterns[2] = ::SCDynamicStoreKeyCreateNetworkServiceEntity(
+        nullptr, kSCDynamicStoreDomainState, kSCCompAnyRegex, kSCEntNetDNS);
     // This pattern is "Setup:/Network/Service/[^/]+/DNS".
-    patterns[3] = ::SCDynamicStoreKeyCreateNetworkServiceEntity(nullptr, kSCDynamicStoreDomainSetup,
-                                                                kSCCompAnyRegex, kSCEntNetDNS);
+    patterns[3] = ::SCDynamicStoreKeyCreateNetworkServiceEntity(
+        nullptr, kSCDynamicStoreDomainSetup, kSCCompAnyRegex, kSCEntNetDNS);
     if (!patterns[0] || !patterns[1] || !patterns[2] || !patterns[3]) {
       err = -1;
     }
@@ -788,13 +828,15 @@ nsresult nsNetworkLinkService::Init(void) {
   // that match that pattern list, then create our run loop
   // source.
   if (err == noErr) {
-    patternList = ::CFArrayCreate(nullptr, (const void**)patterns, 4, &kCFTypeArrayCallBacks);
+    patternList = ::CFArrayCreate(nullptr, (const void**)patterns, 4,
+                                  &kCFTypeArrayCallBacks);
     if (!patternList) {
       err = -1;
     }
   }
   if (err == noErr) {
-    err = getErrorCodeBool(::SCDynamicStoreSetNotificationKeys(mStoreRef, nullptr, patternList));
+    err = getErrorCodeBool(
+        ::SCDynamicStoreSetNotificationKeys(mStoreRef, nullptr, patternList));
   }
 
   if (err == noErr) {
@@ -925,26 +967,31 @@ void nsNetworkLinkService::OnReachabilityChanged() {
     return;
   }
 
-  NotifyObservers(NS_NETWORK_LINK_TOPIC,
-                  mLinkUp ? NS_NETWORK_LINK_DATA_UP : NS_NETWORK_LINK_DATA_DOWN);
+  NotifyObservers(NS_NETWORK_LINK_TOPIC, mLinkUp ? NS_NETWORK_LINK_DATA_UP
+                                                 : NS_NETWORK_LINK_DATA_DOWN);
 }
 
-void nsNetworkLinkService::NotifyObservers(const char* aTopic, const char* aData) {
+void nsNetworkLinkService::NotifyObservers(const char* aTopic,
+                                           const char* aData) {
   MOZ_ASSERT(NS_IsMainThread());
 
-  LOG(("nsNetworkLinkService::NotifyObservers: topic:%s data:%s\n", aTopic, aData ? aData : ""));
+  LOG(("nsNetworkLinkService::NotifyObservers: topic:%s data:%s\n", aTopic,
+       aData ? aData : ""));
 
-  nsCOMPtr<nsIObserverService> observerService = mozilla::services::GetObserverService();
+  nsCOMPtr<nsIObserverService> observerService =
+      mozilla::services::GetObserverService();
 
   if (observerService) {
-    observerService->NotifyObservers(static_cast<nsINetworkLinkService*>(this), aTopic,
-                                     aData ? NS_ConvertASCIItoUTF16(aData).get() : nullptr);
+    observerService->NotifyObservers(
+        static_cast<nsINetworkLinkService*>(this), aTopic,
+        aData ? NS_ConvertASCIItoUTF16(aData).get() : nullptr);
   }
 }
 
 /* static */
 void nsNetworkLinkService::ReachabilityChanged(SCNetworkReachabilityRef target,
-                                               SCNetworkConnectionFlags flags, void* info) {
+                                               SCNetworkConnectionFlags flags,
+                                               void* info) {
   LOG(("nsNetworkLinkService::ReachabilityChanged"));
   nsNetworkLinkService* service = static_cast<nsNetworkLinkService*>(info);
 
@@ -955,3 +1002,8 @@ void nsNetworkLinkService::ReachabilityChanged(SCNetworkReachabilityRef target,
   // update the DNS suffix list.
   service->DNSConfigChanged(0);
 }
+
+// static
+bool nsINetworkLinkService::HasNonLocalIPv6Address() {
+  return sHasNonLocalIPv6;
+}
