@@ -7,13 +7,12 @@
 /* import-globals-from head_cache.js */
 /* import-globals-from head_cookies.js */
 /* import-globals-from head_channels.js */
-
 /* globals require, __dirname, global, Buffer, process */
 
-const { NodeServer } = ChromeUtils.importESModule(
-  "resource://testing-common/httpd.sys.mjs"
-);
-const { AppConstants } = ChromeUtils.importESModule(
+var { NodeHTTP2Server: TRRNodeHttp2Server, NodeServer: TRRNodeServer } =
+  ChromeUtils.importESModule("resource://testing-common/NodeServer.sys.mjs");
+
+const { AppConstants: TRRAppConstants } = ChromeUtils.importESModule(
   "resource://gre/modules/AppConstants.sys.mjs"
 );
 
@@ -31,7 +30,7 @@ function trr_test_setup() {
 
   Services.prefs.setBoolPref("network.http.http2.enabled", true);
   // the TRR server is on 127.0.0.1
-  if (AppConstants.platform == "android") {
+  if (TRRAppConstants.platform == "android") {
     Services.prefs.setCharPref("network.trr.bootstrapAddr", "10.0.2.2");
   } else {
     Services.prefs.setCharPref("network.trr.bootstrapAddr", "127.0.0.1");
@@ -88,7 +87,6 @@ function trr_clear_prefs() {
   Services.prefs.clearUserPref("network.trr.excluded-domains");
   Services.prefs.clearUserPref("network.trr.builtin-excluded-domains");
   Services.prefs.clearUserPref("network.trr.clear-cache-on-pref-change");
-  Services.prefs.clearUserPref("network.trr.fetch_off_main_thread");
   Services.prefs.clearUserPref("captivedetect.canonicalURL");
 
   Services.prefs.clearUserPref("network.http.http2.enabled");
@@ -152,7 +150,7 @@ class TRRDNSListener {
         this.additionalInfo,
         this,
         currentThread,
-        {} // defaultOriginAttributes
+        this.options.originAttributes || {} // defaultOriginAttributes
       );
       Assert.ok(!this.options.expectEarlyFail, "asyncResolve ok");
     } catch (e) {
@@ -162,8 +160,9 @@ class TRRDNSListener {
   }
 
   onLookupComplete(inRequest, inRecord, inStatus) {
-    Assert.ok(
-      inRequest == this.request,
+    Assert.equal(
+      inRequest,
+      this.request,
       "Checking that this is the correct callback"
     );
 
@@ -247,88 +246,42 @@ class TRRDNSListener {
   }
 }
 
-/// Implements a basic HTTP2 server
-class TRRServerCode {
-  static async startServer(port) {
-    const fs = require("fs");
-    const options = {
-      key: fs.readFileSync(__dirname + "/http2-cert.key"),
-      cert: fs.readFileSync(__dirname + "/http2-cert.pem"),
-    };
-
-    const url = require("url");
-    global.path_handlers = {};
-    global.handler = (req, resp) => {
-      const path = req.headers[global.http2.constants.HTTP2_HEADER_PATH];
-      let u = url.parse(req.url, true);
-      let handler = global.path_handlers[u.pathname];
-      if (handler) {
-        handler(req, resp, u);
-        return;
-      }
-
-      // Didn't find a handler for this path.
-      let response = `<h1> 404 Path not found: ${path}</h1>`;
-      resp.setHeader("Content-Type", "text/html");
-      resp.setHeader("Content-Length", response.length);
-      resp.writeHead(404);
-      resp.end(response);
-    };
-
-    // key: string "name/type"
-    // value: array [answer1, answer2]
-    global.dns_query_answers = {};
-
-    // key: domain
-    // value: a map containing {key: type, value: number of requests}
-    global.dns_query_counts = {};
-
-    global.http2 = require("http2");
-    global.server = global.http2.createSecureServer(options, global.handler);
-
-    await global.server.listen(port);
-
-    global.dnsPacket = require(`${__dirname}/../dns-packet`);
-    global.ip = require(`${__dirname}/../node_ip`);
-
-    let serverPort = global.server.address().port;
-
-    if (process.env.MOZ_ANDROID_DATA_DIR) {
-      // When creating a server on Android we must make sure that the port
-      // is forwarded from the host machine to the emulator.
-      let adb_path = "adb";
-      if (process.env.MOZ_FETCHES_DIR) {
-        adb_path = `${process.env.MOZ_FETCHES_DIR}/android-sdk-linux/platform-tools/adb`;
-      }
-
-      await new Promise(resolve => {
-        const { exec } = require("child_process");
-        exec(
-          `${adb_path} reverse tcp:${serverPort} tcp:${serverPort}`,
-          (error, stdout, stderr) => {
-            if (error) {
-              console.log(`error: ${error.message}`);
-              return;
-            }
-            if (stderr) {
-              console.log(`stderr: ${stderr}`);
-            }
-            // console.log(`stdout: ${stdout}`);
-            resolve();
-          }
-        );
-      });
-    }
-
-    return serverPort;
+// This is for reteriiving the raw bytes from a DNS answer.
+function answerHandler(req, resp) {
+  let searchParams = new URL(req.url, "http://example.com").searchParams;
+  console.log("req.searchParams:" + searchParams);
+  if (!searchParams.get("host")) {
+    resp.writeHead(400);
+    resp.end("Missing search parameter");
+    return;
   }
 
-  static getRequestCount(domain, type) {
-    if (!global.dns_query_counts[domain]) {
-      return 0;
-    }
-    return global.dns_query_counts[domain][type] || 0;
+  function processRequest(req1, resp1) {
+    let domain = searchParams.get("host");
+    let type = searchParams.get("type");
+    let response = global.dns_query_answers[`${domain}/${type}`] || {};
+    let buf = global.dnsPacket.encode({
+      type: "response",
+      id: 0,
+      flags: 0,
+      questions: [],
+      answers: response.answers || [],
+      additionals: response.additionals || [],
+    });
+    let writeResponse = (resp2, buf2) => {
+      try {
+        let data = buf2.toString("hex");
+        resp2.setHeader("Content-Length", data.length);
+        resp2.writeHead(200, { "Content-Type": "plain/text" });
+        resp2.write(data);
+        resp2.end("");
+      } catch (e) {}
+    };
+
+    writeResponse(resp1, buf, response);
   }
+
+  processRequest(req, resp);
 }
 
 /// This is the default handler for /dns-query
@@ -361,11 +314,30 @@ function trrQueryHandler(req, resp, url) {
     resp.end("Unexpected method");
   }
 
-  function processRequest(req, resp, payload) {
+  function processRequest(req1, resp1, payload) {
     let dnsQuery = global.dnsPacket.decode(payload);
     let domain = dnsQuery.questions[0].name;
     let type = dnsQuery.questions[0].type;
     let response = global.dns_query_answers[`${domain}/${type}`] || {};
+    let delay = response.delay || 0;
+    let searchParams = new URL(req1.url, "http://example.com").searchParams;
+    if (searchParams.get("conncycle")) {
+      if (domain.startsWith("newconn")) {
+        // If we haven't seen a req for this newconn name before,
+        // or if we've seen one for the same name on the same port,
+        // synthesize a timeout.
+        if (
+          !global.gDoHNewConnLog[domain] ||
+          global.gDoHNewConnLog[domain] == req1.socket.remotePort
+        ) {
+          delay = 1000;
+        }
+        if (!global.gDoHNewConnLog[domain]) {
+          global.gDoHNewConnLog[domain] = req1.socket.remotePort;
+        }
+      }
+      global.gDoHPortsLog.push([domain, req1.socket.remotePort]);
+    }
 
     if (!global.dns_query_counts[domain]) {
       global.dns_query_counts[domain] = {};
@@ -387,28 +359,28 @@ function trrQueryHandler(req, resp, url) {
       additionals: response.additionals || [],
     });
 
-    let writeResponse = (resp, buf, context) => {
+    let writeResponse = (resp2, buf2, context) => {
       try {
         if (context.error) {
           // If the error is a valid HTTP response number just write it out.
           if (context.error < 600) {
-            resp.writeHead(context.error);
-            resp.end("Intentional error");
+            resp2.writeHead(context.error);
+            resp2.end("Intentional error");
             return;
           }
 
           // Bigger error means force close the session
-          req.stream.session.close();
+          req1.stream.session.close();
           return;
         }
-        resp.setHeader("Content-Length", buf.length);
-        resp.writeHead(200, { "Content-Type": "application/dns-message" });
-        resp.write(buf);
-        resp.end("");
+        resp2.setHeader("Content-Length", buf2.length);
+        resp2.writeHead(200, { "Content-Type": "application/dns-message" });
+        resp2.write(buf2);
+        resp2.end("");
       } catch (e) {}
     };
 
-    if (response.delay) {
+    if (delay) {
       // This function is handled within the httpserver where setTimeout is
       // available.
       // eslint-disable-next-line no-undef
@@ -416,49 +388,50 @@ function trrQueryHandler(req, resp, url) {
         arg => {
           writeResponse(arg[0], arg[1], arg[2]);
         },
-        response.delay,
-        [resp, buf, response]
+        delay,
+        [resp1, buf, response]
       );
       return;
     }
 
-    writeResponse(resp, buf, response);
+    writeResponse(resp1, buf, response);
   }
 }
 
+function getRequestCount(domain, type) {
+  if (!global.dns_query_counts[domain]) {
+    return 0;
+  }
+  return global.dns_query_counts[domain][type] || 0;
+}
+
 // A convenient wrapper around NodeServer
-class TRRServer {
+class TRRServer extends TRRNodeHttp2Server {
   /// Starts the server
   /// @port - default 0
   ///    when provided, will attempt to listen on that port.
   async start(port = 0) {
-    this.processId = await NodeServer.fork();
+    await super.start(port);
+    await this.execute(`( () => {
+      // key: string "name/type"
+      // value: array [answer1, answer2]
+      global.dns_query_answers = {};
 
-    await this.execute(TRRServerCode);
-    this.port = await this.execute(`TRRServerCode.startServer(${port})`);
+      // key: domain
+      // value: a map containing {key: type, value: number of requests}
+      global.dns_query_counts = {};
+
+      global.gDoHPortsLog = [];
+      global.gDoHNewConnLog = {};
+
+      global.dnsPacket = require(\`\${__dirname}/../dns-packet\`);
+      global.ip = require(\`\${__dirname}/../node_ip\`);
+      global.http2 = require("http2");
+      global.url = require("url");
+    })()`);
     await this.registerPathHandler("/dns-query", trrQueryHandler);
-  }
-
-  /// Executes a command in the context of the node server
-  async execute(command) {
-    return NodeServer.execute(this.processId, command);
-  }
-
-  /// Stops the server
-  async stop() {
-    if (this.processId) {
-      await NodeServer.kill(this.processId);
-      this.processId = undefined;
-    }
-  }
-
-  /// @path : string - the path on the server that we're handling. ex: /path
-  /// @handler : function(req, resp, url) - function that processes request and
-  ///     emits a response.
-  async registerPathHandler(path, handler) {
-    return this.execute(
-      `global.path_handlers["${path}"] = ${handler.toString()}`
-    );
+    await this.registerPathHandler("/dnsAnswer", answerHandler);
+    await this.execute(getRequestCount);
   }
 
   /// @name : string - name we're providing answers for. eg: foo.example.com
@@ -484,9 +457,7 @@ class TRRServer {
   }
 
   async requestCount(domain, type) {
-    return this.execute(
-      `TRRServerCode.getRequestCount("${domain}", "${type}")`
-    );
+    return this.execute(`getRequestCount("${domain}", "${type}")`);
   }
 }
 
@@ -517,21 +488,16 @@ class TRRProxyCode {
     });
   }
 
-  static proxySessionCount() {
-    if (!global.proxy) {
-      return 0;
-    }
-    return global.proxy.proxy_session_count;
+  static proxyRequestCount() {
+    return global.proxy_stream_count;
   }
 
   static setupProxy() {
     if (!global.proxy) {
       throw new Error("proxy is null");
     }
-    global.proxy.proxy_session_count = 0;
-    global.proxy.on("session", () => {
-      ++global.proxy.proxy_session_count;
-    });
+
+    global.proxy_stream_count = 0;
 
     // We need to track active connections so we can forcefully close keep-alive
     // connections when shutting down the proxy.
@@ -557,7 +523,7 @@ class TRRProxyCode {
         stream.end();
         return;
       }
-
+      global.proxy_stream_count++;
       const net = require("net");
       const socket = net.connect(global.endServerPort, "127.0.0.1", () => {
         try {
@@ -582,32 +548,31 @@ class TRRProxy {
   // Starts the proxy
   async start(port) {
     info("TRRProxy start!");
-    this.processId = await NodeServer.fork();
+    this.processId = await TRRNodeServer.fork();
     info("processid=" + this.processId);
     await this.execute(TRRProxyCode);
     this.port = await this.execute(`TRRProxyCode.startServer(${port})`);
     Assert.notEqual(this.port, null);
-    this.initial_session_count = 0;
   }
 
   // Executes a command in the context of the node server
   async execute(command) {
-    return NodeServer.execute(this.processId, command);
+    return TRRNodeServer.execute(this.processId, command);
   }
 
   // Stops the server
   async stop() {
     if (this.processId) {
-      await NodeServer.execute(this.processId, `TRRProxyCode.closeProxy()`);
-      await NodeServer.kill(this.processId);
+      await TRRNodeServer.execute(this.processId, `TRRProxyCode.closeProxy()`);
+      await TRRNodeServer.kill(this.processId);
     }
   }
 
-  async proxy_session_counter() {
-    let data = await NodeServer.execute(
+  async request_count() {
+    let data = await TRRNodeServer.execute(
       this.processId,
-      `TRRProxyCode.proxySessionCount()`
+      `TRRProxyCode.proxyRequestCount()`
     );
-    return parseInt(data) - this.initial_session_count;
+    return parseInt(data);
   }
 }
