@@ -38,11 +38,11 @@ ValidityChecker.prototype = {
 
   QueryInterface: ChromeUtils.generateQI(["nsICacheEntryOpenCallback"]),
 
-  onCacheEntryCheck(entry) {
+  onCacheEntryCheck() {
     return Ci.nsICacheEntryOpenCallback.ENTRY_WANTED;
   },
 
-  onCacheEntryAvailable(entry, isnew, status) {
+  onCacheEntryAvailable(entry) {
     // Check if forced valid
     Assert.equal(entry.isForcedValid, this.httpStatus === 200);
     this.verifier.maybe_run_next_test();
@@ -155,7 +155,7 @@ var prepListener = {
     this.continueCallback = cb;
   },
 
-  onCacheEntryCheck(entry) {
+  onCacheEntryCheck() {
     return Ci.nsICacheEntryOpenCallback.ENTRY_WANTED;
   },
 
@@ -184,28 +184,6 @@ function open_and_continue(uris, continueCallback) {
   }
 }
 
-function test_link_hover() {
-  if (!running_single_process && !is_child_process()) {
-    // This one we can just proxy to the child and be done with, no extra setup
-    // is necessary.
-    sendCommand("test_link_hover();");
-    return;
-  }
-
-  var uri = newURI("http://localhost:4444/foo/bar");
-  var referrer = newURI("http://localhost:4444/foo");
-  var preconns = ["http://localhost:4444"];
-
-  var verifier = new Verifier("hover", [], preconns, []);
-  predictor.predict(
-    uri,
-    referrer,
-    predictor.PREDICT_LINK,
-    origin_attributes,
-    verifier
-  );
-}
-
 const pageload_toplevel = newURI("http://localhost:4444/index.html");
 
 function continue_test_pageload() {
@@ -492,9 +470,9 @@ function continue_test_origin() {
         origin_attributes
       );
       do_timeout(0, () => {
-        var origin = extract_origin(sruri);
-        if (!preconns.includes(origin)) {
-          preconns.push(origin);
+        var origin1 = extract_origin(sruri);
+        if (!preconns.includes(origin1)) {
+          preconns.push(origin1);
         }
 
         sruri = newURI(subresources[2]);
@@ -505,9 +483,9 @@ function continue_test_origin() {
           origin_attributes
         );
         do_timeout(0, () => {
-          var origin = extract_origin(sruri);
-          if (!preconns.includes(origin)) {
-            preconns.push(origin);
+          var origin2 = extract_origin(sruri);
+          if (!preconns.includes(origin2)) {
+            preconns.push(origin2);
           }
 
           var loaduri = newURI("http://localhost:4444/anotherpage.html");
@@ -555,7 +533,7 @@ var prefetchListener = {
     read_stream(stream, cnt);
   },
 
-  onStopRequest(request, status) {
+  onStopRequest() {
     run_next_test();
   },
 };
@@ -708,9 +686,9 @@ function test_visitor_doom() {
         console.debug("asyncDoomURI", aURI.spec);
         let doomTask = Promise.all(
           storages.map(storage => {
-            return new Promise(resolve => {
+            return new Promise(resolve1 => {
               storage.asyncDoomURI(aURI, aIdEnhance, {
-                onCacheEntryDoomed: resolve,
+                onCacheEntryDoomed: resolve1,
               });
             });
           })
@@ -736,17 +714,7 @@ function test_visitor_doom() {
       let entryCount = 0;
       let visitor = {
         onCacheStorageInfo() {},
-        async onCacheEntryInfo(
-          aURI,
-          aIdEnhance,
-          aDataSize,
-          aAltDataSize,
-          aFetchCount,
-          aLastModifiedTime,
-          aExpirationTime,
-          aPinned,
-          aInfo
-        ) {
+        async onCacheEntryInfo() {
           entryCount++;
         },
         onCacheEntryVisitCompleted() {
@@ -775,7 +743,6 @@ function cleanup() {
 var tests = [
   // This must ALWAYS come first, to ensure a clean slate
   reset_predictor,
-  test_link_hover,
   test_pageload,
   // TODO: These are disabled until the features are re-written
   //test_redirect,
@@ -797,7 +764,7 @@ var observer = {
 
   QueryInterface: ChromeUtils.generateQI(["nsIObserver"]),
 
-  observe(subject, topic, data) {
+  observe(subject, topic) {
     if (topic != "predictor-reset-complete") {
       return;
     }
