@@ -2,51 +2,41 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-
-import json
 import logging
 import os
 import shutil
 import sys
 import time
 from collections import defaultdict
+from pathlib import Path
 
 import yaml
 from redo import retry
 from taskgraph import create
 from taskgraph.create import create_tasks
-
-# TODO: Let standalone taskgraph generate parameters instead of calling internals
-from taskgraph.decision import (
-    _determine_more_accurate_base_ref,
-    _determine_more_accurate_base_rev,
-    _get_env_prefix,
-)
 from taskgraph.generator import TaskGraphGenerator
+from taskgraph.main import format_kind_graph_mermaid
 from taskgraph.parameters import Parameters
 from taskgraph.taskgraph import TaskGraph
+from taskgraph.util import json
 from taskgraph.util.python_path import find_object
-from taskgraph.util.schema import Schema, validate_schema
 from taskgraph.util.taskcluster import get_artifact
 from taskgraph.util.vcs import get_repository
 from taskgraph.util.yaml import load_yaml
-from voluptuous import Any, Optional, Required
 
 from . import GECKO
 from .actions import render_actions_json
 from .parameters import get_app_version, get_version
-from .try_option_syntax import parse_message
-from .util.backstop import BACKSTOP_INDEX, is_backstop
+from .util.backstop import ANDROID_PERFTEST_BACKSTOP_INDEX, BACKSTOP_INDEX, is_backstop
 from .util.bugbug import push_schedules
-from .util.chunking import resolver
-from .util.hg import get_hg_commit_message, get_hg_revision_branch
+from .util.hg import get_hg_revision_branch, get_hg_revision_info
 from .util.partials import populate_release_history
 from .util.taskcluster import insert_index
 from .util.taskgraph import find_decision_task, find_existing_tasks_from_previous_kinds
 
 logger = logging.getLogger(__name__)
 
-ARTIFACTS_DIR = "artifacts"
+ARTIFACTS_DIR = os.environ.get("MOZ_UPLOAD_DIR", "artifacts")
 
 # For each project, this gives a set of parameters specific to the project.
 # See `taskcluster/docs/parameters.rst` for information on parameters.
@@ -54,6 +44,7 @@ PER_PROJECT_PARAMETERS = {
     "try": {
         "enable_always_target": True,
         "target_tasks_method": "try_tasks",
+        "release_type": "nightly",
     },
     "kaios-try": {
         "target_tasks_method": "try_tasks",
@@ -92,75 +83,43 @@ PER_PROJECT_PARAMETERS = {
         "target_tasks_method": "mozilla_release_tasks",
         "release_type": "release",
     },
-    "mozilla-esr102": {
-        "target_tasks_method": "mozilla_esr102_tasks",
-        "release_type": "esr102",
-    },
-    "mozilla-esr115": {
-        "target_tasks_method": "mozilla_esr115_tasks",
-        "release_type": "esr115",
+    "mozilla-esr140": {
+        "target_tasks_method": "mozilla_esr140_tasks",
+        "release_type": "esr140",
     },
     "pine": {
-        "target_tasks_method": "default",
+        "target_tasks_method": "pine_tasks",
         "release_type": "nightly-pine",
     },
+    "maple": {
+        # Prevent it from running everything - For now just use "Add New Jobs"
+        "target_tasks_method": "nothing",
+        "release_type": "release",
+    },
+    "cypress": {
+        "target_tasks_method": "cypress_tasks",
+        "release_type": "nightly-cypress",
+    },
+    "larch": {
+        "target_tasks_method": "larch_tasks",
+        "release_type": "nightly-larch",
+    },
     "kaios": {
         "target_tasks_method": "kaios_tasks",
     },
     "toolchains": {
         "target_tasks_method": "mozilla_central_tasks",
     },
+    # git projects
+    "staging-firefox": {
+        "target_tasks_method": "default",
+    },
     # the default parameters are used for projects that do not match above.
     "default": {
         "target_tasks_method": "default",
     },
 }
 
-try_task_config_schema = Schema(
-    {
-        Required("tasks"): [str],
-        Optional("browsertime"): bool,
-        Optional("chemspill-prio"): bool,
-        Optional("disable-pgo"): bool,
-        Optional("env"): {str: str},
-        Optional("gecko-profile"): bool,
-        Optional("gecko-profile-interval"): float,
-        Optional("gecko-profile-entries"): int,
-        Optional("gecko-profile-features"): str,
-        Optional("gecko-profile-threads"): str,
-        Optional(
-            "perftest-options",
-            description="Options passed from `mach perftest` to try.",
-        ): object,
-        Optional(
-            "optimize-strategies",
-            description="Alternative optimization strategies to use instead of the default. "
-            "A module path pointing to a dict to be use as the `strategy_override` "
-            "argument in `taskgraph.optimize.base.optimize_task_graph`.",
-        ): str,
-        Optional("rebuild"): int,
-        Optional("tasks-regex"): {
-            "include": Any(None, [str]),
-            "exclude": Any(None, [str]),
-        },
-        Optional("use-artifact-builds"): bool,
-        Optional(
-            "worker-overrides",
-            description="Mapping of worker alias to worker pools to use for those aliases.",
-        ): {str: str},
-        Optional("routes"): [str],
-    }
-)
-"""
-Schema for try_task_config.json files.
-"""
-
-try_task_config_schema_v2 = Schema(
-    {
-        Optional("parameters"): {str: object},
-    }
-)
-
 
 def full_task_graph_to_runnable_jobs(full_task_json):
     runnable_jobs = {}
@@ -225,6 +184,7 @@ def taskgraph_decision(options, parameters=None):
         parameters=parameters,
         decision_task_id=decision_task_id,
         write_artifacts=True,
+        enable_verifications=options.get("verify", True),
     )
 
     if not create.testing:
@@ -244,6 +204,9 @@ def taskgraph_decision(options, parameters=None):
     full_task_json = tgg.full_task_graph.to_json()
     write_artifact("full-task-graph.json", full_task_json)
 
+    # write out kind graph
+    write_artifact("kind-graph.mm", format_kind_graph_mermaid(tgg.kind_graph))
+
     # write out the public/runnable-jobs.json file
     write_artifact(
         "runnable-jobs.json", full_task_graph_to_runnable_jobs(full_task_json)
@@ -255,8 +218,11 @@ def taskgraph_decision(options, parameters=None):
         full_task_graph_to_manifests_by_task(full_task_json),
     )
 
-    # write out the public/tests-by-manifest.json file
-    write_artifact("tests-by-manifest.json.gz", resolver.tests_by_manifest)
+    # `tests-by-manifest.json.gz` was previously written out here
+    # it was moved to `loader/test.py` because its contents now depend on
+    # data generated in a subprocess which we do not have access to here
+    # see https://bugzilla.mozilla.org/show_bug.cgi?id=1989038 for additional
+    # details
 
     # this is just a test to check whether the from_json() function is working
     _, _ = TaskGraph.from_json(full_task_json)
@@ -273,12 +239,21 @@ def taskgraph_decision(options, parameters=None):
     if len(push_schedules) > 0:
         write_artifact("bugbug-push-schedules.json", push_schedules.popitem()[1])
 
-    # cache run-task & misc/fetch-content
-    scripts_root_dir = os.path.join(GECKO, "taskcluster/scripts")
-    run_task_file_path = os.path.join(scripts_root_dir, "run-task")
-    fetch_content_file_path = os.path.join(scripts_root_dir, "misc/fetch-content")
-    shutil.copy2(run_task_file_path, ARTIFACTS_DIR)
-    shutil.copy2(fetch_content_file_path, ARTIFACTS_DIR)
+    # upload run-task, fetch-content, robustcheckout.py and more as artifacts
+    mozharness_dir = Path(GECKO, "testing", "mozharness")
+    scripts_dir = Path(GECKO, "taskcluster", "scripts")
+    taskgraph_dir = Path(
+        GECKO, "third_party", "python", "taskcluster_taskgraph", "taskgraph"
+    )
+    to_copy = {
+        scripts_dir / "run-task": f"{ARTIFACTS_DIR}/run-task-hg",
+        scripts_dir / "tester" / "test-linux.sh": ARTIFACTS_DIR,
+        taskgraph_dir / "run-task" / "fetch-content": ARTIFACTS_DIR,
+        taskgraph_dir / "run-task" / "run-task": f"{ARTIFACTS_DIR}/run-task-git",
+        mozharness_dir / "external_tools" / "robustcheckout.py": ARTIFACTS_DIR,
+    }
+    for target, dest in to_copy.items():
+        shutil.copy2(target, dest)
 
     # actually create the graph
     create_tasks(
@@ -320,24 +295,42 @@ def get_decision_parameters(graph_config, options):
         if n in options
     }
 
-    commit_message = get_hg_commit_message(os.path.join(GECKO, product_dir))
-
     repo_path = os.getcwd()
     repo = get_repository(repo_path)
-    parameters["base_ref"] = _determine_more_accurate_base_ref(
-        repo,
-        candidate_base_ref=options.get("base_ref"),
-        head_ref=options.get("head_ref"),
-        base_rev=options.get("base_rev"),
-    )
 
-    parameters["base_rev"] = _determine_more_accurate_base_rev(
-        repo,
-        base_ref=parameters["base_ref"],
-        candidate_base_rev=options.get("base_rev"),
-        head_rev=options.get("head_rev"),
-        env_prefix=_get_env_prefix(graph_config),
-    )
+    try:
+        commit_message = repo.get_commit_message()
+    except UnicodeDecodeError:
+        commit_message = ""
+
+    # Set some vcs specific parameters
+    if parameters["repository_type"] == "hg":
+        if head_git_rev := get_hg_revision_info(
+            GECKO, revision=parameters["head_rev"], info="extras.git_commit"
+        ):
+            parameters["head_git_rev"] = head_git_rev
+
+        parameters["hg_branch"] = get_hg_revision_branch(
+            GECKO, revision=parameters["head_rev"]
+        )
+
+        changed_files_since_base = set(
+            repo.get_changed_files(
+                rev=parameters["head_rev"], base=parameters["base_rev"]
+            )
+        )
+        if "try" in parameters["project"] and options["tasks_for"] == "hg-push":
+            parameters["files_changed"] = sorted(
+                set(repo.get_outgoing_files()) | changed_files_since_base
+            )
+        else:
+            parameters["files_changed"] = sorted(changed_files_since_base)
+
+    elif parameters["repository_type"] == "git":
+        parameters["hg_branch"] = None
+        parameters["files_changed"] = repo.get_changed_files(
+            rev=parameters["head_rev"], base=parameters["base_rev"]
+        )
 
     # Define default filter list, as most configurations shouldn't need
     # custom filters.
@@ -351,9 +344,6 @@ def get_decision_parameters(graph_config, options):
     parameters["version"] = get_version(product_dir)
     parameters["app_version"] = get_app_version(product_dir)
     parameters["message"] = try_syntax_from_message(commit_message)
-    parameters["hg_branch"] = get_hg_revision_branch(
-        GECKO, revision=parameters["head_rev"]
-    )
     parameters["next_version"] = None
     parameters["optimize_strategies"] = None
     parameters["optimize_target_tasks"] = True
@@ -372,7 +362,6 @@ def get_decision_parameters(graph_config, options):
     parameters["test_manifest_loader"] = "default"
     parameters["try_mode"] = None
     parameters["try_task_config"] = {}
-    parameters["try_options"] = None
 
     # owner must be an email, but sometimes (e.g., for ffxbld) it is not, in which
     # case, fake it
@@ -391,9 +380,9 @@ def get_decision_parameters(graph_config, options):
         parameters.update(PER_PROJECT_PARAMETERS[project])
     except KeyError:
         logger.warning(
-            "using default project parameters; add {} to "
-            "PER_PROJECT_PARAMETERS in {} to customize behavior "
-            "for this project".format(project, __file__)
+            f"using default project parameters; add {project} to "
+            f"PER_PROJECT_PARAMETERS in {__file__} to customize behavior "
+            "for this project"
         )
         parameters.update(PER_PROJECT_PARAMETERS["default"])
 
@@ -433,6 +422,14 @@ def get_decision_parameters(graph_config, options):
     # Determine if this should be a backstop push.
     parameters["backstop"] = is_backstop(parameters)
 
+    # For the android perf tasks, run them 50% less often
+    parameters["android_perftest_backstop"] = is_backstop(
+        parameters,
+        push_interval=30,
+        time_interval=60 * 6,
+        backstop_strategy="android_perftest_backstop",
+    )
+
     if "decision-parameters" in graph_config["taskgraph"]:
         find_object(graph_config["taskgraph"]["decision-parameters"])(
             graph_config, parameters
@@ -474,54 +471,33 @@ def set_try_config(parameters, task_config_file):
             task_config = json.load(fh)
         task_config_version = task_config.pop("version", 1)
         if task_config_version == 1:
-            validate_schema(
-                try_task_config_schema,
-                task_config,
-                "Invalid v1 `try_task_config.json`.",
-            )
             parameters["try_mode"] = "try_task_config"
             parameters["try_task_config"] = task_config
         elif task_config_version == 2:
-            validate_schema(
-                try_task_config_schema_v2,
-                task_config,
-                "Invalid v2 `try_task_config.json`.",
-            )
             parameters.update(task_config["parameters"])
-            return
+            parameters["try_mode"] = "try_task_config"
         else:
             raise Exception(
                 f"Unknown `try_task_config.json` version: {task_config_version}"
             )
 
-    if "try:" in parameters["message"]:
-        parameters["try_mode"] = "try_option_syntax"
-        parameters.update(parse_message(parameters["message"]))
-    else:
-        parameters["try_options"] = None
-
-    if parameters["try_mode"] == "try_task_config":
-        # The user has explicitly requested a set of jobs, so run them all
-        # regardless of optimization.  Their dependencies can be optimized,
-        # though.
-        parameters["optimize_target_tasks"] = False
-    else:
-        # For a try push with no task selection, apply the default optimization
-        # process to all of the tasks.
-        parameters["optimize_target_tasks"] = True
-
 
 def set_decision_indexes(decision_task_id, params, graph_config):
     index_paths = []
+    if params["android_perftest_backstop"]:
+        index_paths.insert(0, ANDROID_PERFTEST_BACKSTOP_INDEX)
     if params["backstop"]:
-        index_paths.append(BACKSTOP_INDEX)
+        # When two Decision tasks run at nearly the same time, it's possible
+        # they both end up being backstops if the second checks the backstop
+        # index before the first inserts it. Insert this index first to reduce
+        # the chances of that happening.
+        index_paths.insert(0, BACKSTOP_INDEX)
 
     subs = params.copy()
     subs["trust-domain"] = graph_config["trust-domain"]
 
-    index_paths = [i.format(**subs) for i in index_paths]
     for index_path in index_paths:
-        insert_index(index_path, decision_task_id, use_proxy=True)
+        insert_index(index_path.format(**subs), decision_task_id)
 
 
 def write_artifact(filename, data):
@@ -534,14 +510,15 @@ def write_artifact(filename, data):
             yaml.safe_dump(data, f, allow_unicode=True, default_flow_style=False)
     elif filename.endswith(".json"):
         with open(path, "w") as f:
-            json.dump(data, f, sort_keys=True, indent=2, separators=(",", ": "))
+            json.dump(data, f)
     elif filename.endswith(".json.gz"):
         import gzip
 
         with gzip.open(path, "wb") as f:
             f.write(json.dumps(data).encode("utf-8"))
     else:
-        raise TypeError(f"Don't know how to write to {filename}")
+        with open(path, "w") as f:
+            f.write(data)
 
 
 def read_artifact(filename):
@@ -555,7 +532,7 @@ def read_artifact(filename):
         import gzip
 
         with gzip.open(path, "rb") as f:
-            return json.load(f.decode("utf-8"))
+            return json.load(f)
     else:
         raise TypeError(f"Don't know how to read {filename}")
 
