@@ -4,63 +4,35 @@
 
 import argparse
 import atexit
-import json
 import logging
 import os
-import re
 import shutil
 import subprocess
 import sys
 import tempfile
 import traceback
-from collections import namedtuple
-from concurrent.futures import ProcessPoolExecutor, as_completed
 from pathlib import Path
-from typing import Any, List
+from typing import Any
 
 import appdirs
 import yaml
+from taskgraph.main import (
+    FORMAT_METHODS,
+    argument,
+    command,
+    commands,
+    dump_output,
+    format_kind_graph_mermaid,
+    generate_taskgraph,
+    get_taskgraph_generator,
+)
 
-Command = namedtuple("Command", ["func", "args", "kwargs", "defaults"])
-commands = {}
-
-
-def command(*args, **kwargs):
-    defaults = kwargs.pop("defaults", {})
-
-    def decorator(func):
-        commands[args[0]] = Command(func, args, kwargs, defaults)
-        return func
-
-    return decorator
-
-
-def argument(*args, **kwargs):
-    def decorator(func):
-        if not hasattr(func, "args"):
-            func.args = []
-        func.args.append((args, kwargs))
-        return func
-
-    return decorator
-
-
-def format_taskgraph_labels(taskgraph):
-    return "\n".join(
-        sorted(
-            taskgraph.tasks[index].label for index in taskgraph.graph.visit_postorder()
-        )
-    )
-
-
-def format_taskgraph_json(taskgraph):
-    return json.dumps(
-        taskgraph.to_json(), sort_keys=True, indent=2, separators=(",", ": ")
-    )
+from gecko_taskgraph import GECKO
+from gecko_taskgraph.files_changed import get_locally_changed_files
 
 
 def format_taskgraph_yaml(taskgraph):
-    from mozbuild.util import ReadOnlyDict
+    from taskgraph.util.readonlydict import ReadOnlyDict
 
     class TGDumper(yaml.SafeDumper):
         def ignore_aliases(self, data):
@@ -74,157 +46,81 @@ def format_taskgraph_yaml(taskgraph):
     return yaml.dump(taskgraph.to_json(), Dumper=TGDumper, default_flow_style=False)
 
 
-def get_filtered_taskgraph(taskgraph, tasksregex, exclude_keys):
-    """
-    Filter all the tasks on basis of a regular expression
-    and returns a new TaskGraph object
-    """
-    from taskgraph.graph import Graph
-    from taskgraph.task import Task
-    from taskgraph.taskgraph import TaskGraph
-
-    if tasksregex:
-        named_links_dict = taskgraph.graph.named_links_dict()
-        filteredtasks = {}
-        filterededges = set()
-        regexprogram = re.compile(tasksregex)
-
-        for key in taskgraph.graph.visit_postorder():
-            task = taskgraph.tasks[key]
-            if regexprogram.match(task.label):
-                filteredtasks[key] = task
-                for depname, dep in named_links_dict[key].items():
-                    if regexprogram.match(dep):
-                        filterededges.add((key, dep, depname))
-
-        taskgraph = TaskGraph(filteredtasks, Graph(set(filteredtasks), filterededges))
-
-    if exclude_keys:
-        for label, task in taskgraph.tasks.items():
-            task_dict = task.to_json()
-            for key in exclude_keys:
-                obj = task_dict
-                attrs = key.split(".")
-                while attrs[0] in obj:
-                    if len(attrs) == 1:
-                        del obj[attrs[0]]
-                        break
-                    obj = obj[attrs[0]]
-                    attrs = attrs[1:]
-            taskgraph.tasks[label] = Task.from_json(task_dict)
-
-    return taskgraph
+FORMAT_METHODS["yaml"] = format_taskgraph_yaml
 
 
-FORMAT_METHODS = {
-    "labels": format_taskgraph_labels,
-    "json": format_taskgraph_json,
-    "yaml": format_taskgraph_yaml,
-}
+@command(
+    "kind-graph",
+    help="Show the kind dependency graph as a Mermaid flowchart diagram.",
+)
+@argument("--root", "-r", help="root of the taskgraph definition relative to topsrcdir")
+@argument("--quiet", "-q", action="store_true", help="suppress all logging output")
+@argument(
+    "--verbose", "-v", action="store_true", help="include debug-level logging output"
+)
+@argument(
+    "--parameters",
+    "-p",
+    default=None,
+    help="Parameters to use for the generation. Can be a path to file (.yml or "
+    ".json; see `taskcluster/docs/parameters.rst`), a url, of the form "
+    "`project=mozilla-central` to download latest parameters file for the specified "
+    "project from CI, or of the form `task-id=<decision task id>` to download "
+    "parameters from the specified decision task.",
+)
+@argument(
+    "-o",
+    "--output-file",
+    default=None,
+    help="file path to store generated output.",
+)
+@argument(
+    "-k",
+    "--target-kind",
+    dest="target_kinds",
+    action="append",
+    default=[],
+    help="only return kinds and their dependencies.",
+)
+def show_kinds(options):
+    from taskgraph.parameters import parameters_loader  # noqa: PLC0415
 
+    if options.pop("verbose", False):
+        logging.root.setLevel(logging.DEBUG)
 
-def get_taskgraph_generator(root, parameters):
-    """Helper function to make testing a little easier."""
-    from taskgraph.generator import TaskGraphGenerator
+    setup_logging()
 
-    return TaskGraphGenerator(root_dir=root, parameters=parameters)
-
-
-def format_taskgraph(options, parameters, logfile=None):
-    import taskgraph
-    from taskgraph.parameters import parameters_loader
-
-    if logfile:
-        handler = logging.FileHandler(logfile, mode="w")
-        if logging.root.handlers:
-            oldhandler = logging.root.handlers[-1]
-            logging.root.removeHandler(oldhandler)
-            handler.setFormatter(oldhandler.formatter)
-        logging.root.addHandler(handler)
-
-    if options["fast"]:
-        taskgraph.fast = True
-
-    if isinstance(parameters, str):
+    target_kinds = options.get("target_kinds", [])
+    parameters = options.get("parameters")
+    if not parameters:
+        parameters = parameters_loader(
+            None, strict=False, overrides={"target-kinds": target_kinds}
+        )
+    elif isinstance(parameters, str):
         parameters = parameters_loader(
             parameters,
-            overrides={"target-kind": options.get("target_kind")},
+            overrides={"target-kinds": target_kinds},
             strict=False,
         )
+    else:
+        # Parameters object already exists (from tests)
+        if target_kinds:
+            parameters["target-kinds"] = target_kinds
 
     tgg = get_taskgraph_generator(options.get("root"), parameters)
+    kind_graph = tgg.kind_graph
 
-    tg = getattr(tgg, options["graph_attr"])
-    tg = get_filtered_taskgraph(tg, options["tasks_regex"], options["exclude_keys"])
-    format_method = FORMAT_METHODS[options["format"] or "labels"]
-    return format_method(tg)
+    output = format_kind_graph_mermaid(kind_graph)
 
-
-def dump_output(out, path=None, params_spec=None):
-    from taskgraph.parameters import Parameters
-
-    params_name = Parameters.format_spec(params_spec)
-    fh = None
-    if path:
-        # Substitute params name into file path if necessary
-        if params_spec and "{params}" not in path:
-            name, ext = os.path.splitext(path)
-            name += "_{params}"
-            path = name + ext
-
-        path = path.format(params=params_name)
-        fh = open(path, "w")
+    output_file = options.get("output_file")
+    if output_file:
+        with open(output_file, "w") as fh:
+            print(output, file=fh)
+        print(f"Kind graph written to {output_file}", file=sys.stderr)
     else:
-        print(
-            "Dumping result with parameters from {}:".format(params_name),
-            file=sys.stderr,
-        )
-    print(out + "\n", file=fh)
+        print(output)
 
-
-def generate_taskgraph(options, parameters, logdir):
-    from taskgraph.parameters import Parameters
-
-    def logfile(spec):
-        """Determine logfile given a parameters specification."""
-        if logdir is None:
-            return None
-        return os.path.join(
-            logdir,
-            "{}_{}.log".format(options["graph_attr"], Parameters.format_spec(spec)),
-        )
-
-    # Don't bother using futures if there's only one parameter. This can make
-    # tracebacks a little more readable and avoids additional process overhead.
-    if len(parameters) == 1:
-        spec = parameters[0]
-        out = format_taskgraph(options, spec, logfile(spec))
-        dump_output(out, options["output_file"])
-        return
-
-    futures = {}
-    with ProcessPoolExecutor(max_workers=options["max_workers"]) as executor:
-        for spec in parameters:
-            f = executor.submit(format_taskgraph, options, spec, logfile(spec))
-            futures[f] = spec
-
-    for future in as_completed(futures):
-        output_file = options["output_file"]
-        spec = futures[future]
-        e = future.exception()
-        if e:
-            out = "".join(traceback.format_exception(type(e), e, e.__traceback__))
-            if options["diff"]:
-                # Dump to console so we don't accidentally diff the tracebacks.
-                output_file = None
-        else:
-            out = future.result()
-
-        dump_output(
-            out,
-            path=output_file,
-            params_spec=spec if len(parameters) > 1 else None,
-        )
+    return 0
 
 
 @command(
@@ -298,6 +194,15 @@ def generate_taskgraph(options, parameters, logdir):
     "generations will happen from the same invocation (one per parameters "
     "specified).",
 )
+@argument(
+    "--force-local-files-changed",
+    default=False,
+    action="store_true",
+    help="Compute the 'files-changed' parameter from local version control, "
+    "even when explicitly using a parameter set that already has it defined. "
+    "Note that this is already the default behaviour when no parameters are "
+    "specified.",
+)
 @argument(
     "--no-optimize",
     dest="optimize",
@@ -316,7 +221,7 @@ def generate_taskgraph(options, parameters, logdir):
     "--tasks-regex",
     "--tasks",
     default=None,
-    help="only return tasks with labels matching this regular " "expression.",
+    help="only return tasks with labels matching this regular expression.",
 )
 @argument(
     "--exclude-key",
@@ -327,8 +232,11 @@ def generate_taskgraph(options, parameters, logdir):
     "This is mainly useful with '--diff' to filter out expected differences.",
 )
 @argument(
+    "-k",
     "--target-kind",
-    default=None,
+    dest="target_kinds",
+    action="append",
+    default=[],
     help="only return tasks that are of the given kind, or their dependencies.",
 )
 @argument(
@@ -369,7 +277,11 @@ def show_taskgraph(options):
     output_file = options["output_file"]
 
     if options["diff"]:
-        repo = get_repository(os.getcwd())
+        # --root argument is taskgraph's config at <repo>/taskcluster
+        repo_root = os.getcwd()
+        if options["root"]:
+            repo_root = f"{options['root']}/.."
+        repo = get_repository(repo_root)
 
         if not repo.working_directory_clean():
             print(
@@ -383,25 +295,32 @@ def show_taskgraph(options):
         # branch or bookmark (which are both available on the VCS object)
         # as `branch` is preferable to a specific revision.
         cur_ref = repo.branch or repo.head_ref[:12]
+        cur_ref_file = cur_ref.replace("/", "_")
 
         diffdir = tempfile.mkdtemp()
         atexit.register(
             shutil.rmtree, diffdir
         )  # make sure the directory gets cleaned up
         options["output_file"] = os.path.join(
-            diffdir, f"{options['graph_attr']}_{cur_ref}"
+            diffdir, f"{options['graph_attr']}_{cur_ref_file}"
         )
         print(f"Generating {options['graph_attr']} @ {cur_ref}", file=sys.stderr)
 
-    parameters: List[Any[str, Parameters]] = options.pop("parameters")
+    overrides = {
+        "target-kinds": options.get("target_kinds"),
+    }
+    parameters: list[Any[str, Parameters]] = options.pop("parameters")
     if not parameters:
-        overrides = {
-            "target-kind": options.get("target_kind"),
-        }
         parameters = [
             parameters_loader(None, strict=False, overrides=overrides)
         ]  # will use default values
 
+        # This is the default behaviour anyway, so no need to re-compute.
+        options["force_local_files_changed"] = False
+
+    elif options["force_local_files_changed"]:
+        overrides["files-changed"] = sorted(get_locally_changed_files(GECKO))
+
     for param in parameters[:]:
         if isinstance(param, str) and os.path.isdir(param):
             parameters.remove(param)
@@ -427,7 +346,7 @@ def show_taskgraph(options):
         # to setup its `mach` based logging.
         setup_logging()
 
-    generate_taskgraph(options, parameters, logdir)
+    generate_taskgraph(options, parameters, overrides, logdir)
 
     if options["diff"]:
         assert diffdir is not None
@@ -435,8 +354,10 @@ def show_taskgraph(options):
 
         # Reload taskgraph modules to pick up changes and clear global state.
         for mod in sys.modules.copy():
-            if mod != __name__ and mod.split(".", 1)[0].endswith(
-                ("taskgraph", "mozbuild")
+            if (
+                mod != __name__
+                and mod != "taskgraph.main"
+                and mod.split(".", 1)[0].endswith(("taskgraph", "mozbuild"))
             ):
                 del sys.modules[mod]
 
@@ -450,14 +371,15 @@ def show_taskgraph(options):
         else:
             base_ref = options["diff"]
 
+        base_ref_file = base_ref.replace("/", "_")
         try:
             repo.update(base_ref)
             base_ref = repo.head_ref[:12]
             options["output_file"] = os.path.join(
-                diffdir, f"{options['graph_attr']}_{base_ref}"
+                diffdir, f"{options['graph_attr']}_{base_ref_file}"
             )
             print(f"Generating {options['graph_attr']} @ {base_ref}", file=sys.stderr)
-            generate_taskgraph(options, parameters, logdir)
+            generate_taskgraph(options, parameters, overrides, logdir)
         finally:
             repo.update(cur_ref)
 
@@ -472,8 +394,10 @@ def show_taskgraph(options):
 
         non_fatal_failures = []
         for spec in parameters:
-            base_path = os.path.join(diffdir, f"{options['graph_attr']}_{base_ref}")
-            cur_path = os.path.join(diffdir, f"{options['graph_attr']}_{cur_ref}")
+            base_path = os.path.join(
+                diffdir, f"{options['graph_attr']}_{base_ref_file}"
+            )
+            cur_path = os.path.join(diffdir, f"{options['graph_attr']}_{cur_ref_file}")
 
             params_name = None
             if len(parameters) > 1:
@@ -497,9 +421,8 @@ def show_taskgraph(options):
                 # CalledProcessError with a returncode > 1.
                 proc = subprocess.run(
                     diffcmd + [base_path, cur_path],
-                    stdout=subprocess.PIPE,
-                    stderr=subprocess.PIPE,
-                    universal_newlines=True,
+                    capture_output=True,
+                    text=True,
                     check=True,
                 )
                 diff_output = proc.stdout
@@ -532,13 +455,12 @@ def show_taskgraph(options):
 
         if options["format"] != "json":
             print(
-                "If you were expecting differences in task bodies "
-                'you should pass "-J"\n',
+                'If you were expecting differences in task bodies you should pass "-J"\n',
                 file=sys.stderr,
             )
 
     if len(parameters) > 1:
-        print("See '{}' for logs".format(logdir), file=sys.stderr)
+        print(f"See '{logdir}' for logs", file=sys.stderr)
 
 
 @command("build-image", help="Build a Docker image")
@@ -568,8 +490,7 @@ def build_image(args):
 )
 @argument(
     "--task-id",
-    help="Load the image at public/image.tar.zst in this task, "
-    "rather than searching the index",
+    help="Load the image at public/image.tar.zst in this task, rather than searching the index",
 )
 @argument(
     "-t",
@@ -586,7 +507,7 @@ def build_image(args):
     "or mozilla-inbound)",
 )
 def load_image(args):
-    from gecko_taskgraph.docker import load_image_by_name, load_image_by_task_id
+    from taskgraph.docker import load_image_by_name, load_image_by_task_id
 
     if not args.get("image_name") and not args.get("task_id"):
         print("Specify either IMAGE-NAME or TASK-ID")
@@ -606,11 +527,10 @@ def load_image(args):
 @command("image-digest", help="Print the digest of a docker image.")
 @argument(
     "image_name",
-    help="Print the digest of the image of this name based on the current "
-    "contents of the tree.",
+    help="Print the digest of the image of this name based on the current contents of the tree.",
 )
 def image_digest(args):
-    from gecko_taskgraph.docker import get_image_digest
+    from taskgraph.docker import get_image_digest
 
     try:
         digest = get_image_digest(args["image_name"])
@@ -670,6 +590,13 @@ def image_digest(args):
     "--tasks-for", required=True, help="the tasks_for value used to generate this task"
 )
 @argument("--try-task-config-file", help="path to try task configuration file")
+@argument(
+    "--no-verify",
+    dest="verify",
+    default=True,
+    action="store_false",
+    help="Skip graph verifications.",
+)
 def decision(options):
     from gecko_taskgraph.decision import taskgraph_decision
 
@@ -680,10 +607,12 @@ def decision(options):
 @argument(
     "--root",
     "-r",
-    default="taskcluster/ci",
+    default="taskcluster",
     help="root of the taskgraph definition relative to topsrcdir",
 )
 def action_callback(options):
+    from taskgraph.util import json
+
     from gecko_taskgraph.actions import trigger_action_callback
     from gecko_taskgraph.actions.util import get_parameters
 
@@ -716,14 +645,14 @@ def action_callback(options):
 @argument(
     "--root",
     "-r",
-    default="taskcluster/ci",
+    default="taskcluster",
     help="root of the taskgraph definition relative to topsrcdir",
 )
 @argument(
     "--parameters",
     "-p",
     default="",
-    help="parameters file (.yml or .json; see " "`taskcluster/docs/parameters.rst`)`",
+    help="parameters file (.yml or .json; see `taskcluster/docs/parameters.rst`)`",
 )
 @argument("--task-id", default=None, help="TaskId to which the action applies")
 @argument(
@@ -734,7 +663,7 @@ def action_callback(options):
 def test_action_callback(options):
     import taskgraph.parameters
     from taskgraph.config import load_graph_config
-    from taskgraph.util import yaml
+    from taskgraph.util import json, yaml
 
     import gecko_taskgraph.actions
 
