@@ -2,21 +2,25 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+import copy
 import hashlib
-import json
 import re
 
 from mozbuild.schedules import INCLUSIVE_COMPONENTS
-from mozbuild.util import ReadOnlyDict
 from taskgraph.transforms.base import TransformSequence
+from taskgraph.util import json
 from taskgraph.util.attributes import keymatch
 from taskgraph.util.keyed_by import evaluate_keyed_by
+from taskgraph.util.readonlydict import ReadOnlyDict
 from taskgraph.util.schema import Schema, resolve_keyed_by
-from taskgraph.util.taskcluster import get_artifact_path, get_index_url
+from taskgraph.util.taskcluster import get_artifact_path
+from taskgraph.util.templates import merge
 from voluptuous import Any, Optional, Required
 
 from gecko_taskgraph.transforms.test.variant import TEST_VARIANTS
+from gecko_taskgraph.util.perftest import is_external_browser
 from gecko_taskgraph.util.platforms import platform_family
+from gecko_taskgraph.util.taskcluster import get_index_url
 
 transforms = TransformSequence()
 
@@ -58,6 +62,14 @@ def handle_suite_category(config, tasks):
             category_arg = f"--{category}-suite"
 
         if category_arg:
+            resolve_keyed_by(
+                task,
+                "mozharness.extra-options",
+                item_name=task["test-name"],
+                enforce_single_match=False,
+                variant=task["attributes"].get("unittest_variant"),
+            )
+
             task["mozharness"].setdefault("extra-options", [])
             extra = task["mozharness"]["extra-options"]
             if not any(arg.startswith(category_arg) for arg in extra):
@@ -65,6 +77,14 @@ def handle_suite_category(config, tasks):
 
         # From here on out we only use the suite name.
         task["suite"] = suite
+
+        # in the future we might need to refactor new-test-config to be suite specific
+        if "mochitest" in task["suite"] and config.params["try_task_config"].get(
+            "new-test-config", False
+        ):
+            task = merge(
+                task, {"mozharness": {"extra-options": ["--restartAfterFailure"]}}
+            )
         yield task
 
 
@@ -76,19 +96,41 @@ def setup_talos(config, tasks):
             yield task
             continue
 
+        resolve_keyed_by(
+            task,
+            "mozharness.extra-options",
+            item_name=task["test-name"],
+            enforce_single_match=False,
+            variant=task["attributes"].get("unittest_variant"),
+        )
+
         extra_options = task.setdefault("mozharness", {}).setdefault(
             "extra-options", []
         )
         extra_options.append("--use-talos-json")
 
-        # win7 needs to test skip
-        if task["build-platform"].startswith("win32"):
-            extra_options.append("--add-option")
-            extra_options.append("--setpref,gfx.direct2d.disabled=true")
-
         if config.params.get("project", None):
             extra_options.append("--project=%s" % config.params["project"])
 
+        if "pdfpaint" in task["try-name"]:
+            max_chunks = 10
+            for chunk in range(1, max_chunks + 1):
+                new_task = copy.deepcopy(task)
+                new_task["mozharness"]["extra-options"].append(
+                    f"--pdfPaintChunk={chunk}"
+                )
+                new_task["test-name"] = task["test-name"].replace(
+                    "pdfpaint", f"pdfpaint-{chunk}"
+                )
+                new_task["try-name"] = task["try-name"].replace(
+                    "pdfpaint", f"pdfpaint-{chunk}"
+                )
+                new_task["treeherder-symbol"] = task["treeherder-symbol"].replace(
+                    "pdfpaint", f"pdfpaint-{chunk}"
+                )
+                yield new_task
+            continue
+
         yield task
 
 
@@ -135,12 +177,12 @@ def set_treeherder_machine_platform(config, tasks):
     translation = {
         # Linux64 build platform for asan is specified differently to
         # treeherder.
-        "macosx1100-64/opt": "osx-1100/opt",
-        "macosx1100-64-shippable/opt": "osx-1100-shippable/opt",
-        "macosx1300-64/opt": "osx-1300/opt",
-        "macosx1300-64-shippable/opt": "osx-1300-shippable/opt",
-        "win64-asan/opt": "windows10-64/asan",
-        "win64-aarch64/opt": "windows10-aarch64/opt",
+        "macosx1400-64/opt": "osx-1300/opt",
+        "macosx1400-64-shippable/opt": "osx-1400-shippable/opt",
+        "macosx1500-64/opt": "osx-1500/opt",
+        "macosx1500-64-shippable/opt": "osx-1500-shippable/opt",
+        "win64-asan/opt": "windows11-64-24h2/asan",
+        "win64-aarch64/opt": "windows11-aarch64/opt",
     }
     for task in tasks:
         # For most desktop platforms, the above table is not used for "regular"
@@ -153,34 +195,11 @@ def set_treeherder_machine_platform(config, tasks):
         if "android" in task["test-platform"] and "pgo/opt" in task["test-platform"]:
             platform_new = task["test-platform"].replace("-pgo/opt", "/pgo")
             task["treeherder-machine-platform"] = platform_new
-        elif "android-em-7.0-x86_64-qr" in task["test-platform"]:
-            task["treeherder-machine-platform"] = task["test-platform"].replace(
-                ".", "-"
-            )
-        elif "android-em-7.0-x86_64-shippable-qr" in task["test-platform"]:
-            task["treeherder-machine-platform"] = task["test-platform"].replace(
-                ".", "-"
-            )
-        elif "android-em-7.0-x86_64-lite-qr" in task["test-platform"]:
-            task["treeherder-machine-platform"] = task["test-platform"].replace(
-                ".", "-"
-            )
-        elif "android-em-7.0-x86_64-shippable-lite-qr" in task["test-platform"]:
-            task["treeherder-machine-platform"] = task["test-platform"].replace(
-                ".", "-"
-            )
-        elif "-qr" in task["test-platform"]:
+        elif "android-em-" in task["test-platform"]:
             task["treeherder-machine-platform"] = task["test-platform"]
         elif "android-hw" in task["test-platform"]:
             task["treeherder-machine-platform"] = task["test-platform"]
-        elif "android-em-7.0-x86_64" in task["test-platform"]:
-            task["treeherder-machine-platform"] = task["test-platform"].replace(
-                ".", "-"
-            )
-        elif "android-em-7.0-x86" in task["test-platform"]:
-            task["treeherder-machine-platform"] = task["test-platform"].replace(
-                ".", "-"
-            )
+
         # Bug 1602863 - must separately define linux64/asan and linux1804-64/asan
         # otherwise causes an exception during taskgraph generation about
         # duplicate treeherder platform/symbol.
@@ -188,6 +207,8 @@ def set_treeherder_machine_platform(config, tasks):
             task["treeherder-machine-platform"] = "linux64/asan"
         elif "linux1804-asan/opt" in task["test-platform"]:
             task["treeherder-machine-platform"] = "linux1804-64/asan"
+        elif "-qr" in task["test-platform"]:
+            task["treeherder-machine-platform"] = task["test-platform"]
         else:
             task["treeherder-machine-platform"] = translation.get(
                 task["build-platform"], task["test-platform"]
@@ -230,8 +251,10 @@ def handle_keyed_by(config, tasks):
         "fetches.toolchain",
         "target",
         "webrender-run-on-projects",
+        "mozharness.extra-options",
         "mozharness.requires-signed-builds",
         "build-signing-label",
+        "dependencies",
     ]
     for task in tasks:
         for field in fields:
@@ -246,6 +269,24 @@ def handle_keyed_by(config, tasks):
         yield task
 
 
+@transforms.add
+def setup_raptor_external_browser_platforms(config, tasks):
+    for task in tasks:
+        if task["suite"] != "raptor":
+            yield task
+            continue
+
+        if is_external_browser(task["try-name"]):
+            if "win" in task["build-label"]:
+                task["build-platform"] = "windows2012-64/opt"
+                task["build-label"] = "build-win64/opt"
+            else:
+                task["build-platform"] = "linux64/opt"
+                task["build-label"] = "build-linux64/opt"
+
+        yield task
+
+
 @transforms.add
 def set_target(config, tasks):
     for task in tasks:
@@ -261,13 +302,20 @@ def set_target(config, tasks):
             elif build_platform.startswith("win"):
                 target = "target.zip"
             else:
-                target = "target.tar.bz2"
+                target = "target.tar.xz"
 
         if isinstance(target, dict):
-            # TODO Remove hardcoded mobile artifact prefix
-            index_url = get_index_url(target["index"])
-            installer_url = "{}/artifacts/public/{}".format(index_url, target["name"])
-            task["mozharness"]["installer-url"] = installer_url
+            if "index" in target:
+                # TODO Remove hardcoded mobile artifact prefix
+                index_url = get_index_url(target["index"])
+                installer_url = "{}/artifacts/public/{}".format(
+                    index_url, target["name"]
+                )
+                task["mozharness"]["installer-url"] = installer_url
+            else:
+                task["mozharness"][
+                    "installer-url"
+                ] = f"<{target['upstream-task']}/{target['name']}>"
         else:
             task["mozharness"]["build-artifact-name"] = get_artifact_path(task, target)
 
@@ -295,16 +343,29 @@ def setup_browsertime(config, tasks):
 
         ts = {
             "by-test-platform": {
-                "android.*": ["browsertime", "linux64-geckodriver", "linux64-node-16"],
-                "linux.*": ["browsertime", "linux64-geckodriver", "linux64-node-16"],
-                "macosx.*": ["browsertime", "macosx64-geckodriver", "macosx64-node-16"],
+                "android.*": ["browsertime", "linux64-geckodriver", "linux64-node"],
+                "linux.*": ["browsertime", "linux64-geckodriver", "linux64-node"],
+                "macosx1470.*": [
+                    "browsertime",
+                    "macosx64-geckodriver",
+                    "macosx64-node",
+                ],
+                "macosx1400.*": [
+                    "browsertime",
+                    "macosx64-aarch64-geckodriver",
+                    "macosx64-aarch64-node",
+                ],
+                "macosx1500.*": [
+                    "browsertime",
+                    "macosx64-aarch64-geckodriver",
+                    "macosx64-aarch64-node",
+                ],
                 "windows.*aarch64.*": [
                     "browsertime",
                     "win32-geckodriver",
-                    "win32-node-16",
+                    "win32-node",
                 ],
-                "windows.*-32.*": ["browsertime", "win32-geckodriver", "win32-node-16"],
-                "windows.*-64.*": ["browsertime", "win64-geckodriver", "win64-node-16"],
+                "windows.*-64.*": ["browsertime", "win64-geckodriver", "win64-node"],
             },
         }
 
@@ -314,55 +375,56 @@ def setup_browsertime(config, tasks):
 
         fs = {
             "by-test-platform": {
-                "android.*": ["linux64-ffmpeg-4.4.1"],
-                "linux.*": ["linux64-ffmpeg-4.4.1"],
-                "macosx.*": ["mac64-ffmpeg-4.4.1"],
-                "windows.*aarch64.*": ["win64-ffmpeg-4.4.1"],
-                "windows.*-32.*": ["win64-ffmpeg-4.4.1"],
-                "windows.*-64.*": ["win64-ffmpeg-4.4.1"],
+                "android.*": ["linux64-ffmpeg-7.1"],
+                "linux.*": ["linux64-ffmpeg-7.1"],
+                "macosx1470.*": ["mac64-ffmpeg-7.1"],
+                "macosx1400.*": ["mac64-ffmpeg-7.1"],
+                "macosx1500.*": ["mac64-ffmpeg-7.1"],
+                "windows.*aarch64.*": ["win64-ffmpeg-7.1"],
+                "windows.*-64.*": ["win64-ffmpeg-7.1"],
             },
         }
 
         cd_fetches = {
             "android.*": [
-                "linux64-chromedriver-114",
-                "linux64-chromedriver-115",
-                "linux64-chromedriver-116",
+                "linux64-cft-cd-backup",
+                "linux64-cft-cd-stable",
+                "linux64-cft-cd-beta",
             ],
             "linux.*": [
-                "linux64-chromedriver-114",
-                "linux64-chromedriver-115",
-                "linux64-chromedriver-116",
+                "linux64-cft-cd-backup",
+                "linux64-cft-cd-stable",
+                "linux64-cft-cd-beta",
             ],
-            "macosx.*": [
-                "mac64-chromedriver-109",
-                "mac64-chromedriver-114",
-                "mac64-chromedriver-115",
-                "mac64-chromedriver-116",
+            "macosx1470.*": [
+                "mac-cft-cd-backup",
+                "mac-cft-cd-stable",
+                "mac-cft-cd-beta",
             ],
-            "windows.*aarch64.*": [
-                "win32-chromedriver-114",
-                "win32-chromedriver-115",
-                "win32-chromedriver-116",
+            "macosx1400.*": [
+                "mac-cft-cd-arm-backup",
+                "mac-cft-cd-arm-stable",
+                "mac-cft-cd-arm-beta",
             ],
-            "windows.*-32.*": [
-                "win32-chromedriver-114",
-                "win32-chromedriver-115",
-                "win32-chromedriver-116",
+            "macosx1500.*": [
+                "mac-cft-cd-arm-backup",
+                "mac-cft-cd-arm-stable",
+                "mac-cft-cd-arm-beta",
             ],
             "windows.*-64.*": [
-                "win32-chromedriver-114",
-                "win32-chromedriver-115",
-                "win32-chromedriver-116",
+                "win64-cft-cd-backup",
+                "win64-cft-cd-stable",
+                "win64-cft-cd-beta",
             ],
         }
 
         chromium_fetches = {
-            "linux.*": ["linux64-chromium"],
-            "macosx.*": ["mac-chromium"],
-            "windows.*aarch64.*": ["win32-chromium"],
-            "windows.*-32.*": ["win32-chromium"],
-            "windows.*-64.*": ["win64-chromium"],
+            "linux.*": ["linux64-cft-cd-canary"],
+            "macosx1400.*": ["mac-cft-cd-arm-canary"],
+            "macosx1470.*": ["mac-cft-cd-canary"],
+            "macosx1500.*": ["mac-cft-cd-arm-canary"],
+            "windows.*-64.*": ["win64-cft-cd-canary"],
+            "android.*": ["linux64-cft-cd-canary"],
         }
 
         cd_extracted_name = {
@@ -375,22 +437,22 @@ def setup_browsertime(config, tasks):
             # Only add the chromedriver fetches when chrome is running
             for platform in cd_fetches:
                 fs["by-test-platform"][platform].extend(cd_fetches[platform])
-        if "--app=chromium" in extra_options or "--app=custom-car" in extra_options:
+        if "--app=custom-car" in extra_options or "--app=cstm-car-m" in extra_options:
             for platform in chromium_fetches:
                 fs["by-test-platform"][platform].extend(chromium_fetches[platform])
 
-            # The chromedrivers for chromium are repackaged into the archives
-            # that we get the chromium binary from so we always have a compatible
-            # version.
+            # The Chrome-for-Testing chromedrivers are repackaged into the following
+            # platform specific archives. The versions will always be compatible as
+            # these are fetched from the `Canary` channel.
             cd_extracted_name = {
-                "windows": "chrome-win/chromedriver.exe",
-                "mac": "chrome-mac/chromedriver",
-                "default": "chrome-linux/chromedriver",
+                "windows": "cft-chromedriver-win64/chromedriver.exe",
+                "mac": "cft-chromedriver-mac/chromedriver",
+                "default": "cft-chromedriver-linux/chromedriver",
             }
 
         # Disable the Raptor install step
-        if "--app=chrome-m" in extra_options:
-            extra_options.append("--noinstall")
+        if "--app=chrome-m" in extra_options or "--app=cstm-car-m" in extra_options:
+            extra_options.append("--no-install")
 
         task.setdefault("fetches", {}).setdefault("fetch", []).extend(
             evaluate_keyed_by(fs, "fetches.fetch", task)
@@ -413,7 +475,7 @@ def setup_browsertime(config, tasks):
                     "--browsertime-chromedriver",
                     "$MOZ_FETCHES_DIR/" + cd_extracted_name["windows"],
                     "--browsertime-ffmpeg",
-                    "$MOZ_FETCHES_DIR/ffmpeg-4.4.1-full_build/bin/ffmpeg.exe",
+                    "$MOZ_FETCHES_DIR/ffmpeg-n7.1-latest-win64-gpl-shared-7.1/bin/ffmpeg.exe",
                 ],
                 "macosx.*": [
                     "--browsertime-node",
@@ -423,7 +485,7 @@ def setup_browsertime(config, tasks):
                     "--browsertime-chromedriver",
                     "$MOZ_FETCHES_DIR/" + cd_extracted_name["mac"],
                     "--browsertime-ffmpeg",
-                    "$MOZ_FETCHES_DIR/ffmpeg-macos/ffmpeg",
+                    "$MOZ_FETCHES_DIR/ffmpeg-7.1/bin/ffmpeg",
                 ],
                 "default": [
                     "--browsertime-node",
@@ -433,7 +495,7 @@ def setup_browsertime(config, tasks):
                     "--browsertime-chromedriver",
                     "$MOZ_FETCHES_DIR/" + cd_extracted_name["default"],
                     "--browsertime-ffmpeg",
-                    "$MOZ_FETCHES_DIR/ffmpeg-4.4.1-i686-static/ffmpeg",
+                    "$MOZ_FETCHES_DIR/ffmpeg-n7.1-linux64-gpl-7.1/bin/ffmpeg",
                 ],
             }
         }
@@ -449,7 +511,7 @@ def get_mobile_project(task):
     if not task["build-platform"].startswith("android"):
         return
 
-    mobile_projects = ("fenix", "geckoview", "refbrow", "chrome-m")
+    mobile_projects = ("fenix", "geckoview", "refbrow", "chrome-m", "cstm-car-m")
 
     for name in mobile_projects:
         if name in task["test-name"]:
@@ -480,6 +542,14 @@ def disable_wpt_timeouts_on_autoland(config, tasks):
             "web-platform-tests" in task["test-name"]
             and config.params["project"] == "autoland"
         ):
+            resolve_keyed_by(
+                task,
+                "mozharness.extra-options",
+                item_name=task["test-name"],
+                enforce_single_match=False,
+                variant=task["attributes"].get("unittest_variant"),
+            )
+
             task["mozharness"].setdefault("extra-options", []).append("--skip-timeout")
         yield task
 
@@ -505,7 +575,9 @@ def enable_code_coverage(config, tasks):
                 yield task
                 continue
             task["mozharness"].setdefault("extra-options", []).append("--code-coverage")
-            task["instance-size"] = "xlarge"
+            task["instance-size"] = "xlarge-noscratch"
+            if "jittest" in task["test-name"]:
+                task["instance-size"] = "xlarge"
 
             # Temporarily disable Mac tests on mozilla-central
             if "mac" in task["build-platform"]:
@@ -615,35 +687,25 @@ def handle_tier(config, tasks):
                 "linux1804-64-shippable-qr/opt",
                 "linux1804-64-asan-qr/opt",
                 "linux1804-64-tsan-qr/opt",
-                "windows10-32-qr/debug",
-                "windows10-32-qr/opt",
-                "windows10-32-shippable-qr/opt",
-                "windows10-32-2004-qr/debug",
-                "windows10-32-2004-qr/opt",
-                "windows10-32-2004-shippable-qr/opt",
-                "windows10-aarch64-qr/opt",
-                "windows10-64/debug",
-                "windows10-64/opt",
-                "windows10-64-shippable/opt",
-                "windows10-64-devedition/opt",
-                "windows10-64-qr/opt",
-                "windows10-64-qr/debug",
-                "windows10-64-shippable-qr/opt",
-                "windows10-64-devedition-qr/opt",
-                "windows10-64-asan-qr/opt",
-                "windows10-64-2004-qr/opt",
-                "windows10-64-2004-qr/debug",
-                "windows10-64-2004-shippable-qr/opt",
-                "windows10-64-2004-devedition-qr/opt",
-                "windows10-64-2004-asan-qr/opt",
-                "windows11-32-2009-qr/debug",
-                "windows11-32-2009-qr/opt",
-                "windows11-32-2009-shippable-qr/opt",
-                "windows11-64-2009-qr/opt",
-                "windows11-64-2009-qr/debug",
-                "windows11-64-2009-shippable-qr/opt",
-                "windows11-64-2009-devedition-qr/opt",
-                "windows11-64-2009-asan-qr/opt",
+                "linux2204-64-wayland/debug",
+                "linux2204-64-wayland/opt",
+                "linux2204-64-wayland-shippable/opt",
+                "linux2404-64/opt",
+                "linux2404-64/debug",
+                "linux2404-64-shippable/opt",
+                "linux2404-64-devedition/opt",
+                "linux2404-64-asan/opt",
+                "linux2404-64-tsan/opt",
+                "windows11-32-24h2/debug",
+                "windows11-32-24h2/opt",
+                "windows11-32-24h2-shippable/opt",
+                "windows11-64-24h2-hw-ref/opt",
+                "windows11-64-24h2-hw-ref-shippable/opt",
+                "windows11-64-24h2/opt",
+                "windows11-64-24h2/debug",
+                "windows11-64-24h2-shippable/opt",
+                "windows11-64-24h2-devedition/opt",
+                "windows11-64-24h2-asan/opt",
                 "macosx1015-64/opt",
                 "macosx1015-64/debug",
                 "macosx1015-64-shippable/opt",
@@ -652,25 +714,22 @@ def handle_tier(config, tasks):
                 "macosx1015-64-qr/opt",
                 "macosx1015-64-shippable-qr/opt",
                 "macosx1015-64-qr/debug",
-                "macosx1100-64-shippable-qr/opt",
-                "macosx1100-64-qr/debug",
-                "macosx1300-64-shippable-qr/opt",
-                "macosx1300-64-qr/debug",
-                "android-em-7.0-x86_64-shippable/opt",
-                "android-em-7.0-x86_64-shippable-lite/opt",
-                "android-em-7.0-x86_64/debug",
-                "android-em-7.0-x86_64/debug-isolated-process",
-                "android-em-7.0-x86_64/opt",
-                "android-em-7.0-x86_64-lite/opt",
-                "android-em-7.0-x86-shippable/opt",
-                "android-em-7.0-x86-shippable-lite/opt",
-                "android-em-7.0-x86_64-shippable-qr/opt",
-                "android-em-7.0-x86_64-qr/debug",
-                "android-em-7.0-x86_64-qr/debug-isolated-process",
-                "android-em-7.0-x86_64-qr/opt",
-                "android-em-7.0-x86_64-shippable-lite-qr/opt",
-                "android-em-7.0-x86_64-lite-qr/debug",
-                "android-em-7.0-x86_64-lite-qr/opt",
+                "macosx1470-64/opt",
+                "macosx1470-64/debug",
+                "macosx1470-64-shippable/opt",
+                "macosx1470-64-devedition/opt",
+                "macosx1400-64-shippable-qr/opt",
+                "macosx1400-64-qr/debug",
+                "macosx1500-64-shippable/opt",
+                "macosx1500-64/debug",
+                "android-em-14-x86_64-shippable/opt",
+                "android-em-14-x86_64/opt",
+                "android-em-14-x86_64-shippable-lite/opt",
+                "android-em-14-x86_64-lite/opt",
+                "android-em-14-x86_64/debug",
+                "android-em-14-x86_64/debug-isolated-process",
+                "android-em-14-x86-shippable/opt",
+                "android-em-14-x86/opt",
             ]:
                 task["tier"] = 1
             else:
@@ -714,8 +773,16 @@ def disable_try_only_platforms(config, tasks):
 @transforms.add
 def ensure_spi_disabled_on_all_but_spi(config, tasks):
     for task in tasks:
-        variant = task["attributes"].get("unittest_variant", "")
-        has_no_setpref = ("gtest", "cppunit", "jittest", "junit", "raptor")
+        variant = task["attributes"].get("unittest_variant") or ""
+        has_no_setpref = (
+            "gtest",
+            "cppunit",
+            "jittest",
+            "junit",
+            "raptor",
+            "reftest",
+            "web-platform-tests",
+        )
 
         if (
             all(s not in task["suite"] for s in has_no_setpref)
@@ -735,7 +802,7 @@ test_setting_description_schema = Schema(
     {
         Required("_hash"): str,
         "platform": {
-            Required("arch"): Any("32", "64", "aarch64", "arm7", "x86_64"),
+            Required("arch"): Any("32", "64", "aarch64", "arm7", "x86", "x86_64"),
             Required("os"): {
                 Required("name"): Any("android", "linux", "macosx", "windows"),
                 Required("version"): str,
@@ -743,7 +810,7 @@ test_setting_description_schema = Schema(
             },
             Optional("device"): str,
             Optional("display"): "wayland",
-            Optional("machine"): Any("ref-hw-2017"),
+            Optional("machine"): "hw-ref",
         },
         "build": {
             Required("type"): Any("opt", "debug", "debug-isolated-process"),
@@ -752,7 +819,6 @@ test_setting_description_schema = Schema(
                 "ccov",
                 "clang-trunk",
                 "devedition",
-                "domstreams",
                 "lite",
                 "mingwclang",
                 "nightlyasrelease",
@@ -804,7 +870,7 @@ def set_test_setting(config, tasks):
     # TODO Rename these so they don't have a dash.
     dash_attrs = [
         "clang-trunk",
-        "ref-hw-2017",
+        "hw-ref",
     ]
     dash_token = "%D%"
     platform_re = re.compile(r"(\D+)(\d*)")
@@ -856,15 +922,18 @@ def set_test_setting(config, tasks):
 
         else:
             arch = parts.pop(0)
-            if parts[0].isdigit():
+            if parts and (parts[0].isdigit() or parts[0] in ["24h2"]):
                 os_build = parts.pop(0)
 
-            if parts[0] == "ref-hw-2017":
+            if parts and parts[0] == "hw-ref":
                 machine = parts.pop(0)
 
-            if parts[0] == "wayland":
+            if parts and parts[0] == "wayland":
                 display = parts.pop(0)
 
+            if parts and parts[0] == "aarch64":
+                arch = parts.pop(0)
+
         # It's not always possible to glean the exact architecture used from
         # the task, so sometimes this will just be set to "32" or "64".
         setting["platform"]["arch"] = arch
@@ -963,8 +1032,7 @@ def set_retry_exit_status(config, tasks):
     """Set the retry exit status to TBPL_RETRY, the value returned by mozharness
     scripts to indicate a transient failure that should be retried."""
     for task in tasks:
-        # add in 137 as it is an error with GCP workers
-        task["retry-exit-status"] = [4, 137]
+        task["retry-exit-status"] = [4]
         yield task
 
 
@@ -1002,12 +1070,58 @@ def set_profile(config, tasks):
         yield task
 
 
+@transforms.add
+def add_gecko_profile_symbolication_deps(config, tasks):
+    """Add symbolication dependencies when profiling raptor, talos, or mochitest tests"""
+
+    try_task_config = config.params.get("try_task_config", {})
+    gecko_profile = try_task_config.get("gecko-profile", False)
+    env = try_task_config.get("env", {})
+    startup_profile = env.get("MOZ_PROFILER_STARTUP") == "1"
+
+    for task in tasks:
+
+        if (gecko_profile and task["suite"] in ["talos", "raptor"]) or (
+            startup_profile and "mochitest" in task["suite"]
+        ):
+
+            fetches = task.setdefault("fetches", {})
+            fetch_toolchains = fetches.setdefault("toolchain", [])
+            fetch_toolchains.append("symbolicator-cli")
+
+            test_platform = task["test-platform"]
+
+            if "macosx" in test_platform and "aarch64" in test_platform:
+                fetch_toolchains.append("macosx64-aarch64-samply")
+            elif "macosx" in test_platform:
+                fetch_toolchains.append("macosx64-samply")
+            elif "win" in test_platform:
+                fetch_toolchains.append("win64-samply")
+            else:
+                fetch_toolchains.append("linux64-samply")
+
+            # Add node as a dependency for talos and mochitest tasks if needed.
+            # node is used to run symbolicator-cli, our profile symbolication tool
+            if task["suite"] == "talos" or "mochitest" in task["suite"]:
+                if "macosx" in test_platform and "aarch64" in test_platform:
+                    node_toolchain = "macosx64-aarch64-node"
+                elif "macosx" in test_platform:
+                    node_toolchain = "macosx64-node"
+                elif "win" in test_platform:
+                    node_toolchain = "win64-node"
+                else:
+                    node_toolchain = "linux64-node"
+
+                if node_toolchain not in fetch_toolchains:
+                    fetch_toolchains.append(node_toolchain)
+
+        yield task
+
+
 @transforms.add
 def set_tag(config, tasks):
     """Set test for a specific tag."""
     tag = None
-    if config.params["try_mode"] == "try_option_syntax":
-        tag = config.params["try_options"]["tag"]
     for task in tasks:
         if tag:
             task["mozharness"]["extra-options"].extend(["--tag", tag])
@@ -1047,6 +1161,7 @@ def set_schedules_components(config, tasks):
 
         schedules.add(category)
         schedules.add(platform_family(task["build-platform"]))
+        schedules.add("firefox")
 
         task["schedules-component"] = sorted(schedules)
         yield task
@@ -1065,3 +1180,20 @@ def enable_parallel_marking_in_tsan_tests(config, tasks):
                 )
 
         yield task
+
+
+@transforms.add
+def set_webgpu_ignore_blocklist(config, tasks):
+    """
+    Ignore the WebGPU blocklist on Linux because CI's Mesa is old
+
+    See <https://bugzilla.mozilla.org/show_bug.cgi?id=1985348>
+    """
+    for task in tasks:
+        if "web-platform-tests-webgpu" in task["test-name"] and task[
+            "test-platform"
+        ].startswith("linux"):
+            extra_options = task["mozharness"].setdefault("extra-options", [])
+            extra_options.append("--setpref=gfx.webgpu.ignore-blocklist=true")
+
+        yield task
