@@ -25,7 +25,6 @@
 #include "nsCategoryManagerUtils.h"
 #include "mozilla/dom/GeolocationPosition.h"
 
-#include "mozilla/ArrayUtils.h"
 #include "mozilla/AppShutdown.h"
 #include "mozilla/ClearOnShutdown.h"
 #include "mozilla/Components.h"
@@ -36,6 +35,7 @@
 #include "mozilla/dom/BrowserChild.h"
 #include "mozilla/dom/Document.h"
 #include "mozilla/gfx/gfxVars.h"
+#include "mozilla/gfx/Logging.h"
 #include "mozilla/intl/OSPreferences.h"
 #include "mozilla/ipc/GeckoChildProcessHost.h"
 #include "mozilla/java/GeckoAppShellNatives.h"
@@ -78,6 +78,7 @@
 #include "ScreenHelperAndroid.h"
 #include "WebExecutorSupport.h"
 #include "Base64UtilsSupport.h"
+#include "MozLogSupport.h"
 
 #ifdef DEBUG_ANDROID_EVENTS
 #  define EVLOG(args...) ALOG(args)
@@ -111,7 +112,7 @@ class WakeLockListener final : public nsIDOMMozWakeLockListener {
 };
 
 NS_IMPL_ISUPPORTS(WakeLockListener, nsIDOMMozWakeLockListener)
-MOZ_RUNINIT nsCOMPtr<nsIPowerManagerService> sPowerManagerService = nullptr;
+MOZ_CONSTINIT nsCOMPtr<nsIPowerManagerService> sPowerManagerService{};
 StaticRefPtr<WakeLockListener> sWakeLockListener;
 
 class GeckoThreadSupport final
@@ -362,6 +363,11 @@ class GeckoAppShellSupport final
 
     nsBaseAppShell::OnSystemTimezoneChange();
   }
+
+  static void LogGpuProcessLaunchFailure(jni::String::Param aMessage) {
+    gfxCriticalNote << "Error launching GPU process: "
+                    << aMessage->ToCString().get();
+  }
 };
 
 class XPCOMEventTargetWrapper final
@@ -428,6 +434,7 @@ nsAppShell::nsAppShell()
       GeckoThreadSupport::Init();
       GeckoAppShellSupport::Init();
       XPCOMEventTargetWrapper::Init();
+      mozilla::widget::MozLogSupport::Init();
 
       if (XRE_IsGPUProcess()) {
         mozilla::gl::AndroidSurfaceTexture::Init();
@@ -457,6 +464,7 @@ nsAppShell::nsAppShell()
     mozilla::widget::Base64UtilsSupport::Init();
     nsWindow::InitNatives();
     mozilla::gl::AndroidSurfaceTexture::Init();
+    mozilla::widget::MozLogSupport::Init();
 
     java::GeckoThread::SetState(java::GeckoThread::State::JNI_READY());
 
