@@ -6,8 +6,6 @@
 #include <OpenGL/OpenGL.h>
 #include <OpenGL/CGLRenderers.h>
 
-#include "mozilla/ArrayUtils.h"
-
 #include "GfxInfo.h"
 #include "nsUnicharUtils.h"
 #include "nsExceptionHandler.h"
@@ -16,8 +14,6 @@
 #include "mozilla/Preferences.h"
 #include "js/PropertyAndElement.h"  // JS_SetElement, JS_SetProperty
 
-#include <algorithm>
-
 #import <Foundation/Foundation.h>
 #import <IOKit/IOKitLib.h>
 #import <Cocoa/Cocoa.h>
@@ -176,9 +172,28 @@ void GfxInfo::GetDeviceInfo() {
   }
 #endif
 
-  CFMutableDictionaryRef apv_dev_dict = IOServiceMatching("AppleParavirtGPU");
-  if (IOServiceGetMatchingServices(kIOMasterPortDefault, apv_dev_dict,
-                                   &io_iter) == kIOReturnSuccess) {
+  // "AppleParavirtGPU" is the class name in VMs that use the Apple
+  // Virtualization framework. But it's "AppleParavirtGPUControl" in VMware
+  // VMs (on Intel) that use VMware's "paravirtualized driver". Parallels
+  // Intel VMs have a "AppleParavirtGPUControl" service. But, like VMware
+  // VMs that don't use the "paravirtualized driver", they also have
+  // kClassCodeDisplayVGA devices. So these cases will have been dealt with
+  // above. On Apple Silicon, Parallels uses Apple's Virtualization
+  // framework. VMware doesn't support macOS guest VMs on Apple Silicon.
+  for (const char* className :
+       {"AppleParavirtGPU", "AppleParavirtGPUControl"}) {
+    CFMutableDictionaryRef apv_dev_dict = IOServiceMatching(className);
+    if (IOServiceGetMatchingServices(kIOMasterPortDefault, apv_dev_dict,
+                                     &io_iter) == kIOReturnSuccess) {
+      if (IOIteratorNext(io_iter) != IO_OBJECT_NULL) {
+        IOIteratorReset(io_iter);
+        break;
+      }
+      IOObjectRelease(io_iter);
+    }
+    io_iter = IO_OBJECT_NULL;
+  }
+  if (io_iter) {
     io_registry_entry_t entry = IO_OBJECT_NULL;
     while ((entry = IOIteratorNext(io_iter)) != IO_OBJECT_NULL) {
       CFTypeRef vendor_id_ref =
@@ -226,9 +241,6 @@ nsresult GfxInfo::Init() {
   return rv;
 }
 
-NS_IMETHODIMP
-GfxInfo::GetD2DEnabled(bool* aEnabled) { return NS_ERROR_FAILURE; }
-
 NS_IMETHODIMP
 GfxInfo::GetDWriteEnabled(bool* aEnabled) { return NS_ERROR_FAILURE; }
 
@@ -468,12 +480,6 @@ const nsTArray<RefPtr<GfxDriverInfo>>& GfxInfo::GetGfxDriverInfo() {
         OperatingSystem::OSX, DeviceFamily::IntelWebRenderBlocked,
         nsIGfxInfo::FEATURE_WEBRENDER, nsIGfxInfo::FEATURE_BLOCKED_DEVICE,
         "FEATURE_FAILURE_INTEL_GEN5_OR_OLDER");
-
-    // Intel HD3000 disabled due to bug 1661505
-    IMPLEMENT_MAC_DRIVER_BLOCKLIST(
-        OperatingSystem::OSX, DeviceFamily::IntelSandyBridge,
-        nsIGfxInfo::FEATURE_WEBRENDER, nsIGfxInfo::FEATURE_BLOCKED_DEVICE,
-        "FEATURE_FAILURE_INTEL_MAC_HD3000_NO_WEBRENDER");
   }
   return *sDriverInfo;
 }
