@@ -15,8 +15,6 @@
 #import <UIKit/UIWindow.h>
 #import <QuartzCore/QuartzCore.h>
 
-#include <algorithm>
-
 #include "nsWindow.h"
 #include "ScreenHelperUIKit.h"
 #include "nsAppShell.h"
@@ -46,7 +44,6 @@
 #include "mozilla/EventForwards.h"
 #include "mozilla/ProfilerLabels.h"
 #include "mozilla/TouchEvents.h"
-#include "mozilla/Unused.h"
 #include "mozilla/dom/MouseEventBinding.h"
 #include "mozilla/gfx/Logging.h"
 #include "mozilla/widget/GeckoViewSupport.h"
@@ -250,8 +247,7 @@ class nsAutoRetainUIKitObject {
   event.mButton = MouseButton::ePrimary;
   event.mInputSource = mozilla::dom::MouseEvent_Binding::MOZ_SOURCE_UNKNOWN;
 
-  nsEventStatus status;
-  aWindow->DispatchEvent(&event, status);
+  aWindow->DispatchEvent(&event);
 }
 
 - (void)handleTap:(UITapGestureRecognizer*)sender {
@@ -346,10 +342,10 @@ class nsAutoRetainUIKitObject {
   }
 
   CGFloat scaleFactor = [self contentScaleFactor];
-  mGeckoChild->Resize(self.frame.origin.x * scaleFactor,
-                      self.frame.origin.y * scaleFactor,
-                      self.frame.size.width * scaleFactor,
-                      self.frame.size.height * scaleFactor, false);
+  mGeckoChild->DoResize(self.frame.origin.x * scaleFactor,
+                        self.frame.origin.y * scaleFactor,
+                        self.frame.size.width * scaleFactor,
+                        self.frame.size.height * scaleFactor, false);
 }
 
 - (BOOL)canBecomeFirstResponder {
@@ -416,11 +412,7 @@ class nsAutoRetainUIKitObject {
   if (!mGeckoChild->IsVisible()) return;
 
   mWaitingForPaint = NO;
-
-  LayoutDeviceIntRect geckoBounds = mGeckoChild->GetBounds();
-  LayoutDeviceIntRegion region(geckoBounds);
-
-  mGeckoChild->PaintWindow(region);
+  mGeckoChild->PaintWindow();
 }
 
 // Called asynchronously after setNeedsDisplay in order to avoid entering the
@@ -745,7 +737,7 @@ class nsAutoRetainUIKitObject {
 
 @end
 
-NS_IMPL_ISUPPORTS_INHERITED(nsWindow, nsBaseWidget, nsWindow);
+NS_IMPL_ISUPPORTS_INHERITED(nsWindow, nsIWidget, nsWindow);
 
 nsWindow::nsWindow()
     : mNativeView(nullptr),
@@ -778,7 +770,7 @@ bool nsWindow::IsTopLevel() {
 //
 
 nsresult nsWindow::Create(nsIWidget* aParent, const LayoutDeviceIntRect& aRect,
-                          widget::InitData* aInitData) {
+                          const widget::InitData& aInitData) {
   ALOG("nsWindow[%p]::Create %p [%d %d %d %d]", (void*)this, (void*)aParent,
        aRect.x, aRect.y, aRect.width, aRect.height);
   nsWindow* parent = (nsWindow*)aParent;
@@ -792,7 +784,7 @@ nsresult nsWindow::Create(nsIWidget* aParent, const LayoutDeviceIntRect& aRect,
   mWindowType = WindowType::TopLevel;
   mBorderStyle = BorderStyle::Default;
 
-  nsBaseWidget::BaseCreate(aParent, aInitData);
+  nsIWidget::BaseCreate(aParent, aInitData);
 
   NS_ASSERTION(IsTopLevel() || parent,
                "non top level window doesn't have a parent!");
@@ -839,13 +831,13 @@ void nsWindow::Destroy() {
 
   nsCOMPtr<nsIWidget> kungFuDeathGrip(this);
 
-  nsBaseWidget::Destroy();
+  nsIWidget::Destroy();
 
   // ReportDestroyEvent();
 
   TearDownView();
 
-  nsBaseWidget::OnDestroy();
+  nsIWidget::OnDestroy();
 }
 
 void nsWindow::Show(bool aState) {
@@ -861,14 +853,16 @@ void nsWindow::Show(bool aState) {
   }
 }
 
-void nsWindow::Move(double aX, double aY) {
-  if (!mNativeView || (mBounds.x == aX && mBounds.y == aY)) return;
+void nsWindow::Move(const DesktopPoint& aPoint) {
+  if (!mNativeView || (mBounds.x == aPoint.x && mBounds.y == aPoint.y)) {
+    return;
+  }
 
   // XXX: handle this
   // The point we have is in Gecko coordinates (origin top-left). Convert
   // it to Cocoa ones (origin bottom-left).
-  mBounds.x = aX;
-  mBounds.y = aY;
+  mBounds.x = aPoint.x;
+  mBounds.y = aPoint.y;
 
   if (mWindowType != WindowType::TopLevel) {
     mNativeView.frame = DevPixelsToUIKitPoints(mBounds, BackingScaleFactor());
@@ -879,11 +873,19 @@ void nsWindow::Move(double aX, double aY) {
   ReportMoveEvent();
 }
 
-void nsWindow::Resize(double aX, double aY, double aWidth, double aHeight,
-                      bool aRepaint) {
-  BOOL isMoving = (mBounds.x != aX || mBounds.y != aY);
-  BOOL isResizing = (mBounds.width != aWidth || mBounds.height != aHeight);
-  if (!mNativeView || (!isMoving && !isResizing)) return;
+void nsWindow::Resize(const DesktopRect& aRect, bool aRepaint) {
+  DoResize(aRect.x, aRect.y, aRect.width, aRect.height, aRepaint);
+}
+
+void nsWindow::DoResize(double aX, double aY, double aWidth, double aHeight,
+                        bool aRepaint) {
+  // FIXME: This code is confused about integers vs. double coords, and desktop
+  // vs. device pixels.
+  BOOL isMoving = mBounds.x != aX || mBounds.y != aY;
+  BOOL isResizing = mBounds.width != aWidth || mBounds.height != aHeight;
+  if (!mNativeView || (!isMoving && !isResizing)) {
+    return;
+  }
 
   if (isMoving) {
     mBounds.x = aX;
@@ -906,12 +908,14 @@ void nsWindow::Resize(double aX, double aY, double aWidth, double aHeight,
   if (isResizing) ReportSizeEvent();
 }
 
-void nsWindow::Resize(double aWidth, double aHeight, bool aRepaint) {
-  if (!mNativeView || (mBounds.width == aWidth && mBounds.height == aHeight))
+void nsWindow::Resize(const DesktopSize& aSize, bool aRepaint) {
+  if (!mNativeView ||
+      (mBounds.width == aSize.width && mBounds.height == aSize.height)) {
     return;
+  }
 
-  mBounds.width = aWidth;
-  mBounds.height = aHeight;
+  mBounds.width = aSize.width;
+  mBounds.height = aSize.height;
 
   if (mWindowType != WindowType::TopLevel) {
     [mNativeView
@@ -932,7 +936,7 @@ void nsWindow::SetSizeMode(nsSizeMode aMode) {
   mSizeMode = static_cast<nsSizeMode>(aMode);
   if (aMode == nsSizeMode_Maximized || aMode == nsSizeMode_Fullscreen) {
     // Resize to fill screen
-    nsBaseWidget::InfallibleMakeFullScreen(true);
+    nsIWidget::InfallibleMakeFullScreen(true);
   }
   ReportSizeModeEvent(aMode);
 }
@@ -950,25 +954,12 @@ void nsWindow::SetFocus(Raise, mozilla::dom::CallerType) {
   [mNativeView becomeFirstResponder];
 }
 
-void nsWindow::WillPaintWindow() {
+void nsWindow::PaintWindow() {
   if (mWidgetListener) {
-    mWidgetListener->WillPaintWindow(this);
+    mWidgetListener->PaintWindow(this);
   }
 }
 
-bool nsWindow::PaintWindow(LayoutDeviceIntRegion aRegion) {
-  if (!mWidgetListener) return false;
-
-  bool returnValue = false;
-  returnValue = mWidgetListener->PaintWindow(this, aRegion);
-
-  if (mWidgetListener) {
-    mWidgetListener->DidPaintWindow();
-  }
-
-  return returnValue;
-}
-
 void nsWindow::ReportMoveEvent() { NotifyWindowMoved(mBounds.x, mBounds.y); }
 
 void nsWindow::ReportSizeModeEvent(nsSizeMode aMode) {
@@ -1025,21 +1016,6 @@ LayoutDeviceIntPoint nsWindow::WidgetToScreenOffset() {
   return offset;
 }
 
-nsresult nsWindow::DispatchEvent(mozilla::WidgetGUIEvent* aEvent,
-                                 nsEventStatus& aStatus) {
-  aStatus = nsEventStatus_eIgnore;
-  nsCOMPtr<nsIWidget> kungFuDeathGrip(aEvent->mWidget);
-  mozilla::Unused << kungFuDeathGrip;  // Not used within this function
-
-  if (mAttachedWidgetListener) {
-    aStatus = mAttachedWidgetListener->HandleEvent(aEvent, mUseAttachedEvents);
-  } else if (mWidgetListener) {
-    aStatus = mWidgetListener->HandleEvent(aEvent, mUseAttachedEvents);
-  }
-
-  return NS_OK;
-}
-
 void nsWindow::SetInputContext(const InputContext& aContext,
                                const InputContextAction& aAction) {
   NS_OBJC_BEGIN_TRY_IGNORE_BLOCK;
@@ -1143,18 +1119,16 @@ int32_t nsWindow::RoundsWidgetCoordinatesTo() {
   return 1;
 }
 
-RefPtr<layers::NativeLayerRoot> nsWindow::GetNativeLayerRoot() {
+layers::NativeLayerRoot* nsWindow::GetNativeLayerRoot() {
   return mNativeLayerRoot;
 }
 
 void nsWindow::HandleMainThreadCATransaction() {
-  WillPaintWindow();
-
   // Trigger a synchronous OMTC composite. This will call NextSurface and
   // NotifySurfaceReady on the compositor thread to update mNativeLayerRoot's
   // contents, and the main thread (this thread) will wait inside PaintWindow
   // during that time.
-  PaintWindow(LayoutDeviceIntRegion(GetBounds()));
+  PaintWindow();
 
   {
     // Apply the changes inside mNativeLayerRoot to the underlying CALayers. Now
